@@ -56,9 +56,11 @@
 
 #define PAS_ID_PREFIX "pas-id-"
 
-#define SQLITEDB_EMAIL_ID    "addressbook@localbackend.com"
+/* We need to specify the folderid in order to properly
+ * migrate data from the old EBookSqliteDB API.
+ */
 #define SQLITEDB_FOLDER_ID   "folder_id"
-#define SQLITEDB_FOLDER_NAME "folder"
+#define SQLITE_REVISION_KEY  "revision"
 
 /* Forward Declarations */
 static void	e_book_backend_file_initable_init
@@ -82,7 +84,7 @@ struct _EBookBackendFilePrivate {
 	GRWLock    lock;
 	GList     *cursors;
 
-	EBookBackendSqliteDB *sqlitedb;
+	EBookSqlite *sqlitedb;
 };
 
 /****************************************************************
@@ -225,7 +227,7 @@ maybe_delete_unused_uris (EBookBackendFile *bf,
 	 * old uris for those fields from old_contact to delete
 	 */
 	uri_photo = check_remove_uri_for_field (old_contact, new_contact, E_CONTACT_PHOTO);
-	uri_logo  = check_remove_uri_for_field (old_contact, new_contact, E_CONTACT_LOGO);
+	uri_logo = check_remove_uri_for_field (old_contact, new_contact, E_CONTACT_LOGO);
 
 	if (uri_photo) {
 		maybe_delete_uri (bf, uri_photo);
@@ -331,7 +333,7 @@ safe_name_for_photo (EBookBackendFile *bf,
 	do {
 		g_free (fullname);
 
-		str      = e_filename_mkdir_encoded (bf->priv->photo_dirname, name, NULL, i);
+		str = e_filename_mkdir_encoded (bf->priv->photo_dirname, name, NULL, i);
 		fullname = g_strdup_printf ("%s.%s", str, suffix);
 		g_free (str);
 
@@ -372,7 +374,7 @@ hard_link_photo (EBookBackendFile *bf,
 	do {
 		g_free (fullname);
 
-		str      = e_filename_mkdir_encoded (bf->priv->photo_dirname, name, NULL, i);
+		str = e_filename_mkdir_encoded (bf->priv->photo_dirname, name, NULL, i);
 		fullname = g_strdup_printf ("%s.%s", str, suffix);
 		g_free (str);
 
@@ -465,8 +467,8 @@ maybe_transform_vcard_field_for_photo (EBookBackendFile *bf,
 
 			status = STATUS_ERROR;
 		} else {
-			new_photo           = e_contact_photo_new ();
-			new_photo->type     = E_CONTACT_PHOTO_TYPE_URI;
+			new_photo = e_contact_photo_new ();
+			new_photo->type = E_CONTACT_PHOTO_TYPE_URI;
 			new_photo->data.uri = g_strdup (uri);
 
 			e_contact_set (contact, field, new_photo);
@@ -543,8 +545,8 @@ maybe_transform_vcard_field_for_photo (EBookBackendFile *bf,
 				status = STATUS_ERROR;
 			} else {
 
-				new_photo           = e_contact_photo_new ();
-				new_photo->type     = E_CONTACT_PHOTO_TYPE_URI;
+				new_photo = e_contact_photo_new ();
+				new_photo->type = E_CONTACT_PHOTO_TYPE_URI;
 				new_photo->data.uri = new_uri;
 
 				e_contact_set (contact, field, new_photo);
@@ -640,18 +642,18 @@ e_book_backend_file_new_revision (EBookBackendFile *bf)
  */
 static gboolean
 e_book_backend_file_bump_revision (EBookBackendFile *bf,
-				   GError **error)
+                                   GError **error)
 {
 	GError *local_error = NULL;
 	gchar *new_revision;
 	gboolean success;
 
 	new_revision = e_book_backend_file_new_revision (bf);
-
-	success = e_book_backend_sqlitedb_set_revision (bf->priv->sqlitedb,
-							SQLITEDB_FOLDER_ID,
-							new_revision,
-							&local_error);
+	success = e_book_sqlite_set_key_value (
+		bf->priv->sqlitedb,
+		SQLITE_REVISION_KEY,
+		new_revision,
+		&local_error);
 
 	if (success) {
 		g_free (bf->priv->revision);
@@ -676,10 +678,10 @@ e_book_backend_file_load_revision (EBookBackendFile *bf)
 {
 	GError *error = NULL;
 
-	if (!e_book_backend_sqlitedb_get_revision (bf->priv->sqlitedb,
-						   SQLITEDB_FOLDER_ID,
-						   &bf->priv->revision,
-						   &error)) {
+	if (!e_book_sqlite_get_key_value (bf->priv->sqlitedb,
+					  SQLITE_REVISION_KEY,
+					  &bf->priv->revision,
+					  &error)) {
 		g_warning (
 			G_STRLOC ": Error loading database revision: %s",
 			error ? error->message : "Unknown error");
@@ -694,10 +696,9 @@ e_book_backend_file_load_locale (EBookBackendFile *bf)
 {
 	GError *error = NULL;
 
-	if (!e_book_backend_sqlitedb_get_locale (bf->priv->sqlitedb,
-						 SQLITEDB_FOLDER_ID,
-						 &bf->priv->locale,
-						 &error)) {
+	if (!e_book_sqlite_get_locale (bf->priv->sqlitedb,
+				       &bf->priv->locale,
+				       &error)) {
 		g_warning (
 			G_STRLOC ": Error loading database locale setting: %s",
 			error ? error->message : "Unknown error");
@@ -716,29 +717,12 @@ set_revision (EBookBackendFile *bf,
 	g_free (rev);
 }
 
-
 /****************************************************************
  *                   Dealing with cursor updates                *
  ****************************************************************/
-static void
-cursors_locale_changed (EBookBackendFile *bf)
-{
-	GList *l;
-
-	for (l = bf->priv->cursors; l; l = l->next) {
-		EDataBookCursor *cursor = l->data;
-		GError *error = NULL;
-
-		if (!e_data_book_cursor_load_locale (cursor, NULL, &error)) {
-			g_warning ("Error loading cursor locale: %s", error->message);
-			g_clear_error (&error);
-		}
-	}
-}
-
 static void
 cursors_contact_added (EBookBackendFile *bf,
-		       EContact         *contact)
+                       EContact *contact)
 {
 	GList *l;
 
@@ -751,7 +735,7 @@ cursors_contact_added (EBookBackendFile *bf,
 
 static void
 cursors_contact_removed (EBookBackendFile *bf,
-			 EContact         *contact)
+                         EContact *contact)
 {
 	GList *l;
 
@@ -776,6 +760,7 @@ static gboolean
 do_create (EBookBackendFile *bf,
            const gchar * const *vcards,
            GQueue *out_contacts,
+           GCancellable *cancellable,
            GError **error)
 {
 	PhotoModifiedStatus status = STATUS_NORMAL;
@@ -826,19 +811,19 @@ do_create (EBookBackendFile *bf,
 		GList *tail, *link;
 		GSList *slist = NULL, *l;
 
-		/* XXX EBookBackendSqliteDB still uses GSList. */
+		/* EBookSqlite uses GSList. */
 		tail = g_queue_peek_tail_link (&queue);
 		for (link = tail; link != NULL; link = g_list_previous (link))
 			slist = g_slist_prepend (slist, link->data);
 
-		if (!e_book_backend_sqlitedb_new_contacts (bf->priv->sqlitedb,
-							   SQLITEDB_FOLDER_ID,
-							   slist, FALSE,
-							   &local_error)) {
+		if (!e_book_sqlite_add_contacts (bf->priv->sqlitedb,
+						 slist, NULL, FALSE,
+						 cancellable,
+						 &local_error)) {
 
 			if (g_error_matches (local_error,
-					     E_BOOK_SDB_ERROR,
-					     E_BOOK_SDB_ERROR_CONSTRAINT)) {
+					     E_BOOK_SQLITE_ERROR,
+					     E_BOOK_SQLITE_ERROR_CONSTRAINT)) {
 				g_set_error (
 					error, E_BOOK_CLIENT_ERROR,
 					E_BOOK_CLIENT_ERROR_CONTACT_ID_ALREADY_EXISTS,
@@ -923,6 +908,28 @@ notify_update_vcard (EDataBookView *book_view,
 		e_data_book_view_notify_update_vcard (book_view, id, vcard);
 }
 
+static gboolean
+uid_rev_fields (GHashTable *fields_of_interest)
+{
+	GHashTableIter iter;
+	gpointer key, value;
+
+	if (!fields_of_interest || g_hash_table_size (fields_of_interest) > 2)
+		return FALSE;
+
+	g_hash_table_iter_init (&iter, fields_of_interest);
+	while (g_hash_table_iter_next (&iter, &key, &value)) {
+		const gchar *field_name = key;
+		EContactField field = e_contact_field_id (field_name);
+
+		if (field != E_CONTACT_UID &&
+		    field != E_CONTACT_REV)
+			return FALSE;
+	}
+
+	return TRUE;
+}
+
 static gpointer
 book_view_thread (gpointer data)
 {
@@ -931,9 +938,10 @@ book_view_thread (gpointer data)
 	EBookBackendFile *bf;
 	EBookBackendSExp *sexp;
 	const gchar *query;
-	GSList *summary_list, *l;
+	GSList *summary_list = NULL, *l;
 	GHashTable *fields_of_interest;
 	GError *local_error = NULL;
+	gboolean meta_contact, success;
 
 	g_return_val_if_fail (E_IS_DATA_BOOK_VIEW (data), NULL);
 
@@ -955,6 +963,7 @@ book_view_thread (gpointer data)
 	query = e_book_backend_sexp_text (sexp);
 
 	fields_of_interest = e_data_book_view_get_fields_of_interest (book_view);
+	meta_contact = uid_rev_fields (fields_of_interest);
 
 	if ( !strcmp (query, "(contains \"x-evolution-any-field\" \"\")")) {
 		e_data_book_view_notify_progress (book_view, -1, _("Loading..."));
@@ -966,14 +975,16 @@ book_view_thread (gpointer data)
 	e_flag_set (closure->running);
 
 	g_rw_lock_reader_lock (&(bf->priv->lock));
-	summary_list = e_book_backend_sqlitedb_search (
+	success = e_book_sqlite_search (
 		bf->priv->sqlitedb,
-		SQLITEDB_FOLDER_ID,
-		query, fields_of_interest,
-		NULL, NULL, &local_error);
+		query,
+		meta_contact,
+		&summary_list,
+		NULL, /* GCancellable */
+		&local_error);
 	g_rw_lock_reader_unlock (&(bf->priv->lock));
 
-	if (!summary_list && local_error != NULL) {
+	if (!success) {
 		g_warning (G_STRLOC ": Failed to query initial contacts: %s", local_error->message);
 		g_error_free (local_error);
 		e_data_book_view_notify_complete (
@@ -988,7 +999,7 @@ book_view_thread (gpointer data)
 	}
 
 	for (l = summary_list; l; l = l->next) {
-		EbSdbSearchData *data = l->data;
+		EbSqlSearchData *data = l->data;
 		gchar *vcard = NULL;
 
 		vcard = data->vcard;
@@ -998,7 +1009,7 @@ book_view_thread (gpointer data)
 		g_free (vcard);
 	}
 
-	g_slist_foreach (summary_list, (GFunc) e_book_backend_sqlitedb_search_data_free, NULL);
+	g_slist_foreach (summary_list, (GFunc) e_book_sqlite_search_data_free, NULL);
 	g_slist_free (summary_list);
 
 	if (e_flag_is_set (closure->running))
@@ -1011,44 +1022,6 @@ book_view_thread (gpointer data)
 	return NULL;
 }
 
-static void
-view_notify_update (EBookBackendFile *backend,
-                    EDataBookView *view,
-                    EContact *contact)
-{
-	EBookBackendSExp *sexp;
-	GHashTable *fields   = e_data_book_view_get_fields_of_interest (view);
-	const gchar *query;
-	gboolean    notified = FALSE;
-	gboolean    with_all_required_fields = FALSE;
-
-	sexp = e_data_book_view_get_sexp (view);
-	query = e_book_backend_sexp_text (sexp);
-
-	if (e_book_backend_sqlitedb_check_summary_query (backend->priv->sqlitedb, query, NULL) &&
-	    e_book_backend_sqlitedb_check_summary_fields (backend->priv->sqlitedb, fields)) {
-
-		const gchar *uid = e_contact_get_const (contact, E_CONTACT_UID);
-		gchar       *vcard;
-
-		vcard = e_book_backend_sqlitedb_get_vcard_string (
-			backend->priv->sqlitedb,
-			SQLITEDB_FOLDER_ID, uid,
-			fields, &with_all_required_fields, NULL);
-
-		if (vcard) {
-			if (with_all_required_fields) {
-				e_data_book_view_notify_update_prefiltered_vcard (view, uid, vcard);
-				notified = TRUE;
-			}
-			g_free (vcard);
-		}
-	}
-
-	if (!notified)
-		e_data_book_view_notify_update (view, contact);
-}
-
 static void
 book_backend_file_dispose (GObject *object)
 {
@@ -1154,11 +1127,11 @@ book_backend_file_open_sync (EBookBackend *backend,
 	g_rw_lock_writer_lock (&(bf->priv->lock));
 	if (!bf->priv->revision) {
 		e_book_backend_file_load_revision (bf);
-		e_book_backend_notify_property_changed (E_BOOK_BACKEND (backend),
-							BOOK_BACKEND_PROPERTY_REVISION,
-							bf->priv->revision);
+		e_book_backend_notify_property_changed (
+			E_BOOK_BACKEND (backend),
+			BOOK_BACKEND_PROPERTY_REVISION,
+			bf->priv->revision);
 	}
-
 	g_rw_lock_writer_unlock (&(bf->priv->lock));
 
 	e_backend_set_online (E_BACKEND (backend), TRUE);
@@ -1175,28 +1148,40 @@ book_backend_file_create_contacts_sync (EBookBackend *backend,
                                         GError **error)
 {
 	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
-	gboolean success;
+	gboolean success = FALSE;
 
 	g_rw_lock_writer_lock (&(bf->priv->lock));
+	if (!e_book_sqlite_lock (bf->priv->sqlitedb,
+				 EBSQL_LOCK_WRITE,
+				 cancellable, error)) {
+		g_rw_lock_writer_unlock (&(bf->priv->lock));
+		return FALSE;
+	}
 
-	success = e_book_backend_sqlitedb_lock_updates (bf->priv->sqlitedb, error);
-
-	if (success)
-		success = do_create (bf, vcards, out_contacts, error);
+	success = do_create (bf, vcards, out_contacts, cancellable, error);
 
 	if (success)
 		success = e_book_backend_file_bump_revision (bf, error);
 
 	if (success)
-		success = e_book_backend_sqlitedb_unlock_updates (bf->priv->sqlitedb,
-								  TRUE, error);
+		success = e_book_sqlite_unlock (
+			bf->priv->sqlitedb,
+			EBSQL_UNLOCK_COMMIT,
+			error);
 	else {
 		GError *local_error = NULL;
 
 		/* Rollback transaction */
-		if (!e_book_backend_sqlitedb_unlock_updates (bf->priv->sqlitedb, FALSE, &local_error)) {
-			g_warning ("Failed to rollback transaction after failing to add contacts: %s",
-				   local_error->message);
+		e_book_sqlite_unlock (
+			bf->priv->sqlitedb,
+			EBSQL_UNLOCK_ROLLBACK,
+			&local_error);
+
+		if (local_error != NULL) {
+			g_warning (
+				"Failed to rollback transaction "
+				"after failing to add contacts: %s",
+				local_error->message);
 			g_clear_error (&local_error);
 		}
 	}
@@ -1225,12 +1210,14 @@ book_backend_file_modify_contacts_sync (EBookBackend *backend,
 
 	g_rw_lock_writer_lock (&(bf->priv->lock));
 
-	if (!e_book_backend_sqlitedb_lock_updates (bf->priv->sqlitedb, error))
-		status = STATUS_ERROR;
+	if (!e_book_sqlite_lock (bf->priv->sqlitedb, EBSQL_LOCK_WRITE, cancellable, error)) {
+		g_rw_lock_writer_unlock (&(bf->priv->lock));
+		return FALSE;
+	}
 
 	for (ii = 0; ii < length && status != STATUS_ERROR; ii++) {
 		gchar *id;
-		EContact *mod_contact, *old_contact;
+		EContact *mod_contact, *old_contact = NULL;
 		const gchar *mod_contact_rev, *old_contact_rev;
 
 		mod_contact = e_contact_new_from_vcard (vcards[ii]);
@@ -1247,11 +1234,9 @@ book_backend_file_modify_contacts_sync (EBookBackend *backend,
 			break;
 		}
 
-		old_contact = e_book_backend_sqlitedb_get_contact (
-			bf->priv->sqlitedb,
-			SQLITEDB_FOLDER_ID, id,
-			NULL, NULL, &local_error);
-		if (!old_contact) {
+		if (!e_book_sqlite_get_contact (bf->priv->sqlitedb,
+						id, FALSE, &old_contact,
+						&local_error)) {
 			g_warning (G_STRLOC ": Failed to load contact %s: %s", id, local_error->message);
 			g_propagate_error (error, local_error);
 			local_error = NULL;
@@ -1325,7 +1310,7 @@ book_backend_file_modify_contacts_sync (EBookBackend *backend,
 			mod_link = g_list_next (mod_link);
 		}
 
-		/* XXX EBookBackendSqliteDB still uses GSList. */
+		/* EBookSqlite uses GSList. */
 		mod_link = g_queue_peek_tail_link (&mod_contact_queue);
 		while (mod_link != NULL) {
 			slist = g_slist_prepend (slist, mod_link->data);
@@ -1333,11 +1318,15 @@ book_backend_file_modify_contacts_sync (EBookBackend *backend,
 		}
 
 		/* Update summary as well */
-		if (!e_book_backend_sqlitedb_new_contacts (bf->priv->sqlitedb,
-							   SQLITEDB_FOLDER_ID,
-							   slist, TRUE,
-							   &local_error)) {
-			g_warning ("Failed to modify contacts: %s", local_error->message);
+		e_book_sqlite_add_contacts (
+			bf->priv->sqlitedb,
+			slist, NULL, TRUE,
+			cancellable, &local_error);
+
+		if (local_error != NULL) {
+			g_warning (
+				"Failed to modify contacts: %s",
+				local_error->message);
 			g_propagate_error (error, local_error);
 			local_error = NULL;
 
@@ -1355,16 +1344,27 @@ book_backend_file_modify_contacts_sync (EBookBackend *backend,
 
 	/* Commit or rollback transaction */
 	if (status != STATUS_ERROR) {
+		gboolean success;
 
-		if (!e_book_backend_sqlitedb_unlock_updates (bf->priv->sqlitedb,
-							     TRUE, error))
+		success = e_book_sqlite_unlock (
+			bf->priv->sqlitedb,
+			EBSQL_UNLOCK_COMMIT,
+			error);
+		if (!success)
 			status = STATUS_ERROR;
 
 	} else {
 		/* Rollback transaction */
-		if (!e_book_backend_sqlitedb_unlock_updates (bf->priv->sqlitedb, FALSE, &local_error)) {
-			g_warning ("Failed to rollback transaction after failing to modify contacts: %s",
-				   local_error->message);
+		e_book_sqlite_unlock (
+			bf->priv->sqlitedb,
+			EBSQL_UNLOCK_ROLLBACK,
+			&local_error);
+
+		if (local_error != NULL) {
+			g_warning (
+				"Failed to rollback transaction after "
+				"failing to modify contacts: %s",
+				local_error->message);
 			g_clear_error (&local_error);
 		}
 	}
@@ -1372,8 +1372,8 @@ book_backend_file_modify_contacts_sync (EBookBackend *backend,
 	if (status != STATUS_ERROR)
 		e_queue_transfer (&mod_contact_queue, out_contacts);
 
-	/* Now that we've modified the contact(s), notify cursors of the changes
-	 */
+	/* Now that we've modified the contact(s),
+	 * notify cursors of the changes. */
 	if (status != STATUS_ERROR) {
 		GList *l;
 
@@ -1418,31 +1418,33 @@ book_backend_file_remove_contacts_sync (EBookBackend *backend,
 
 	g_rw_lock_writer_lock (&(bf->priv->lock));
 
-	success = e_book_backend_sqlitedb_lock_updates (bf->priv->sqlitedb, error);
+	if (!e_book_sqlite_lock (bf->priv->sqlitedb,
+				 EBSQL_LOCK_WRITE,
+				 cancellable, error)) {
+		g_rw_lock_writer_unlock (&(bf->priv->lock));
+		return FALSE;
+	}
 
 	for (ii = 0; ii < length && success; ii++) {
-		EContact *contact;
+		EContact *contact = NULL;
 
 		/* First load the EContacts which need to be removed, we might delete some
 		 * photos from disk because of this...
 		 *
-		 * Note: sqlitedb backend can probably make this faster by executing a
+		 * Note: sqlite backend can probably make this faster by executing a
 		 * single query to fetch a list of contacts for a list of ids, the
 		 * current method makes a query for each UID.
 		 */
-		contact = e_book_backend_sqlitedb_get_contact (
-			bf->priv->sqlitedb,
-			SQLITEDB_FOLDER_ID, uids[ii],
-			NULL, NULL, &local_error);
-
-		if (contact) {
-			removed_ids      = g_slist_prepend (removed_ids, g_strdup (uids[ii]));
+		if (e_book_sqlite_get_contact (bf->priv->sqlitedb,
+					       uids[ii], FALSE, &contact,
+					       &local_error)) {
+			removed_ids = g_slist_prepend (removed_ids, g_strdup (uids[ii]));
 			removed_contacts = g_slist_prepend (removed_contacts, contact);
 		} else {
 
 			if (g_error_matches (local_error,
-					     E_BOOK_SDB_ERROR,
-					     E_BOOK_SDB_ERROR_CONTACT_NOT_FOUND)) {
+					     E_BOOK_SQLITE_ERROR,
+					     E_BOOK_SQLITE_ERROR_CONTACT_NOT_FOUND)) {
 				g_set_error (
 					error, E_BOOK_CLIENT_ERROR,
 					E_BOOK_CLIENT_ERROR_CONTACT_NOT_FOUND,
@@ -1467,9 +1469,8 @@ book_backend_file_remove_contacts_sync (EBookBackend *backend,
 		}
 
 		/* Remove from summary as well */
-		if (!e_book_backend_sqlitedb_remove_contacts (bf->priv->sqlitedb,
-						      SQLITEDB_FOLDER_ID,
-						      removed_ids, &local_error)) {
+		if (!e_book_sqlite_remove_contacts (bf->priv->sqlitedb, removed_ids,
+						    cancellable, &local_error)) {
 			g_warning ("Failed to remove contacts: %s", local_error->message);
 			g_propagate_error (error, local_error);
 		}
@@ -1479,13 +1480,13 @@ book_backend_file_remove_contacts_sync (EBookBackend *backend,
 
 	/* Commit or rollback transaction */
 	if (success) {
-		success = e_book_backend_sqlitedb_unlock_updates (bf->priv->sqlitedb,
-								  TRUE, error);
+		success = e_book_sqlite_unlock (bf->priv->sqlitedb, EBSQL_UNLOCK_COMMIT, error);
 	} else {
 		/* Rollback transaction */
-		if (!e_book_backend_sqlitedb_unlock_updates (bf->priv->sqlitedb, FALSE, &local_error)) {
-			g_warning ("Failed to rollback transaction after failing to modify contacts: %s",
-				   local_error->message);
+		if (!e_book_sqlite_unlock (bf->priv->sqlitedb, EBSQL_UNLOCK_ROLLBACK, &local_error)) {
+			g_warning (
+				"Failed to rollback transaction after failing to modify contacts: %s",
+				local_error->message);
 			g_clear_error (&local_error);
 		}
 	}
@@ -1512,28 +1513,21 @@ book_backend_file_get_contact_sync (EBookBackend *backend,
                                     GError **error)
 {
 	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
-	EContact *contact;
+	EContact *contact = NULL;
+	gboolean success;
 	GError *local_error = NULL;
 
 	g_rw_lock_reader_lock (&(bf->priv->lock));
-
-	contact = e_book_backend_sqlitedb_get_contact (
+	success = e_book_sqlite_get_contact (
 		bf->priv->sqlitedb,
-		SQLITEDB_FOLDER_ID, uid,
-		NULL, NULL, &local_error);
-
+		uid, FALSE, &contact,
+		&local_error);
 	g_rw_lock_reader_unlock (&(bf->priv->lock));
 
-	/* Sanity check. */
-	g_return_val_if_fail (
-		((contact != NULL) && (local_error == NULL)) ||
-		((contact == NULL) && (local_error != NULL)), NULL);
-
-	if (local_error) {
-
+	if (!success) {
 		if (g_error_matches (local_error,
-				     E_BOOK_SDB_ERROR,
-				     E_BOOK_SDB_ERROR_CONTACT_NOT_FOUND)) {
+				     E_BOOK_SQLITE_ERROR,
+				     E_BOOK_SQLITE_ERROR_CONTACT_NOT_FOUND)) {
 			g_set_error (
 				error, E_BOOK_CLIENT_ERROR,
 				E_BOOK_CLIENT_ERROR_CONTACT_NOT_FOUND,
@@ -1541,7 +1535,6 @@ book_backend_file_get_contact_sync (EBookBackend *backend,
 			g_error_free (local_error);
 		} else
 			g_propagate_error (error, local_error);
-
 	}
 
 	return contact;
@@ -1555,7 +1548,7 @@ book_backend_file_get_contact_list_sync (EBookBackend *backend,
                                          GError **error)
 {
 	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
-	GSList *summary_list;
+	GSList *summary_list = NULL;
 	GSList *link;
 	gboolean success = TRUE;
 	GError *local_error = NULL;
@@ -1563,19 +1556,38 @@ book_backend_file_get_contact_list_sync (EBookBackend *backend,
 	d (printf ("book_backend_file_get_contact_list_sync (%s)\n", query));
 
 	g_rw_lock_reader_lock (&(bf->priv->lock));
-	summary_list = e_book_backend_sqlitedb_search (
-		bf->priv->sqlitedb, SQLITEDB_FOLDER_ID,
-		query, NULL,
-		NULL, NULL, &local_error);
+
+	success = e_book_sqlite_lock (
+		bf->priv->sqlitedb,
+		EBSQL_LOCK_READ,
+		cancellable, error);
+	if (!success) {
+		g_rw_lock_writer_unlock (&(bf->priv->lock));
+		return FALSE;
+	}
+
+	success = e_book_sqlite_search (
+		bf->priv->sqlitedb,
+		query,
+		FALSE,
+		&summary_list,
+		cancellable,
+		&local_error);
+
+	e_book_sqlite_unlock (
+		bf->priv->sqlitedb,
+		EBSQL_UNLOCK_NONE,
+		success ? &local_error : NULL);
+
 	g_rw_lock_reader_unlock (&(bf->priv->lock));
 
-	if (local_error != NULL) {
+	if (!success) {
 
 		g_warn_if_fail (summary_list == NULL);
 
 		if (g_error_matches (local_error,
-				     E_BOOK_SDB_ERROR,
-				     E_BOOK_SDB_ERROR_NOT_SUPPORTED)) {
+				     E_BOOK_SQLITE_ERROR,
+				     E_BOOK_SQLITE_ERROR_UNSUPPORTED_QUERY)) {
 			g_set_error (
 				error, E_CLIENT_ERROR,
 				E_CLIENT_ERROR_NOT_SUPPORTED,
@@ -1583,8 +1595,8 @@ book_backend_file_get_contact_list_sync (EBookBackend *backend,
 			g_error_free (local_error);
 
 		} else if (g_error_matches (local_error,
-				     E_BOOK_SDB_ERROR,
-				     E_BOOK_SDB_ERROR_INVALID_QUERY)) {
+				     E_BOOK_SQLITE_ERROR,
+				     E_BOOK_SQLITE_ERROR_INVALID_QUERY)) {
 			g_set_error (
 				error, E_CLIENT_ERROR,
 				E_CLIENT_ERROR_INVALID_QUERY,
@@ -1595,12 +1607,10 @@ book_backend_file_get_contact_list_sync (EBookBackend *backend,
 			g_warning ("Failed to fetch contact ids: %s", local_error->message);
 			g_propagate_error (error, local_error);
 		}
-
-		success = FALSE;
 	}
 
 	for (link = summary_list; link != NULL; link = g_slist_next (link)) {
-		EbSdbSearchData *data = link->data;
+		EbSqlSearchData *data = link->data;
 		EContact *contact;
 
 		contact = e_contact_new_from_vcard (data->vcard);
@@ -1609,7 +1619,7 @@ book_backend_file_get_contact_list_sync (EBookBackend *backend,
 
 	g_slist_free_full (
 		summary_list, (GDestroyNotify)
-		e_book_backend_sqlitedb_search_data_free);
+		e_book_sqlite_search_data_free);
 
 	return success;
 }
@@ -1622,7 +1632,7 @@ book_backend_file_get_contact_list_uids_sync (EBookBackend *backend,
                                               GError **error)
 {
 	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
-	GSList *uids;
+	GSList *uids = NULL;
 	GSList *link;
 	gboolean success = TRUE;
 	GError *local_error = NULL;
@@ -1630,19 +1640,35 @@ book_backend_file_get_contact_list_uids_sync (EBookBackend *backend,
 	d (printf ("book_backend_file_get_contact_list_sync (%s)\n", query));
 
 	g_rw_lock_reader_lock (&(bf->priv->lock));
-	uids = e_book_backend_sqlitedb_search_uids (
+
+	success = e_book_sqlite_lock (
 		bf->priv->sqlitedb,
-		SQLITEDB_FOLDER_ID,
-		query, NULL, &local_error);
+		EBSQL_LOCK_READ,
+		cancellable, error);
+	if (!success) {
+		g_rw_lock_writer_unlock (&(bf->priv->lock));
+		return FALSE;
+	}
+
+	success = e_book_sqlite_search_uids (
+		bf->priv->sqlitedb,
+		query,
+		&uids,
+		cancellable,
+		&local_error);
+	e_book_sqlite_unlock (
+		bf->priv->sqlitedb,
+		EBSQL_UNLOCK_NONE,
+		success ? &local_error : NULL);
+
 	g_rw_lock_reader_unlock (&(bf->priv->lock));
 
-	if (local_error != NULL) {
-
+	if (!success) {
 		g_warn_if_fail (uids == NULL);
 
 		if (g_error_matches (local_error,
-				     E_BOOK_SDB_ERROR,
-				     E_BOOK_SDB_ERROR_NOT_SUPPORTED)) {
+				     E_BOOK_SQLITE_ERROR,
+				     E_BOOK_SQLITE_ERROR_UNSUPPORTED_QUERY)) {
 			g_set_error (
 				error, E_CLIENT_ERROR,
 				E_CLIENT_ERROR_NOT_SUPPORTED,
@@ -1650,8 +1676,8 @@ book_backend_file_get_contact_list_uids_sync (EBookBackend *backend,
 			g_error_free (local_error);
 
 		} else if (g_error_matches (local_error,
-				     E_BOOK_SDB_ERROR,
-				     E_BOOK_SDB_ERROR_INVALID_QUERY)) {
+					    E_BOOK_SQLITE_ERROR,
+					    E_BOOK_SQLITE_ERROR_INVALID_QUERY)) {
 			g_set_error (
 				error, E_CLIENT_ERROR,
 				E_CLIENT_ERROR_INVALID_QUERY,
@@ -1659,11 +1685,11 @@ book_backend_file_get_contact_list_uids_sync (EBookBackend *backend,
 			g_error_free (local_error);
 
 		} else {
-			g_warning ("Failed to fetch contact ids: %s", local_error->message);
+			g_warning (
+				"Failed to fetch contact ids: %s",
+				local_error->message);
 			g_propagate_error (error, local_error);
 		}
-
-		success = FALSE;
 	}
 
 	/* Transfer UID strings to the GQueue. */
@@ -1679,7 +1705,9 @@ static void
 book_backend_file_start_view (EBookBackend *backend,
                               EDataBookView *book_view)
 {
-	FileBackendSearchClosure *closure = init_closure (book_view, E_BOOK_BACKEND_FILE (backend));
+	FileBackendSearchClosure *closure;
+
+	closure = init_closure (book_view, E_BOOK_BACKEND_FILE (backend));
 
 	d (printf ("starting book view thread\n"));
 	closure->thread = g_thread_new (NULL, book_view_thread, book_view);
@@ -1710,23 +1738,6 @@ book_backend_file_stop_view (EBookBackend *backend,
 	}
 }
 
-static void
-book_backend_file_notify_update (EBookBackend *backend,
-                                 const EContact *contact)
-{
-	GList *list, *link;
-
-	list = e_book_backend_list_views (backend);
-
-	for (link = list; link != NULL; link = g_list_next (link))
-		view_notify_update (
-			E_BOOK_BACKEND_FILE (backend),
-			E_DATA_BOOK_VIEW (link->data),
-			(EContact *) contact);
-
-	g_list_free_full (list, (GDestroyNotify) g_object_unref);
-}
-
 static EDataBookDirect *
 book_backend_file_get_direct_book (EBookBackend *backend)
 {
@@ -1745,11 +1756,16 @@ book_backend_file_get_direct_book (EBookBackend *backend)
 		registry, source, GET_PATH_DB_DIR);
 
 	/* Support in-tree testing / relocated modules */
-	if (modules_env)
-		backend_path = g_build_filename (modules_env, "libebookbackendfile.so", NULL);
-	else
-		backend_path = g_build_filename (BACKENDDIR, "libebookbackendfile.so", NULL);
-	direct = e_data_book_direct_new (backend_path, "EBookBackendFileFactory", dirname);
+	if (modules_env) {
+		backend_path = g_build_filename (
+			modules_env, "libebookbackendfile.so", NULL);
+	} else {
+		backend_path = g_build_filename (
+			BACKENDDIR, "libebookbackendfile.so", NULL);
+	}
+
+	direct = e_data_book_direct_new (
+		backend_path, "EBookBackendFileFactory", dirname);
 
 	g_free (backend_path);
 	g_free (dirname);
@@ -1778,64 +1794,82 @@ book_backend_file_sync (EBookBackend *backend)
 }
 
 static void
+book_backend_file_vcard_changed (EbSqlChangeType change_type,
+                                 const gchar *uid,
+                                 const gchar *extra,
+                                 const gchar *vcard,
+                                 gpointer user_data)
+{
+	EBookBackend *backend = E_BOOK_BACKEND (user_data);
+	EContact     *contact;
+
+	if (change_type == EBSQL_CHANGE_LOCALE_CHANGED) {
+		contact = e_contact_new_from_vcard_with_uid (vcard, uid);
+		e_book_backend_notify_update (backend, contact);
+		g_object_unref (contact);
+	}
+}
+
+static gboolean
 book_backend_file_set_locale (EBookBackend *backend,
-			      const gchar  *locale)
+                              const gchar *locale,
+                              GCancellable *cancellable,
+                              GError **error)
 {
 	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
 	gboolean success;
-	GError *error = NULL;
+	GList *l;
 
 	g_rw_lock_writer_lock (&(bf->priv->lock));
 
-	success = e_book_backend_sqlitedb_lock_updates (bf->priv->sqlitedb,
-							&error);
-
+	success = e_book_sqlite_lock (
+		bf->priv->sqlitedb,
+		EBSQL_LOCK_WRITE,
+		cancellable, error);
 	if (!success) {
-		g_warning ("Failed to start SQLite transaction: %s", error->message);
-		g_clear_error (&error);
+		g_rw_lock_writer_unlock (&(bf->priv->lock));
+		return FALSE;
 	}
 
-	if (success) {
-		success = e_book_backend_sqlitedb_set_locale (bf->priv->sqlitedb,
-							      SQLITEDB_FOLDER_ID,
-							      locale,
-							      &error);
-		if (!success) {
-			g_warning ("Failed to set locale on SQLiteDB: %s", error->message);
-			g_clear_error (&error);
-		}
-	}
+	success = e_book_sqlite_set_locale (
+		bf->priv->sqlitedb, locale,
+		cancellable, error);
+
+	if (success)
+		success = e_book_backend_file_bump_revision (bf, error);
 
 	if (success) {
-		success = e_book_backend_file_bump_revision (bf, &error);
-
-		if (!success) {
-			g_warning ("Failed to set locale on SQLiteDB: %s", error->message);
-			g_clear_error (&error);
-		}
-	}
-
-	if (success) {
-		success = e_book_backend_sqlitedb_unlock_updates (bf->priv->sqlitedb,
-								  TRUE, &error);
-
-		if (!success) {
-			g_warning ("Failed to commit SQLite transaction: %s", error->message);
-			g_clear_error (&error);
-		}
+		success = e_book_sqlite_unlock (
+			bf->priv->sqlitedb,
+			EBSQL_UNLOCK_COMMIT,
+			error);
 
 	} else {
-		GError *error = NULL;
+		GError *local_error = NULL;
 
 		/* Rollback transaction */
-		if (!e_book_backend_sqlitedb_unlock_updates (bf->priv->sqlitedb, FALSE, &error)) {
-			g_warning ("Failed to rollback transaction after failing to add contacts: %s",
-				   error->message);
-			g_clear_error (&error);
+		e_book_sqlite_unlock (
+			bf->priv->sqlitedb,
+			EBSQL_UNLOCK_ROLLBACK,
+			&local_error);
+
+		if (local_error != NULL) {
+			g_warning (
+				"Failed to rollback transaction "
+				"after failing to set locale: %s",
+				local_error->message);
+			g_clear_error (&local_error);
 		}
 	}
 
-	cursors_locale_changed (bf);
+	/* This must be done outside the EBookSqlite lock,
+	 * as it may try to acquire the lock as well. */
+	for (l = bf->priv->cursors; success && l; l = l->next) {
+		EDataBookCursor *cursor = l->data;
+
+		success = e_data_book_cursor_load_locale (
+			cursor, NULL, cancellable, error);
+	}
 
 	/* We set the new locale, now update our local variable */
 	if (success) {
@@ -1844,6 +1878,8 @@ book_backend_file_set_locale (EBookBackend *backend,
 	}
 
 	g_rw_lock_writer_unlock (&(bf->priv->lock));
+
+	return success;
 }
 
 static gchar *
@@ -1861,27 +1897,29 @@ book_backend_file_dup_locale (EBookBackend *backend)
 
 static EDataBookCursor *
 book_backend_file_create_cursor (EBookBackend *backend,
-				 EContactField *sort_fields,
-				 EBookCursorSortType *sort_types,
-				 guint n_fields,
-				 GError **error)
+                                 EContactField *sort_fields,
+                                 EBookCursorSortType *sort_types,
+                                 guint n_fields,
+                                 GError **error)
 {
 	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
-	EDataBookCursor  *cursor;
+	EDataBookCursor *cursor;
 
 	g_rw_lock_writer_lock (&(bf->priv->lock));
 
-	cursor = e_data_book_cursor_sqlite_new (backend,
-						bf->priv->sqlitedb,
-						SQLITEDB_FOLDER_ID,
-						sort_fields,
-						sort_types,
-						n_fields,
-						error);
+	cursor = e_data_book_cursor_sqlite_new (
+		backend,
+		bf->priv->sqlitedb,
+		SQLITE_REVISION_KEY,
+		sort_fields,
+		sort_types,
+		n_fields,
+		error);
 
-	if (cursor)
+	if (cursor != NULL) {
 		bf->priv->cursors =
 			g_list_prepend (bf->priv->cursors, cursor);
+	}
 
 	g_rw_lock_writer_unlock (&(bf->priv->lock));
 
@@ -1890,8 +1928,8 @@ book_backend_file_create_cursor (EBookBackend *backend,
 
 static gboolean
 book_backend_file_delete_cursor (EBookBackend *backend,
-				 EDataBookCursor *cursor,
-				 GError **error)
+                                 EDataBookCursor *cursor,
+                                 GError **error)
 {
 	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
 	GList *link;
@@ -1904,10 +1942,11 @@ book_backend_file_delete_cursor (EBookBackend *backend,
 		bf->priv->cursors = g_list_delete_link (bf->priv->cursors, link);
 		g_object_unref (cursor);
 	} else {
-		g_set_error_literal (error,
-				     E_CLIENT_ERROR,
-				     E_CLIENT_ERROR_INVALID_ARG,
-				     _("Requested to delete an unrelated cursor"));
+		g_set_error_literal (
+			error,
+			E_CLIENT_ERROR,
+			E_CLIENT_ERROR_INVALID_ARG,
+			_("Requested to delete an unrelated cursor"));
 	}
 
 	g_rw_lock_writer_unlock (&(bf->priv->lock));
@@ -1925,9 +1964,8 @@ book_backend_file_initable_init (GInitable *initable,
 	ESourceRegistry *registry;
 	ESource *source;
 	const gchar *extension_name;
-	gchar *backup;
-	gchar *dirname;
-	gchar *filename;
+	gchar *backup, *dirname;
+	gchar *filename, *fullpath;
 	gboolean success = TRUE;
 
 	priv = E_BOOK_BACKEND_FILE_GET_PRIVATE (initable);
@@ -1947,26 +1985,24 @@ book_backend_file_initable_init (GInitable *initable,
 
 	filename = g_build_filename (dirname, "addressbook.db", NULL);
 	backup = g_build_filename (dirname, "addressbook.db.old", NULL);
+	fullpath = g_build_filename (dirname, "contacts.db", NULL);
 
 	/* The old BDB exists, lets migrate that to sqlite right away. */
 	if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
-		priv->sqlitedb = e_book_backend_sqlitedb_new_full (
-			dirname,
-			SQLITEDB_EMAIL_ID,
-			SQLITEDB_FOLDER_ID,
-			SQLITEDB_FOLDER_NAME,
-			TRUE, setup_extension,
-			error);
+		priv->sqlitedb = e_book_sqlite_new_full (
+			fullpath, setup_extension,
+			NULL,
+			book_backend_file_vcard_changed,
+			initable, NULL, cancellable, error);
 
 		if (priv->sqlitedb == NULL) {
 			success = FALSE;
 			goto exit;
 		}
 
+		/* Do the migration from BDB, see e-book-backend-file-migrate-bdb.c */
 		success = e_book_backend_file_migrate_bdb (
-			priv->sqlitedb,
-			SQLITEDB_FOLDER_ID,
-			dirname, filename, error);
+			priv->sqlitedb, dirname, filename, cancellable, error);
 
 		if (!success)
 			goto exit;
@@ -1988,7 +2024,7 @@ book_backend_file_initable_init (GInitable *initable,
 	/* If we already have a handle on this, it means there
 	 * was an old BDB migrated and no need to reopen it. */
 	if (priv->sqlitedb == NULL) {
-		gboolean populated;
+		gint populated = 0;
 		GError *local_error = NULL;
 
 		/* Ensure the directory exists first. */
@@ -1998,13 +2034,11 @@ book_backend_file_initable_init (GInitable *initable,
 			goto exit;
 
 		/* Create the sqlitedb. */
-		priv->sqlitedb = e_book_backend_sqlitedb_new_full (
-			dirname,
-			SQLITEDB_EMAIL_ID,
-			SQLITEDB_FOLDER_ID,
-			SQLITEDB_FOLDER_NAME,
-			TRUE, setup_extension,
-			error);
+		priv->sqlitedb = e_book_sqlite_new_full (
+			fullpath, setup_extension,
+			NULL,
+			book_backend_file_vcard_changed,
+			initable, NULL, cancellable, error);
 
 		if (priv->sqlitedb == NULL) {
 			success = FALSE;
@@ -2012,9 +2046,10 @@ book_backend_file_initable_init (GInitable *initable,
 		}
 
 		/* An sqlite DB only 'exists' if the populated flag is set. */
-		populated = e_book_backend_sqlitedb_get_is_populated (
+		e_book_sqlite_get_key_value_int (
 			priv->sqlitedb,
-			SQLITEDB_FOLDER_ID,
+			E_BOOK_SQL_IS_POPULATED_KEY,
+			&populated,
 			&local_error);
 
 		if (local_error != NULL) {
@@ -2025,10 +2060,11 @@ book_backend_file_initable_init (GInitable *initable,
 
 		if (!populated) {
 			/* Set the populated flag. */
-			success = e_book_backend_sqlitedb_set_is_populated (
+			success = e_book_sqlite_set_key_value_int (
 				priv->sqlitedb,
-				SQLITEDB_FOLDER_ID,
-				TRUE, error);
+				E_BOOK_SQL_IS_POPULATED_KEY,
+				TRUE,
+				error);
 
 			if (!success)
 				goto exit;
@@ -2046,6 +2082,7 @@ book_backend_file_initable_init (GInitable *initable,
 
 exit:
 	g_free (dirname);
+	g_free (fullpath);
 	g_free (filename);
 	g_free (backup);
 
@@ -2075,7 +2112,6 @@ e_book_backend_file_class_init (EBookBackendFileClass *class)
 	backend_class->get_contact_list_uids_sync = book_backend_file_get_contact_list_uids_sync;
 	backend_class->start_view = book_backend_file_start_view;
 	backend_class->stop_view = book_backend_file_stop_view;
-	backend_class->notify_update = book_backend_file_notify_update;
 	backend_class->get_direct_book = book_backend_file_get_direct_book;
 	backend_class->configure_direct = book_backend_file_configure_direct;
 	backend_class->sync = book_backend_file_sync;
