@@ -24,6 +24,16 @@
  *    Tristan Van Berkom <tristanvb@openismus.com>
  */
 
+/**
+ * SECTION: e-book-query
+ * @include: libebook-contacts/libebook-contacts.h
+ * @short_description: Querying and filtering contacts in an addressbook
+ *
+ * This utility can be used to conveniently create search expressions
+ * which can later be used to query and filter results in the #EBookClient,
+ * #EBookClientView and #EBookClientCursor interfaces.
+ **/
+
 #include "e-book-query.h"
 
 #include <config.h>
@@ -806,6 +816,8 @@ field_test_name (EBookQueryTest field_test)
 		return "regex_normal";
 	case E_BOOK_QUERY_REGEX_RAW:
 		return "regex_raw";
+	case E_BOOK_QUERY_LAST:
+		g_return_val_if_reached (NULL);
 	}
 
 	return NULL;
@@ -826,6 +838,7 @@ is_phone_test (EBookQueryTest field_test)
 	case E_BOOK_QUERY_ENDS_WITH:
 	case E_BOOK_QUERY_REGEX_NORMAL:
 	case E_BOOK_QUERY_REGEX_RAW:
+	case E_BOOK_QUERY_LAST:
 		break;
 	}
 
