@@ -26,7 +26,7 @@
  * SECTION:e-vcard
  * @short_description: vCard parsing and interpretation
  * @stability: Stable
- * @include: libebook-contacts/e-vcard.h
+ * @include: libebook-contacts/libebook-contacts.h
  *
  * #EVCard is a low-level representation of a vCard, as specified in RFCs
  * 2425 and 2426 (for vCard version 3.0); this class only supports versions 2.1
@@ -960,6 +960,27 @@ void
 e_vcard_construct_with_uid (EVCard *evc,
                             const gchar *str,
                             const gchar *uid)
+{
+	e_vcard_construct_full (evc, str, -1, uid);
+}
+
+/**
+ * e_vcard_construct_full:
+ * @evc: an existing #EVCard
+ * @str: a vCard string
+ * @len: length of @str, or -1 if @str is %NULL terminated
+ * @uid: (allow-none): a unique ID string
+ *
+ * Similar to e_vcard_construct_with_uid(), but can also
+ * be used with an @str that is not %NULL terminated.
+ *
+ * Since: 3.12
+ **/
+void
+e_vcard_construct_full (EVCard *evc,
+                        const gchar *str,
+                        gssize len,
+                        const gchar *uid)
 {
 	g_return_if_fail (E_IS_VCARD (evc));
 	g_return_if_fail (str != NULL);
@@ -967,8 +988,13 @@ e_vcard_construct_with_uid (EVCard *evc,
 	g_return_if_fail (evc->priv->attributes == NULL);
 
 	/* Lazy construction */
-	if (*str)
-		evc->priv->vcard = g_strdup (str);
+	if (*str) {
+
+		if (len < 0)
+			evc->priv->vcard = g_strdup (str);
+		else
+			evc->priv->vcard = g_strndup (str, len);
+	}
 
 	/* Add UID attribute */
 	if (uid) {
