@@ -21,6 +21,13 @@
  * 02110-1301, USA.
  */
 
+/**
+ * SECTION: e-book-backend-summary
+ * @include: libedata-book/libedata-book.h
+ * @short_description: A utility for storing contact data and searching for contacts
+ *
+ * The #EBookBackendSummary is deprecated, use #EBookSqlite instead.
+ */
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -153,6 +160,8 @@ clear_items (EBookBackendSummary *summary)
  * the summary is changed until it is flushed to disk.
  *
  * Returns: A new #EBookBackendSummary.
+ *
+ * Deprecated: 3.12: Use #EBookSqlite instead
  **/
 EBookBackendSummary *
 e_book_backend_summary_new (const gchar *summary_path,
@@ -441,8 +450,9 @@ e_book_backend_summary_open (EBookBackendSummary *summary)
 		}
 		else {
 			if (g_rename (new_filename, summary->priv->summary_path) == -1) {
-				g_warning ("%s: Failed to rename '%s' to '%s': %s", G_STRFUNC,
-					   new_filename, summary->priv->summary_path, g_strerror (errno));
+				g_warning (
+					"%s: Failed to rename '%s' to '%s': %s", G_STRFUNC,
+					new_filename, summary->priv->summary_path, g_strerror (errno));
 			}
 			g_free (new_filename);
 		}
@@ -483,6 +493,8 @@ e_book_backend_summary_open (EBookBackendSummary *summary)
  * not out of date.
  *
  * Returns: %TRUE if the load succeeded, %FALSE if it failed.
+ *
+ * Deprecated: 3.12: Use #EBookSqlite instead
  **/
 gboolean
 e_book_backend_summary_load (EBookBackendSummary *summary)
@@ -640,6 +652,8 @@ e_book_backend_summary_save_item (EBookBackendSummary *summary,
  * Attempts to save @summary to disk.
  *
  * Returns: %TRUE if the save succeeded, %FALSE otherwise.
+ *
+ * Deprecated: 3.12: Use #EBookSqlite instead
  **/
 gboolean
 e_book_backend_summary_save (EBookBackendSummary *summary)
@@ -691,8 +705,9 @@ e_book_backend_summary_save (EBookBackendSummary *summary)
 	/* unlink the old summary and rename the new one */
 	g_unlink (summary->priv->summary_path);
 	if (g_rename (new_filename, summary->priv->summary_path) == -1) {
-		g_warning ("%s: Failed to rename '%s' to '%s': %s", G_STRFUNC,
-			   new_filename, summary->priv->summary_path, g_strerror (errno));
+		g_warning (
+			"%s: Failed to rename '%s' to '%s': %s", G_STRFUNC,
+			new_filename, summary->priv->summary_path, g_strerror (errno));
 	}
 
 	g_free (new_filename);
@@ -723,6 +738,8 @@ e_book_backend_summary_save (EBookBackendSummary *summary)
  *
  * Adds a summary of @contact to @summary. Does not check if
  * the contact already has a summary.
+ *
+ * Deprecated: 3.12: Use #EBookSqlite instead
  **/
 void
 e_book_backend_summary_add_contact (EBookBackendSummary *summary,
@@ -749,17 +766,17 @@ e_book_backend_summary_add_contact (EBookBackendSummary *summary,
 
 	new_item = g_new0 (EBookBackendSummaryItem, 1);
 
-	new_item->id         = id;
-	new_item->nickname   = e_contact_get (contact, E_CONTACT_NICKNAME);
-	new_item->full_name  = e_contact_get (contact, E_CONTACT_FULL_NAME);
+	new_item->id = id;
+	new_item->nickname = e_contact_get (contact, E_CONTACT_NICKNAME);
+	new_item->full_name = e_contact_get (contact, E_CONTACT_FULL_NAME);
 	new_item->given_name = e_contact_get (contact, E_CONTACT_GIVEN_NAME);
-	new_item->surname    = e_contact_get (contact, E_CONTACT_FAMILY_NAME);
-	new_item->file_as    = e_contact_get (contact, E_CONTACT_FILE_AS);
-	new_item->email_1    = e_contact_get (contact, E_CONTACT_EMAIL_1);
-	new_item->email_2    = e_contact_get (contact, E_CONTACT_EMAIL_2);
-	new_item->email_3    = e_contact_get (contact, E_CONTACT_EMAIL_3);
-	new_item->email_4    = e_contact_get (contact, E_CONTACT_EMAIL_4);
-	new_item->list       = GPOINTER_TO_INT (e_contact_get (contact, E_CONTACT_IS_LIST));
+	new_item->surname = e_contact_get (contact, E_CONTACT_FAMILY_NAME);
+	new_item->file_as = e_contact_get (contact, E_CONTACT_FILE_AS);
+	new_item->email_1 = e_contact_get (contact, E_CONTACT_EMAIL_1);
+	new_item->email_2 = e_contact_get (contact, E_CONTACT_EMAIL_2);
+	new_item->email_3 = e_contact_get (contact, E_CONTACT_EMAIL_3);
+	new_item->email_4 = e_contact_get (contact, E_CONTACT_EMAIL_4);
+	new_item->list = GPOINTER_TO_INT (e_contact_get (contact, E_CONTACT_IS_LIST));
 	new_item->list_show_addresses = GPOINTER_TO_INT (e_contact_get (contact, E_CONTACT_LIST_SHOW_ADDRESSES));
 	new_item->wants_html = GPOINTER_TO_INT (e_contact_get (contact, E_CONTACT_WANTS_HTML));
 
@@ -788,6 +805,8 @@ e_book_backend_summary_add_contact (EBookBackendSummary *summary,
  * @id: a unique contact ID string
  *
  * Removes the summary of the contact identified by @id from @summary.
+ *
+ * Deprecated: 3.12: Use #EBookSqlite instead
  **/
 void
 e_book_backend_summary_remove_contact (EBookBackendSummary *summary,
@@ -819,6 +838,8 @@ e_book_backend_summary_remove_contact (EBookBackendSummary *summary,
  * exists in @summary.
  *
  * Returns: %TRUE if the summary exists, %FALSE otherwise.
+ *
+ * Deprecated: 3.12: Use #EBookSqlite instead
  **/
 gboolean
 e_book_backend_summary_check_contact (EBookBackendSummary *summary,
@@ -860,6 +881,8 @@ summary_flush_func (gpointer data)
  * @summary: an #EBookBackendSummary
  *
  * Indicates that @summary has changed and should be flushed to disk.
+ *
+ * Deprecated: 3.12: Use #EBookSqlite instead
  **/
 void
 e_book_backend_summary_touch (EBookBackendSummary *summary)
@@ -883,6 +906,8 @@ e_book_backend_summary_touch (EBookBackendSummary *summary)
  * Checks if @summary is more recent than @t.
  *
  * Returns: %TRUE if the summary is up to date, %FALSE otherwise.
+ *
+ * Deprecated: 3.12: Use #EBookSqlite instead
  **/
 gboolean
 e_book_backend_summary_is_up_to_date (EBookBackendSummary *summary,
@@ -955,6 +980,8 @@ static const struct {
  * stored by @summary.
  *
  * Returns: %TRUE if the query can be satisfied, %FALSE otherwise.
+ *
+ * Deprecated: 3.12: Use #EBookSqlite instead
  **/
 gboolean
 e_book_backend_summary_is_summary_query (EBookBackendSummary *summary,
@@ -1201,6 +1228,8 @@ static const struct {
  * Searches @summary for contacts matching @query.
  *
  * Returns: A #GPtrArray of pointers to contact ID strings.
+ *
+ * Deprecated: 3.12: Use #EBookSqlite instead
  **/
 GPtrArray *
 e_book_backend_summary_search (EBookBackendSummary *summary,
@@ -1261,6 +1290,8 @@ e_book_backend_summary_search (EBookBackendSummary *summary,
  * by @id.
  *
  * Returns: A new VCard, or %NULL if the contact summary didn't exist.
+ *
+ * Deprecated: 3.12: Use #EBookSqlite instead
  **/
 gchar *
 e_book_backend_summary_get_summary_vcard (EBookBackendSummary *summary,
