@@ -16,6 +16,15 @@
  *
  */
 
+/**
+ * SECTION: e-data-book
+ * @include: libedata-book/libedata-book.h
+ * @short_description: Server side D-Bus layer to communicate with addressbooks
+ *
+ * This class communicates with #EBookClients over the bus and accesses
+ * an #EBookBackend to satisfy client requests.
+ **/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -37,7 +46,6 @@
 #include "e-book-backend.h"
 #include "e-book-backend-sexp.h"
 #include "e-book-backend-factory.h"
-#include "e-dbus-localed.h"
 
 #define E_DATA_BOOK_GET_PRIVATE(obj) \
 	(G_TYPE_INSTANCE_GET_PRIVATE \
@@ -56,10 +64,6 @@ struct _EDataBookPrivate {
 
 	GMutex sender_lock;
 	GHashTable *sender_table;
-
-	guint localed_watch_id;
-	EDBusLocale1 *localed_proxy;
-	GCancellable *localed_cancel;
 };
 
 struct _AsyncContext {
@@ -444,6 +448,13 @@ e_data_book_error_quark (void)
 
 /**
  * e_data_book_create_error:
+ * @status: #EDataBookStatus code
+ * @custom_msg: Custom message to use for the error. When NULL,
+ *              then uses a default message based on the @status code.
+ *
+ * Returns: NULL, when the @status is E_DATA_BOOK_STATUS_SUCCESS,
+ *          or a newly allocated GError, which should be freed
+ *          with g_error_free() call.
  *
  * Since: 2.32
  **/
@@ -460,6 +471,9 @@ e_data_book_create_error (EDataBookStatus status,
 /**
  * e_data_book_create_error_fmt:
  *
+ * Similar as e_data_book_create_error(), only here, instead of custom_msg,
+ * is used a printf() format to create a custom_msg for the error.
+ *
  * Since: 2.32
  **/
 GError *
@@ -849,6 +863,7 @@ data_book_complete_create_contacts_cb (GObject *source_object,
 			contact = g_queue_pop_head (&queue);
 			uid = e_contact_get_const (contact, E_CONTACT_UID);
 			strv[ii++] = e_util_utf8_make_valid (uid);
+			g_object_unref (contact);
 		}
 
 		e_dbus_address_book_complete_create_contacts (
@@ -1049,11 +1064,11 @@ data_book_handle_get_view_cb (EDBusAddressBook *interface,
 
 static gboolean
 data_book_interpret_sort_keys (const gchar * const *in_sort_keys,
-			       const gchar * const *in_sort_types,
-			       EContactField **out_sort_keys,
-			       EBookCursorSortType **out_sort_types,
-			       gint *n_fields,
-			       GError **error)
+                               const gchar * const *in_sort_types,
+                               EContactField **out_sort_keys,
+                               EBookCursorSortType **out_sort_types,
+                               gint *n_fields,
+                               GError **error)
 {
 	gint i, key_count = 0, type_count = 0;
 	EContactField *sort_keys;
@@ -1061,10 +1076,11 @@ data_book_interpret_sort_keys (const gchar * const *in_sort_keys,
 	gboolean success = TRUE;
 
 	if (!in_sort_keys || !in_sort_types) {
-		g_set_error (error,
-			     E_CLIENT_ERROR,
-			     E_CLIENT_ERROR_INVALID_ARG,
-			     "Missing sort keys while trying to create a Cursor");
+		g_set_error (
+			error,
+			E_CLIENT_ERROR,
+			E_CLIENT_ERROR_INVALID_ARG,
+			"Missing sort keys while trying to create a Cursor");
 		return FALSE;
 	}
 
@@ -1074,10 +1090,11 @@ data_book_interpret_sort_keys (const gchar * const *in_sort_keys,
 		type_count++;
 
 	if (key_count != type_count) {
-		g_set_error (error,
-			     E_CLIENT_ERROR,
-			     E_CLIENT_ERROR_INVALID_ARG,
-			     "Must specify the same amount of sort keys as sort types while creating a Cursor");
+		g_set_error (
+			error,
+			E_CLIENT_ERROR,
+			E_CLIENT_ERROR_INVALID_ARG,
+			"Must specify the same amount of sort keys as sort types while creating a Cursor");
 		return FALSE;
 	}
 
@@ -1089,11 +1106,12 @@ data_book_interpret_sort_keys (const gchar * const *in_sort_keys,
 		sort_keys[i] = e_contact_field_id (in_sort_keys[i]);
 
 		if (sort_keys[i] == 0) {
-			g_set_error (error,
-				     E_CLIENT_ERROR,
-				     E_CLIENT_ERROR_INVALID_ARG,
-				     "Invalid sort key '%s' specified when creating a Cursor",
-				     in_sort_keys[i]);
+			g_set_error (
+				error,
+				E_CLIENT_ERROR,
+				E_CLIENT_ERROR_INVALID_ARG,
+				"Invalid sort key '%s' specified when creating a Cursor",
+				in_sort_keys[i]);
 			success = FALSE;
 		}
 	}
@@ -1104,11 +1122,12 @@ data_book_interpret_sort_keys (const gchar * const *in_sort_keys,
 		if (!e_enum_from_string (E_TYPE_BOOK_CURSOR_SORT_TYPE,
 					 in_sort_types[i],
 					 &enum_value)) {
-			g_set_error (error,
-				     E_CLIENT_ERROR,
-				     E_CLIENT_ERROR_INVALID_ARG,
-				     "Invalid sort type '%s' specified when creating a Cursor",
-				     in_sort_types[i]);
+			g_set_error (
+				error,
+				E_CLIENT_ERROR,
+				E_CLIENT_ERROR_INVALID_ARG,
+				"Invalid sort type '%s' specified when creating a Cursor",
+				in_sort_types[i]);
 			success = FALSE;
 		}
 
@@ -1129,11 +1148,11 @@ data_book_interpret_sort_keys (const gchar * const *in_sort_keys,
 
 static gboolean
 data_book_handle_get_cursor_cb (EDBusAddressBook *interface,
-				GDBusMethodInvocation *invocation,
-				const gchar *in_query,
-				const gchar * const *in_sort_keys,
-				const gchar * const *in_sort_types,
-				EDataBook *data_book)
+                                GDBusMethodInvocation *invocation,
+                                const gchar *in_query,
+                                const gchar * const *in_sort_keys,
+                                const gchar * const *in_sort_types,
+                                EDataBook *data_book)
 {
 	EBookBackend *backend;
 	EDataBookCursor *cursor;
@@ -1164,11 +1183,8 @@ data_book_handle_get_cursor_cb (EDBusAddressBook *interface,
 	/*
 	 * Create cursor
 	 */
-	cursor = e_book_backend_create_cursor (backend,
-					       sort_keys,
-					       sort_types,
-					       n_fields,
-					       &error);
+	cursor = e_book_backend_create_cursor (
+		backend, sort_keys, sort_types, n_fields, &error);
 	g_free (sort_keys);
 	g_free (sort_types);
 
@@ -1181,7 +1197,7 @@ data_book_handle_get_cursor_cb (EDBusAddressBook *interface,
 	/*
 	 * Set the query, if any (no query is allowed)
 	 */
-	if (!e_data_book_cursor_set_sexp (cursor, in_query, &error)) {
+	if (!e_data_book_cursor_set_sexp (cursor, in_query, NULL, &error)) {
 
 		e_book_backend_delete_cursor (backend, cursor, NULL);
 		g_dbus_method_invocation_take_error (invocation, error);
@@ -1206,9 +1222,8 @@ data_book_handle_get_cursor_cb (EDBusAddressBook *interface,
 	/*
 	 * All is good in the hood, complete the method call
 	 */
-	e_dbus_address_book_complete_get_cursor (interface,
-						 invocation,
-						 object_path);
+	e_dbus_address_book_complete_get_cursor (
+		interface, invocation, object_path);
 	g_free (object_path);
 	g_object_unref (backend);
 	return TRUE;
@@ -1643,156 +1658,6 @@ e_data_book_report_backend_property_changed (EDataBook *book,
 	/* Disregard anything else. */
 }
 
-static gchar *
-data_book_interpret_locale_value (const gchar *value)
-{
-	gchar *interpreted_value = NULL;
-	gchar **split;
-
-	split = g_strsplit (value, "=", 2);
-
-	if (split && split[0] && split[1])
-		interpreted_value = g_strdup (split[1]);
-
-	g_strfreev (split);
-
-	if (!interpreted_value)
-		g_warning ("Failed to interpret locale value: %s", value);
-
-	return interpreted_value;
-}
-
-static gchar *
-data_book_interpret_locale (const gchar * const * locale)
-{
-	gint i;
-	gchar *interpreted_locale = NULL;
-
-	/* Prioritize LC_COLLATE and then LANG values
-	 * in the 'locale' specified by localed.
-	 *
-	 * If localed explicitly specifies no locale, then
-	 * default to checking system locale.
-	 */
-	if (locale) {
-
-		for (i = 0; locale[i] != NULL && interpreted_locale == NULL; i++) {
-
-			if (strncmp (locale[i], "LC_COLLATE", 10) == 0)
-				interpreted_locale = data_book_interpret_locale_value (locale[i]);
-		}
-
-		for (i = 0; locale[i] != NULL && interpreted_locale == NULL; i++) {
-
-			if (strncmp (locale[i], "LANG", 4) == 0)
-				interpreted_locale = data_book_interpret_locale_value (locale[i]);
-		}
-	}
-
-	if (!interpreted_locale) {
-		const gchar *system_locale = setlocale (LC_COLLATE, NULL);
-
-		interpreted_locale = g_strdup (system_locale);
-	}
-
-	return interpreted_locale;
-}
-
-static void
-data_book_locale_changed (GObject *object,
-			  GParamSpec *pspec,
-			  gpointer user_data)
-{
-	EDBusLocale1 *locale_proxy = E_DBUS_LOCALE1 (object);
-	EDataBook *book = (EDataBook *)user_data;
-	EBookBackend *backend;
-
-	backend = e_data_book_ref_backend (book);
-
-	if (backend) {
-		const gchar * const *locale;
-		gchar *interpreted_locale;
-
-		locale = e_dbus_locale1_get_locale (locale_proxy);
-		interpreted_locale = data_book_interpret_locale (locale);
-
-		e_book_backend_set_locale (backend, interpreted_locale);
-
-		e_dbus_address_book_set_locale (book->priv->dbus_interface, interpreted_locale);
-
-		g_free (interpreted_locale);
-	}
-
-	g_object_unref (backend);
-}
-
-static void
-data_book_localed_ready (GObject *source_object,
-			 GAsyncResult *res,
-			 gpointer user_data)
-{
-	EDataBook *book = (EDataBook *)user_data;
-	GError *error = NULL;
-
-	book->priv->localed_proxy = e_dbus_locale1_proxy_new_finish (res, &error);
-
-	if (book->priv->localed_proxy == NULL) {
-		g_warning ("Error fetching localed proxy: %s", error->message);
-		g_error_free (error);
-	}
-
-	if (book->priv->localed_cancel) {
-		g_object_unref (book->priv->localed_cancel);
-		book->priv->localed_cancel = NULL;
-	}
-
-	if (book->priv->localed_proxy) {
-		g_signal_connect (book->priv->localed_proxy, "notify::locale",
-				  G_CALLBACK (data_book_locale_changed), book);
-
-		/* Initial refresh of the locale */
-		data_book_locale_changed (G_OBJECT (book->priv->localed_proxy), NULL, book);
-	}
-}
-
-static void
-data_book_localed_appeared (GDBusConnection *connection,
-			    const gchar *name,
-			    const gchar *name_owner,
-			    gpointer user_data)
-{
-	EDataBook *book = (EDataBook *)user_data;
-
-	book->priv->localed_cancel = g_cancellable_new ();
-
-	e_dbus_locale1_proxy_new (connection,
-				  G_DBUS_PROXY_FLAGS_GET_INVALIDATED_PROPERTIES,
-				  "org.freedesktop.locale1",
-				  "/org/freedesktop/locale1",
-				  book->priv->localed_cancel,
-				  data_book_localed_ready,
-				  book);
-}
-
-static void
-data_book_localed_vanished (GDBusConnection *connection,
-			    const gchar *name,
-			    gpointer user_data)
-{
-	EDataBook *book = (EDataBook *)user_data;
-
-	if (book->priv->localed_cancel) {
-		g_cancellable_cancel (book->priv->localed_cancel);
-		g_object_unref (book->priv->localed_cancel);
-		book->priv->localed_cancel = NULL;
-	}
-
-	if (book->priv->localed_proxy) {
-		g_object_unref (book->priv->localed_proxy);
-		book->priv->localed_proxy = NULL;
-	}
-}
-
 static void
 data_book_set_backend (EDataBook *book,
                        EBookBackend *backend)
@@ -1908,17 +1773,6 @@ data_book_dispose (GObject *object)
 		priv->direct_module = NULL;
 	}
 
-	if (priv->localed_cancel) {
-		g_cancellable_cancel (priv->localed_cancel);
-		g_object_unref (priv->localed_cancel);
-		priv->localed_cancel = NULL;
-	}
-
-	if (priv->localed_proxy) {
-		g_object_unref (priv->localed_proxy);
-		priv->localed_proxy = NULL;
-	}
-
 	g_hash_table_remove_all (priv->sender_table);
 
 	/* Chain up to parent's dispose() metnod. */
@@ -1942,9 +1796,6 @@ data_book_finalize (GObject *object)
 		priv->dbus_interface = NULL;
 	}
 
-	if (priv->localed_watch_id > 0)
-		g_bus_unwatch_name (priv->localed_watch_id);
-
 	/* Chain up to parent's finalize() method. */
 	G_OBJECT_CLASS (e_data_book_parent_class)->finalize (object);
 }
@@ -2010,8 +1861,9 @@ data_book_constructed (GObject *object)
 	/* Initialize the locale to the value reported by setlocale() until
 	 * systemd says otherwise.
 	 */
-	e_dbus_address_book_set_locale (book->priv->dbus_interface,
-					setlocale (LC_COLLATE, NULL));
+	e_dbus_address_book_set_locale (
+		book->priv->dbus_interface,
+		setlocale (LC_COLLATE, NULL));
 
 	g_object_unref (backend);
 }
@@ -2024,7 +1876,6 @@ data_book_initable_init (GInitable *initable,
 	EBookBackend *backend;
 	EDataBook *book;
 	gchar *locale;
-	GBusType bus_type = G_BUS_TYPE_SYSTEM;
 
 	book = E_DATA_BOOK (initable);
 
@@ -2061,22 +1912,6 @@ data_book_initable_init (GInitable *initable,
 	e_dbus_address_book_set_locale (book->priv->dbus_interface, locale);
 	g_free (locale);
 
-	/* When running tests, we pretend to be the "org.freedesktop.locale1" service
-	 * on the session bus instead of the real location on the system bus.
-	 */
-	if (g_getenv ("EDS_TESTING") != NULL)
-		bus_type = G_BUS_TYPE_SESSION;
-
-	/* Watch system bus for locale change notifications */
-	book->priv->localed_watch_id =
-		g_bus_watch_name (bus_type,
-				  "org.freedesktop.locale1",
-				  G_BUS_NAME_WATCHER_FLAGS_NONE,
-				  data_book_localed_appeared,
-				  data_book_localed_vanished,
-				  book,
-				  NULL);
-
 	return g_dbus_interface_skeleton_export (
 		G_DBUS_INTERFACE_SKELETON (book->priv->dbus_interface),
 		book->priv->connection,
@@ -2299,3 +2134,40 @@ e_data_book_get_object_path (EDataBook *book)
 	return book->priv->object_path;
 }
 
+/**
+ * e_data_book_set_locale:
+ * @book: an #EDataBook 
+ * @locale: the new locale to set for this book
+ * @cancellable: (allow-none): a #GCancellable
+ * @error: (allow-none): a location to store any error which might occur
+ *
+ * Set's the locale for this addressbook, this can result in renormalization of
+ * locale sensitive data.
+ *
+ * Returns: %TRUE on success, otherwise %FALSE is returned and @error is set appropriately.
+ *
+ * Since: 3.12
+ */
+gboolean
+e_data_book_set_locale (EDataBook *book,
+                        const gchar *locale,
+                        GCancellable *cancellable,
+                        GError **error)
+{
+	EBookBackend *backend;
+	gboolean success;
+
+	g_return_val_if_fail (E_IS_DATA_BOOK (book), FALSE);
+
+	backend = e_data_book_ref_backend (book);
+	success = e_book_backend_set_locale (
+		backend, locale, cancellable, error);
+
+	if (success)
+		e_dbus_address_book_set_locale (
+			book->priv->dbus_interface, locale);
+
+	g_object_unref (backend);
+
+	return success;
+}
