@@ -70,25 +70,9 @@ GQuark e_data_book_error_quark (void);
  **/
 #define E_DATA_BOOK_ERROR e_data_book_error_quark ()
 
-/**
- * e_data_book_create_error:
- * @status: #EDataBookStatus code
- * @custom_msg: Custom message to use for the error. When NULL,
- *              then uses a default message based on the @status code.
- *
- * Returns: NULL, when the @status is E_DATA_BOOK_STATUS_SUCCESS,
- *          or a newly allocated GError, which should be freed
- *          with g_error_free() call.
- **/
 GError *	e_data_book_create_error	(EDataBookStatus status,
 						 const gchar *custom_msg);
 
-/**
- * e_data_book_create_error_fmt:
- *
- * Similar as e_data_book_create_error(), only here, instead of custom_msg,
- * is used a printf() format to create a custom_msg for the error.
- **/
 GError *	e_data_book_create_error_fmt	(EDataBookStatus status,
 						 const gchar *custom_msg_fmt,
 						 ...) G_GNUC_PRINTF (2, 3);
@@ -106,6 +90,10 @@ GDBusConnection *
 		e_data_book_get_connection	(EDataBook *book);
 const gchar *	e_data_book_get_object_path	(EDataBook *book);
 
+gboolean	e_data_book_set_locale		(EDataBook *book,
+						 const gchar *locale,
+						 GCancellable *cancellable,
+						 GError **error);
 void		e_data_book_respond_open	(EDataBook *book,
 						 guint32 opid,
 						 GError *error);
