@@ -46,6 +46,7 @@ static gboolean	begin_retrieval_cb		(ECalBackendWeather *cbw);
 static ECalComponent *
 		create_weather			(ECalBackendWeather *cbw,
 						 GWeatherInfo *report,
+						 GWeatherTemperatureUnit unit,
 						 gboolean is_forecast);
 static void	e_cal_backend_weather_add_timezone
 						(ECalBackendSync *backend,
@@ -69,9 +70,6 @@ struct _ECalBackendWeatherPrivate {
 	/* Flags */
 	gboolean opened;
 
-	/* City (for summary) */
-	gchar *city;
-
 	/* Weather source */
 	EWeatherSource *source;
 
@@ -167,6 +165,9 @@ finished_retrieval_cb (GWeatherInfo *info,
 	ECalBackendWeatherPrivate *priv;
 	ECalComponent *comp;
 	GSList *comps, *l;
+	GWeatherTemperatureUnit unit;
+	ESource *source;
+	ESourceWeather *weather_extension;
 
 	priv = cbw->priv;
 
@@ -175,6 +176,16 @@ finished_retrieval_cb (GWeatherInfo *info,
 		return;
 	}
 
+	source = e_backend_get_source (E_BACKEND (cbw));
+	weather_extension = e_source_get_extension (source, E_SOURCE_EXTENSION_WEATHER_BACKEND);
+
+	if (e_source_weather_get_units (weather_extension) == E_SOURCE_WEATHER_UNITS_CENTIGRADE)
+		unit = GWEATHER_TEMP_UNIT_CENTIGRADE;
+	else if (e_source_weather_get_units (weather_extension) == E_SOURCE_WEATHER_UNITS_KELVIN)
+		unit = GWEATHER_TEMP_UNIT_KELVIN;
+	else /* E_SOURCE_WEATHER_UNITS_FAHRENHEIT */
+		unit = GWEATHER_TEMP_UNIT_FAHRENHEIT;
+
 	/* update cache */
 	comps = e_cal_backend_store_get_components (priv->store);
 
@@ -192,7 +203,7 @@ finished_retrieval_cb (GWeatherInfo *info,
 	g_slist_free (comps);
 	e_cal_backend_store_clean (priv->store);
 
-	comp = create_weather (cbw, info, FALSE);
+	comp = create_weather (cbw, info, unit, FALSE);
 	if (comp) {
 		GSList *forecasts;
 
@@ -209,7 +220,7 @@ finished_retrieval_cb (GWeatherInfo *info,
 				GWeatherInfo *nfo = f->data;
 
 				if (nfo) {
-					comp = create_weather (cbw, nfo, TRUE);
+					comp = create_weather (cbw, nfo, unit, TRUE);
 					if (comp) {
 						put_component_to_store (cbw, comp);
 						e_cal_backend_notify_component_created (E_CAL_BACKEND (cbw), comp);
@@ -309,12 +320,34 @@ getCategory (GWeatherInfo *report)
 	return NULL;
 }
 
+static gchar *
+cal_backend_weather_get_temp (gdouble value,
+                              GWeatherTemperatureUnit unit)
+{
+	switch (unit) {
+	case GWEATHER_TEMP_UNIT_FAHRENHEIT:
+		/* TRANSLATOR: This is the temperature in degrees Fahrenheit (\302\260 is U+00B0 DEGREE SIGN) */
+		return g_strdup_printf (_("%.1f \302\260F"), value);
+	case GWEATHER_TEMP_UNIT_CENTIGRADE:
+		/* TRANSLATOR: This is the temperature in degrees Celsius (\302\260 is U+00B0 DEGREE SIGN) */
+		return g_strdup_printf (_("%.1f \302\260C"), value);
+	case GWEATHER_TEMP_UNIT_KELVIN:
+		/* TRANSLATOR: This is the temperature in kelvin */
+		return g_strdup_printf (_("%.1f K"), value);
+	default:
+		g_warn_if_reached ();
+		break;
+	}
+
+	return g_strdup_printf (_("%.1f"), value);
+}
+
 static ECalComponent *
 create_weather (ECalBackendWeather *cbw,
                 GWeatherInfo *report,
+                GWeatherTemperatureUnit unit,
                 gboolean is_forecast)
 {
-	ECalBackendWeatherPrivate *priv;
 	ECalComponent             *cal_comp;
 	ECalComponentText          comp_summary;
 	icalcomponent             *ical_comp;
@@ -323,19 +356,18 @@ create_weather (ECalBackendWeather *cbw,
 	gchar			  *uid;
 	GSList                    *text_list = NULL;
 	ECalComponentText         *description;
-	gchar                     *tmp;
+	gchar                     *tmp, *city_name;
 	time_t			   update_time;
 	icaltimezone		  *update_zone = NULL;
 	const GWeatherLocation    *location;
 	const GWeatherTimezone    *w_timezone;
+	gdouble tmin = 0.0, tmax = 0.0, temp = 0.0;
 
 	g_return_val_if_fail (E_IS_CAL_BACKEND_WEATHER (cbw), NULL);
 
 	if (!gweather_info_get_value_update (report, &update_time))
 		return NULL;
 
-	priv = cbw->priv;
-
 	/* create the component and event object */
 	ical_comp = icalcomponent_new (ICAL_VEVENT_COMPONENT);
 	cal_comp = e_cal_component_new ();
@@ -374,28 +406,32 @@ create_weather (ECalBackendWeather *cbw,
 	/* We have to add 1 day to DTEND, as it is not inclusive. */
 	e_cal_component_set_dtend (cal_comp, &dt);
 
-	{
-		gdouble tmin = 0.0, tmax = 0.0;
+	city_name = gweather_info_get_location_name (report);
+	if (gweather_info_get_value_temp_min (report, unit, &tmin) &&
+	    gweather_info_get_value_temp_max (report, unit, &tmax) &&
+	    tmin != tmax) {
+		gchar *min, *max;
 
-		if (gweather_info_get_value_temp_min (report, GWEATHER_TEMP_UNIT_DEFAULT, &tmin) &&
-		    gweather_info_get_value_temp_max (report, GWEATHER_TEMP_UNIT_DEFAULT, &tmax) &&
-		    tmin != tmax) {
-			gchar *min, *max;
+		min = cal_backend_weather_get_temp (tmin, unit);
+		max = cal_backend_weather_get_temp (tmax, unit);
+		comp_summary.value = g_strdup_printf ("%s : %s / %s", city_name, min, max);
 
-			min = gweather_info_get_temp_min (report);
-			max = gweather_info_get_temp_max (report);
-			comp_summary.value = g_strdup_printf ("%s : %s / %s", priv->city, min, max);
+		g_free (min); g_free (max);
+	} else if (gweather_info_get_value_temp (report, unit, &temp)) {
+		tmp = cal_backend_weather_get_temp (temp, unit);
+		comp_summary.value = g_strdup_printf ("%s : %s", city_name, tmp);
 
-			g_free (min); g_free (max);
-		} else {
-			gchar *temp;
+		g_free (tmp);
+	} else {
+		gchar *temp;
 
-			temp = gweather_info_get_temp (report);
-			comp_summary.value = g_strdup_printf ("%s : %s", priv->city, temp);
+		temp = gweather_info_get_temp (report);
+		comp_summary.value = g_strdup_printf ("%s : %s", city_name, temp);
 
-			g_free (temp);
-		}
+		g_free (temp);
 	}
+	g_free (city_name);
+
 	comp_summary.altrep = NULL;
 	e_cal_component_set_summary (cal_comp, &comp_summary);
 	g_free ((gchar *) comp_summary.value);
@@ -483,27 +519,13 @@ e_cal_backend_weather_open (ECalBackendSync *backend,
 {
 	ECalBackendWeather *cbw;
 	ECalBackendWeatherPrivate *priv;
-	ESource *source;
-	ESourceWeather *extension;
-	const gchar *extension_name;
 	const gchar *cache_dir;
-	gchar *location;
 
 	cbw = E_CAL_BACKEND_WEATHER (backend);
 	priv = cbw->priv;
 
-	source = e_backend_get_source (E_BACKEND (backend));
 	cache_dir = e_cal_backend_get_cache_dir (E_CAL_BACKEND (backend));
 
-	extension_name = E_SOURCE_EXTENSION_WEATHER_BACKEND;
-	extension = e_source_get_extension (source, extension_name);
-
-	g_free (priv->city);
-
-	location = e_source_weather_dup_location (extension);
-	priv->city = g_strdup (strrchr (location, '/') + 1);
-	g_free (location);
-
 	e_cal_backend_set_writable (E_CAL_BACKEND (backend), FALSE);
 
 	if (!priv->store) {
@@ -778,8 +800,6 @@ e_cal_backend_weather_finalize (GObject *object)
 		priv->store = NULL;
 	}
 
-	g_free (priv->city);
-
 	/* Chain up to parent's finalize() method. */
 	G_OBJECT_CLASS (e_cal_backend_weather_parent_class)->finalize (object);
 }
@@ -816,15 +836,15 @@ e_cal_backend_weather_class_init (ECalBackendWeatherClass *class)
 
 	backend_class->get_backend_property = e_cal_backend_weather_get_backend_property;
 
-	sync_class->open_sync			= e_cal_backend_weather_open;
-	sync_class->refresh_sync		= e_cal_backend_weather_refresh;
-	sync_class->receive_objects_sync	= e_cal_backend_weather_receive_objects;
-	sync_class->get_object_sync		= e_cal_backend_weather_get_object;
-	sync_class->get_object_list_sync	= e_cal_backend_weather_get_object_list;
-	sync_class->add_timezone_sync		= e_cal_backend_weather_add_timezone;
-	sync_class->get_free_busy_sync		= e_cal_backend_weather_get_free_busy;
+	sync_class->open_sync = e_cal_backend_weather_open;
+	sync_class->refresh_sync = e_cal_backend_weather_refresh;
+	sync_class->receive_objects_sync = e_cal_backend_weather_receive_objects;
+	sync_class->get_object_sync = e_cal_backend_weather_get_object;
+	sync_class->get_object_list_sync = e_cal_backend_weather_get_object_list;
+	sync_class->add_timezone_sync = e_cal_backend_weather_add_timezone;
+	sync_class->get_free_busy_sync = e_cal_backend_weather_get_free_busy;
 
-	backend_class->start_view		= e_cal_backend_weather_start_view;
+	backend_class->start_view = e_cal_backend_weather_start_view;
 
 	/* Register our ESource extension. */
 	E_TYPE_SOURCE_WEATHER;
