@@ -23,6 +23,16 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+/**
+ * SECTION: e-data-cal-factory
+ * @include: libedata-cal/libedata-cal.h
+ * @short_description: The main calendar server object
+ *
+ * This class handles incomming D-Bus connections and creates
+ * the #EDataCal layer for server side calendars to communicate
+ * with client side #ECalClient objects.
+ **/
+
 #include <config.h>
 #include <locale.h>
 #include <stdlib.h>
@@ -398,8 +408,6 @@ data_cal_factory_open (EDataCalFactory *factory,
 		 * So we must track each and every connection made. */
 		data_cal_factory_connections_add (
 			factory, sender, E_CAL_BACKEND (backend));
-
-		g_object_unref (data_cal);
 	}
 
 	g_object_unref (backend);
