@@ -16,6 +16,15 @@
  *
  */
 
+/**
+ * SECTION: e-data-cal
+ * @include: libedata-cal/libedata-cal.h
+ * @short_description: Server side D-Bus layer to communicate with calendars
+ *
+ * This class communicates with #ECalClients over the bus and accesses
+ * an #ECalBackend to satisfy client requests.
+ **/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -163,7 +172,7 @@ async_context_new (EDataCal *data_cal,
 		(GBusNameAppearedCallback) NULL,
 		(GBusNameVanishedCallback) sender_vanished_cb,
 		g_object_ref (async_context->cancellable),
-		(GDestroyNotify) g_object_unref);
+		(GDestroyNotify) g_object_ref);
 
 	sender_table_insert (
 		async_context->data_cal,
