@@ -377,9 +377,10 @@ camel_certdb_load (CamelCertDB *certdb)
 	if (!class->header_load || !class->cert_load) {
 		fclose (in);
 		in = NULL;
+		g_warn_if_reached ();
+
+		return -1;
 	}
-	g_return_val_if_fail (class->header_load != NULL, -1);
-	g_return_val_if_fail (class->cert_load != NULL, -1);
 
 	g_mutex_lock (&certdb->priv->io_lock);
 	if (class->header_load (certdb, in) == -1)
@@ -426,6 +427,7 @@ camel_certdb_save (CamelCertDB *certdb)
 	CamelCertDBClass *class;
 	CamelCert *cert;
 	gchar *filename;
+	gsize filename_len;
 	gint fd, i;
 	FILE *out;
 
@@ -436,8 +438,9 @@ camel_certdb_save (CamelCertDB *certdb)
 	if (!certdb->priv->dirty)
 		return 0;
 
-	filename = alloca (strlen (certdb->priv->filename) + 4);
-	sprintf (filename, "%s~", certdb->priv->filename);
+	filename_len = strlen (certdb->priv->filename) + 4;
+	filename = alloca (filename_len);
+	g_snprintf (filename, filename_len, "%s~", certdb->priv->filename);
 
 	fd = g_open (filename, O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, 0600);
 	if (fd == -1)
@@ -456,9 +459,10 @@ camel_certdb_save (CamelCertDB *certdb)
 	if (!class->header_save || !class->cert_save) {
 		fclose (out);
 		out = NULL;
+		g_warn_if_reached ();
+
+		return -1;
 	}
-	g_return_val_if_fail (class->header_save != NULL, -1);
-	g_return_val_if_fail (class->cert_save != NULL, -1);
 
 	g_mutex_lock (&certdb->priv->io_lock);
 
