@@ -69,9 +69,6 @@ struct _AsyncContext {
 	CamelStream *stream;
 	GPtrArray *strings;
 	gchar *userid;
-
-	/* results */
-	CamelCipherValidity *validity;
 };
 
 enum {
@@ -99,9 +96,6 @@ async_context_free (AsyncContext *async_context)
 		g_ptr_array_free (async_context->strings, TRUE);
 	}
 
-	if (async_context->validity != NULL)
-		camel_cipher_validity_free (async_context->validity);
-
 	g_free (async_context->userid);
 
 	g_slice_free (AsyncContext, async_context);
@@ -253,304 +247,6 @@ cipher_context_decrypt_sync (CamelCipherContext *context,
 	return NULL;
 }
 
-static void
-cipher_context_sign_thread (GSimpleAsyncResult *simple,
-                            GObject *object,
-                            GCancellable *cancellable)
-{
-	AsyncContext *async_context;
-	GError *error = NULL;
-
-	async_context = g_simple_async_result_get_op_res_gpointer (simple);
-
-	camel_cipher_context_sign_sync (
-		CAMEL_CIPHER_CONTEXT (object),
-		async_context->userid, async_context->hash,
-		async_context->ipart, async_context->opart,
-		cancellable, &error);
-
-	if (error != NULL)
-		g_simple_async_result_take_error (simple, error);
-}
-
-static void
-cipher_context_sign (CamelCipherContext *context,
-                     const gchar *userid,
-                     CamelCipherHash hash,
-                     CamelMimePart *ipart,
-                     CamelMimePart *opart,
-                     gint io_priority,
-                     GCancellable *cancellable,
-                     GAsyncReadyCallback callback,
-                     gpointer user_data)
-{
-	GSimpleAsyncResult *simple;
-	AsyncContext *async_context;
-
-	async_context = g_slice_new0 (AsyncContext);
-	async_context->userid = g_strdup (userid);
-	async_context->hash = hash;
-	async_context->ipart = g_object_ref (ipart);
-	async_context->opart = g_object_ref (opart);
-
-	simple = g_simple_async_result_new (
-		G_OBJECT (context), callback, user_data, cipher_context_sign);
-
-	g_simple_async_result_set_check_cancellable (simple, cancellable);
-
-	g_simple_async_result_set_op_res_gpointer (
-		simple, async_context, (GDestroyNotify) async_context_free);
-
-	g_simple_async_result_run_in_thread (
-		simple, cipher_context_sign_thread, io_priority, cancellable);
-
-	g_object_unref (simple);
-}
-
-static gboolean
-cipher_context_sign_finish (CamelCipherContext *context,
-                            GAsyncResult *result,
-                            GError **error)
-{
-	GSimpleAsyncResult *simple;
-
-	g_return_val_if_fail (
-		g_simple_async_result_is_valid (
-		result, G_OBJECT (context), cipher_context_sign), FALSE);
-
-	simple = G_SIMPLE_ASYNC_RESULT (result);
-
-	/* Assume success unless a GError is set. */
-	return !g_simple_async_result_propagate_error (simple, error);
-}
-
-static void
-cipher_context_verify_thread (GSimpleAsyncResult *simple,
-                              GObject *object,
-                              GCancellable *cancellable)
-{
-	AsyncContext *async_context;
-	GError *error = NULL;
-
-	async_context = g_simple_async_result_get_op_res_gpointer (simple);
-
-	async_context->validity = camel_cipher_context_verify_sync (
-		CAMEL_CIPHER_CONTEXT (object), async_context->ipart,
-		cancellable, &error);
-
-	if (error != NULL)
-		g_simple_async_result_take_error (simple, error);
-}
-
-static void
-cipher_context_verify (CamelCipherContext *context,
-                       CamelMimePart *ipart,
-                       gint io_priority,
-                       GCancellable *cancellable,
-                       GAsyncReadyCallback callback,
-                       gpointer user_data)
-{
-	GSimpleAsyncResult *simple;
-	AsyncContext *async_context;
-
-	async_context = g_slice_new0 (AsyncContext);
-	async_context->ipart = g_object_ref (ipart);
-
-	simple = g_simple_async_result_new (
-		G_OBJECT (context), callback,
-		user_data, cipher_context_verify);
-
-	g_simple_async_result_set_check_cancellable (simple, cancellable);
-
-	g_simple_async_result_set_op_res_gpointer (
-		simple, async_context, (GDestroyNotify) async_context_free);
-
-	g_simple_async_result_run_in_thread (
-		simple, cipher_context_verify_thread,
-		io_priority, cancellable);
-
-	g_object_unref (simple);
-}
-
-static CamelCipherValidity *
-cipher_context_verify_finish (CamelCipherContext *context,
-                              GAsyncResult *result,
-                              GError **error)
-{
-	GSimpleAsyncResult *simple;
-	AsyncContext *async_context;
-	CamelCipherValidity *validity;
-
-	g_return_val_if_fail (
-		g_simple_async_result_is_valid (
-		result, G_OBJECT (context), cipher_context_verify), NULL);
-
-	simple = G_SIMPLE_ASYNC_RESULT (result);
-	async_context = g_simple_async_result_get_op_res_gpointer (simple);
-
-	if (g_simple_async_result_propagate_error (simple, error))
-		return NULL;
-
-	validity = async_context->validity;
-	async_context->validity = NULL;
-
-	return validity;
-}
-
-static void
-cipher_context_encrypt_thread (GSimpleAsyncResult *simple,
-                               GObject *object,
-                               GCancellable *cancellable)
-{
-	AsyncContext *async_context;
-	GError *error = NULL;
-
-	async_context = g_simple_async_result_get_op_res_gpointer (simple);
-
-	camel_cipher_context_encrypt_sync (
-		CAMEL_CIPHER_CONTEXT (object),
-		async_context->userid, async_context->strings,
-		async_context->ipart, async_context->opart,
-		cancellable, &error);
-
-	if (error != NULL)
-		g_simple_async_result_take_error (simple, error);
-}
-
-static void
-cipher_context_encrypt (CamelCipherContext *context,
-                        const gchar *userid,
-                        GPtrArray *recipients,
-                        CamelMimePart *ipart,
-                        CamelMimePart *opart,
-                        gint io_priority,
-                        GCancellable *cancellable,
-                        GAsyncReadyCallback callback,
-                        gpointer user_data)
-{
-	GSimpleAsyncResult *simple;
-	AsyncContext *async_context;
-	guint ii;
-
-	async_context = g_slice_new0 (AsyncContext);
-	async_context->userid = g_strdup (userid);
-	async_context->strings = g_ptr_array_new ();
-	async_context->ipart = g_object_ref (ipart);
-	async_context->opart = g_object_ref (opart);
-
-	for (ii = 0; ii < recipients->len; ii++)
-		g_ptr_array_add (
-			async_context->strings,
-			g_strdup (recipients->pdata[ii]));
-
-	simple = g_simple_async_result_new (
-		G_OBJECT (context), callback,
-		user_data, cipher_context_encrypt);
-
-	g_simple_async_result_set_check_cancellable (simple, cancellable);
-
-	g_simple_async_result_set_op_res_gpointer (
-		simple, async_context, (GDestroyNotify) async_context_free);
-
-	g_simple_async_result_run_in_thread (
-		simple, cipher_context_encrypt_thread,
-		io_priority, cancellable);
-
-	g_object_unref (simple);
-}
-
-static gboolean
-cipher_context_encrypt_finish (CamelCipherContext *context,
-                               GAsyncResult *result,
-                               GError **error)
-{
-	GSimpleAsyncResult *simple;
-
-	g_return_val_if_fail (
-		g_simple_async_result_is_valid (
-		result, G_OBJECT (context), cipher_context_encrypt), FALSE);
-
-	simple = G_SIMPLE_ASYNC_RESULT (result);
-
-	/* Assume success unless a GError is set. */
-	return !g_simple_async_result_propagate_error (simple, error);
-}
-
-static void
-cipher_context_decrypt_thread (GSimpleAsyncResult *simple,
-                               GObject *object,
-                               GCancellable *cancellable)
-{
-	AsyncContext *async_context;
-	GError *error = NULL;
-
-	async_context = g_simple_async_result_get_op_res_gpointer (simple);
-
-	async_context->validity = camel_cipher_context_decrypt_sync (
-		CAMEL_CIPHER_CONTEXT (object), async_context->ipart,
-		async_context->opart, cancellable, &error);
-
-	if (error != NULL)
-		g_simple_async_result_take_error (simple, error);
-}
-
-static void
-cipher_context_decrypt (CamelCipherContext *context,
-                        CamelMimePart *ipart,
-                        CamelMimePart *opart,
-                        gint io_priority,
-                        GCancellable *cancellable,
-                        GAsyncReadyCallback callback,
-                        gpointer user_data)
-{
-	GSimpleAsyncResult *simple;
-	AsyncContext *async_context;
-
-	async_context = g_slice_new0 (AsyncContext);
-	async_context->ipart = g_object_ref (ipart);
-	async_context->opart = g_object_ref (opart);
-
-	simple = g_simple_async_result_new (
-		G_OBJECT (context), callback,
-		user_data, cipher_context_decrypt);
-
-	g_simple_async_result_set_check_cancellable (simple, cancellable);
-
-	g_simple_async_result_set_op_res_gpointer (
-		simple, async_context, (GDestroyNotify) async_context_free);
-
-	g_simple_async_result_run_in_thread (
-		simple, cipher_context_decrypt_thread,
-		io_priority, cancellable);
-
-	g_object_unref (simple);
-}
-
-static CamelCipherValidity *
-cipher_context_decrypt_finish (CamelCipherContext *context,
-                               GAsyncResult *result,
-                               GError **error)
-{
-	GSimpleAsyncResult *simple;
-	AsyncContext *async_context;
-	CamelCipherValidity *validity;
-
-	g_return_val_if_fail (
-		g_simple_async_result_is_valid (
-		result, G_OBJECT (context), cipher_context_decrypt), NULL);
-
-	simple = G_SIMPLE_ASYNC_RESULT (result);
-	async_context = g_simple_async_result_get_op_res_gpointer (simple);
-
-	if (g_simple_async_result_propagate_error (simple, error))
-		return NULL;
-
-	validity = async_context->validity;
-	async_context->validity = NULL;
-
-	return validity;
-}
-
 static void
 camel_cipher_context_class_init (CamelCipherContextClass *class)
 {
@@ -572,15 +268,6 @@ camel_cipher_context_class_init (CamelCipherContextClass *class)
 	class->encrypt_sync = cipher_context_encrypt_sync;
 	class->decrypt_sync = cipher_context_decrypt_sync;
 
-	class->sign = cipher_context_sign;
-	class->sign_finish = cipher_context_sign_finish;
-	class->verify = cipher_context_verify;
-	class->verify_finish = cipher_context_verify_finish;
-	class->encrypt = cipher_context_encrypt;
-	class->encrypt_finish = cipher_context_encrypt_finish;
-	class->decrypt = cipher_context_decrypt;
-	class->decrypt_finish = cipher_context_decrypt_finish;
-
 	g_object_class_install_property (
 		object_class,
 		PROP_SESSION,
@@ -600,6 +287,34 @@ camel_cipher_context_init (CamelCipherContext *context)
 	g_mutex_init (&context->priv->lock);
 }
 
+/* Helper for camel_cipher_context_sign() */
+static void
+cipher_context_sign_thread (GTask *task,
+                            gpointer source_object,
+                            gpointer task_data,
+                            GCancellable *cancellable)
+{
+	gboolean success;
+	AsyncContext *async_context;
+	GError *local_error = NULL;
+
+	async_context = (AsyncContext *) task_data;
+
+	success = camel_cipher_context_sign_sync (
+		CAMEL_CIPHER_CONTEXT (source_object),
+		async_context->userid,
+		async_context->hash,
+		async_context->ipart,
+		async_context->opart,
+		cancellable, &local_error);
+
+	if (local_error != NULL) {
+		g_task_return_error (task, local_error);
+	} else {
+		g_task_return_boolean (task, success);
+	}
+}
+
 /**
  * camel_cipher_context_sign_sync:
  * @context: a #CamelCipherContext
@@ -688,16 +403,30 @@ camel_cipher_context_sign (CamelCipherContext *context,
                            GAsyncReadyCallback callback,
                            gpointer user_data)
 {
-	CamelCipherContextClass *class;
+	GTask *task;
+	AsyncContext *async_context;
 
 	g_return_if_fail (CAMEL_IS_CIPHER_CONTEXT (context));
+	g_return_if_fail (CAMEL_IS_MIME_PART (ipart));
+	g_return_if_fail (CAMEL_IS_MIME_PART (opart));
 
-	class = CAMEL_CIPHER_CONTEXT_GET_CLASS (context);
-	g_return_if_fail (class->sign != NULL);
+	async_context = g_slice_new0 (AsyncContext);
+	async_context->userid = g_strdup (userid);
+	async_context->hash = hash;
+	async_context->ipart = g_object_ref (ipart);
+	async_context->opart = g_object_ref (opart);
 
-	class->sign (
-		context, userid, hash, ipart, opart, io_priority,
-		cancellable, callback, user_data);
+	task = g_task_new (context, cancellable, callback, user_data);
+	g_task_set_source_tag (task, camel_cipher_context_sign);
+	g_task_set_priority (task, io_priority);
+
+	g_task_set_task_data (
+		task, async_context,
+		(GDestroyNotify) async_context_free);
+
+	g_task_run_in_thread (task, cipher_context_sign_thread);
+
+	g_object_unref (task);
 }
 
 /**
@@ -717,15 +446,14 @@ camel_cipher_context_sign_finish (CamelCipherContext *context,
                                   GAsyncResult *result,
                                   GError **error)
 {
-	CamelCipherContextClass *class;
-
 	g_return_val_if_fail (CAMEL_IS_CIPHER_CONTEXT (context), FALSE);
-	g_return_val_if_fail (G_IS_ASYNC_RESULT (result), FALSE);
+	g_return_val_if_fail (g_task_is_valid (result, context), FALSE);
 
-	class = CAMEL_CIPHER_CONTEXT_GET_CLASS (context);
-	g_return_val_if_fail (class->sign_finish != NULL, FALSE);
+	g_return_val_if_fail (
+		g_async_result_is_tagged (
+		result, camel_cipher_context_sign), FALSE);
 
-	return class->sign_finish (context, result, error);
+	return g_task_propagate_boolean (G_TASK (result), error);
 }
 
 /**
@@ -772,6 +500,34 @@ camel_cipher_context_verify_sync (CamelCipherContext *context,
 	return valid;
 }
 
+/* Helper for camel_cipher_context_verify() */
+static void
+cipher_context_verify_thread (GTask *task,
+                              gpointer source_object,
+                              gpointer task_data,
+                              GCancellable *cancellable)
+{
+	CamelCipherValidity *validity;
+	AsyncContext *async_context;
+	GError *local_error = NULL;
+
+	async_context = (AsyncContext *) task_data;
+
+	validity = camel_cipher_context_verify_sync (
+		CAMEL_CIPHER_CONTEXT (source_object),
+		async_context->ipart,
+		cancellable, &local_error);
+
+	if (local_error != NULL) {
+		g_warn_if_fail (validity == NULL);
+		g_task_return_error (task, local_error);
+	} else {
+		g_task_return_pointer (
+			task, validity,
+			(GDestroyNotify) camel_cipher_validity_free);
+	}
+}
+
 /**
  * camel_cipher_context_verify:
  * @context: a #CamelCipherContext
@@ -797,17 +553,26 @@ camel_cipher_context_verify (CamelCipherContext *context,
                              GAsyncReadyCallback callback,
                              gpointer user_data)
 {
-	CamelCipherContextClass *class;
+	GTask *task;
+	AsyncContext *async_context;
 
 	g_return_if_fail (CAMEL_IS_CIPHER_CONTEXT (context));
 	g_return_if_fail (CAMEL_IS_MIME_PART (ipart));
 
-	class = CAMEL_CIPHER_CONTEXT_GET_CLASS (context);
-	g_return_if_fail (class->verify != NULL);
+	async_context = g_slice_new0 (AsyncContext);
+	async_context->ipart = g_object_ref (ipart);
 
-	class->verify (
-		context, ipart, io_priority,
-		cancellable, callback, user_data);
+	task = g_task_new (context, cancellable, callback, user_data);
+	g_task_set_source_tag (task, camel_cipher_context_verify);
+	g_task_set_priority (task, io_priority);
+
+	g_task_set_task_data (
+		task, async_context,
+		(GDestroyNotify) async_context_free);
+
+	g_task_run_in_thread (task, cipher_context_verify_thread);
+
+	g_object_unref (task);
 }
 
 /**
@@ -829,15 +594,14 @@ camel_cipher_context_verify_finish (CamelCipherContext *context,
                                     GAsyncResult *result,
                                     GError **error)
 {
-	CamelCipherContextClass *class;
-
 	g_return_val_if_fail (CAMEL_IS_CIPHER_CONTEXT (context), NULL);
-	g_return_val_if_fail (G_IS_ASYNC_RESULT (context), NULL);
+	g_return_val_if_fail (g_task_is_valid (result, context), NULL);
 
-	class = CAMEL_CIPHER_CONTEXT_GET_CLASS (context);
-	g_return_val_if_fail (class->verify_finish != NULL, NULL);
+	g_return_val_if_fail (
+		g_async_result_is_tagged (
+		result, camel_cipher_context_verify), NULL);
 
-	return class->verify_finish (context, result, error);
+	return g_task_propagate_pointer (G_TASK (result), error);
 }
 
 /**
@@ -898,6 +662,34 @@ camel_cipher_context_encrypt_sync (CamelCipherContext *context,
 	return success;
 }
 
+/* Helper for camel_cipher_context_encrypt_thread() */
+static void
+cipher_context_encrypt_thread (GTask *task,
+                               gpointer source_object,
+                               gpointer task_data,
+                               GCancellable *cancellable)
+{
+	gboolean success;
+	AsyncContext *async_context;
+	GError *local_error = NULL;
+
+	async_context = (AsyncContext *) task_data;
+
+	success = camel_cipher_context_encrypt_sync (
+		CAMEL_CIPHER_CONTEXT (source_object),
+		async_context->userid,
+		async_context->strings,
+		async_context->ipart,
+		async_context->opart,
+		cancellable, &local_error);
+
+	if (local_error != NULL) {
+		g_task_return_error (task, local_error);
+	} else {
+		g_task_return_boolean (task, success);
+	}
+}
+
 /**
  * camel_cipher_context_encrypt:
  * @context: a #CamelCipherContext
@@ -930,18 +722,36 @@ camel_cipher_context_encrypt (CamelCipherContext *context,
                               GAsyncReadyCallback callback,
                               gpointer user_data)
 {
-	CamelCipherContextClass *class;
+	GTask *task;
+	AsyncContext *async_context;
+	guint ii;
 
 	g_return_if_fail (CAMEL_IS_CIPHER_CONTEXT (context));
 	g_return_if_fail (CAMEL_IS_MIME_PART (ipart));
 	g_return_if_fail (CAMEL_IS_MIME_PART (opart));
 
-	class = CAMEL_CIPHER_CONTEXT_GET_CLASS (context);
-	g_return_if_fail (class->encrypt != NULL);
+	async_context = g_slice_new0 (AsyncContext);
+	async_context->userid = g_strdup (userid);
+	async_context->strings = g_ptr_array_new ();
+	async_context->ipart = g_object_ref (ipart);
+	async_context->opart = g_object_ref (opart);
 
-	class->encrypt (
-		context, userid, recipients, ipart, opart,
-		io_priority, cancellable, callback, user_data);
+	for (ii = 0; ii < recipients->len; ii++)
+		g_ptr_array_add (
+			async_context->strings,
+			g_strdup (recipients->pdata[ii]));
+
+	task = g_task_new (context, cancellable, callback, user_data);
+	g_task_set_source_tag (task, camel_cipher_context_encrypt);
+	g_task_set_priority (task, io_priority);
+
+	g_task_set_task_data (
+		task, async_context,
+		(GDestroyNotify) async_context_free);
+
+	g_task_run_in_thread (task, cipher_context_encrypt_thread);
+
+	g_object_unref (task);
 }
 
 /**
@@ -961,15 +771,14 @@ camel_cipher_context_encrypt_finish (CamelCipherContext *context,
                                      GAsyncResult *result,
                                      GError **error)
 {
-	CamelCipherContextClass *class;
-
 	g_return_val_if_fail (CAMEL_IS_CIPHER_CONTEXT (context), FALSE);
-	g_return_val_if_fail (G_IS_ASYNC_RESULT (result), FALSE);
+	g_return_val_if_fail (g_task_is_valid (result, context), FALSE);
 
-	class = CAMEL_CIPHER_CONTEXT_GET_CLASS (context);
-	g_return_val_if_fail (class->encrypt_finish != NULL, FALSE);
+	g_return_val_if_fail (
+		g_async_result_is_tagged (
+		result, camel_cipher_context_encrypt), FALSE);
 
-	return class->encrypt_finish (context, result, error);
+	return g_task_propagate_boolean (G_TASK (result), error);
 }
 
 /**
@@ -1024,6 +833,35 @@ camel_cipher_context_decrypt_sync (CamelCipherContext *context,
 	return valid;
 }
 
+/* Helper for camel_cipher_context_decrypt() */
+static void
+cipher_context_decrypt_thread (GTask *task,
+                               gpointer source_object,
+                               gpointer task_data,
+                               GCancellable *cancellable)
+{
+	CamelCipherValidity *validity;
+	AsyncContext *async_context;
+	GError *local_error = NULL;
+
+	async_context = (AsyncContext *) task_data;
+
+	validity = camel_cipher_context_decrypt_sync (
+		CAMEL_CIPHER_CONTEXT (source_object),
+		async_context->ipart,
+		async_context->opart,
+		cancellable, &local_error);
+
+	if (local_error != NULL) {
+		g_warn_if_fail (validity == NULL);
+		g_task_return_error (task, local_error);
+	} else {
+		g_task_return_pointer (
+			task, validity,
+			(GDestroyNotify) camel_cipher_validity_free);
+	}
+}
+
 /**
  * camel_cipher_context_decrypt:
  * @context: a #CamelCipherContext
@@ -1051,18 +889,28 @@ camel_cipher_context_decrypt (CamelCipherContext *context,
                               GAsyncReadyCallback callback,
                               gpointer user_data)
 {
-	CamelCipherContextClass *class;
+	GTask *task;
+	AsyncContext *async_context;
 
 	g_return_if_fail (CAMEL_IS_CIPHER_CONTEXT (context));
 	g_return_if_fail (CAMEL_IS_MIME_PART (ipart));
 	g_return_if_fail (CAMEL_IS_MIME_PART (opart));
 
-	class = CAMEL_CIPHER_CONTEXT_GET_CLASS (context);
-	g_return_if_fail (class->decrypt != NULL);
+	async_context = g_slice_new0 (AsyncContext);
+	async_context->ipart = g_object_ref (ipart);
+	async_context->opart = g_object_ref (opart);
 
-	class->decrypt (
-		context, ipart, opart, io_priority,
-		cancellable, callback, user_data);
+	task = g_task_new (context, cancellable, callback, user_data);
+	g_task_set_source_tag (task, camel_cipher_context_decrypt);
+	g_task_set_priority (task, io_priority);
+
+	g_task_set_task_data (
+		task, async_context,
+		(GDestroyNotify) async_context_free);
+
+	g_task_run_in_thread (task, cipher_context_decrypt_thread);
+
+	g_object_unref (task);
 }
 
 /**
@@ -1082,15 +930,14 @@ camel_cipher_context_decrypt_finish (CamelCipherContext *context,
                                      GAsyncResult *result,
                                      GError **error)
 {
-	CamelCipherContextClass *class;
-
 	g_return_val_if_fail (CAMEL_IS_CIPHER_CONTEXT (context), NULL);
-	g_return_val_if_fail (G_IS_ASYNC_RESULT (result), NULL);
+	g_return_val_if_fail (g_task_is_valid (result, context), NULL);
 
-	class = CAMEL_CIPHER_CONTEXT_GET_CLASS (context);
-	g_return_val_if_fail (class->decrypt_finish != NULL, NULL);
+	g_return_val_if_fail (
+		g_async_result_is_tagged (
+		result, camel_cipher_context_decrypt), NULL);
 
-	return class->decrypt_finish (context, result, error);
+	return g_task_propagate_pointer (G_TASK (result), error);
 }
 
 /* a couple of util functions */
