@@ -55,12 +55,12 @@ typedef struct _CamelDataCacheClass CamelDataCacheClass;
 typedef struct _CamelDataCachePrivate CamelDataCachePrivate;
 
 struct _CamelDataCache {
-	CamelObject parent;
+	GObject parent;
 	CamelDataCachePrivate *priv;
 };
 
 struct _CamelDataCacheClass {
-	CamelObjectClass parent_class;
+	GObjectClass parent_class;
 };
 
 GType		camel_data_cache_get_type	(void);
@@ -74,11 +74,11 @@ void		camel_data_cache_set_expire_age	(CamelDataCache *cdc,
 void		camel_data_cache_set_expire_access
 						(CamelDataCache *cdc,
 						 time_t when);
-CamelStream *	camel_data_cache_add		(CamelDataCache *cdc,
+GIOStream *	camel_data_cache_add		(CamelDataCache *cdc,
 						 const gchar *path,
 						 const gchar *key,
 						 GError **error);
-CamelStream *	camel_data_cache_get		(CamelDataCache *cdc,
+GIOStream *	camel_data_cache_get		(CamelDataCache *cdc,
 						 const gchar *path,
 						 const gchar *key,
 						 GError **error);
