@@ -219,11 +219,6 @@ struct _CamelFolderClass {
 	gboolean	(*expunge_sync)		(CamelFolder *folder,
 						 GCancellable *cancellable,
 						 GError **error);
-	gboolean	(*fetch_messages_sync)	(CamelFolder *folder,
-						 CamelFetchType type,
-						 gint limit,
-						 GCancellable *cancellable,
-						 GError **error);
 	CamelMimeMessage *
 			(*get_message_sync)	(CamelFolder *folder,
 						 const gchar *message_uid,
@@ -260,110 +255,8 @@ struct _CamelFolderClass {
 						 GCancellable *cancellable,
 						 GError **error);
 
-	/* Asynchronous I/O Methods (all have defaults) */
-	void		(*append_message)	(CamelFolder *folder,
-						 CamelMimeMessage *message,
-						 CamelMessageInfo *info,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-	gboolean	(*append_message_finish)
-						(CamelFolder *folder,
-						 GAsyncResult *result,
-						 gchar **appended_uid,
-						 GError **error);
-	void		(*expunge)		(CamelFolder *folder,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-	gboolean	(*expunge_finish)	(CamelFolder *folder,
-						 GAsyncResult *result,
-						 GError **error);
-	void		(*fetch_messages)	(CamelFolder *folder,
-						 CamelFetchType type,
-						 gint limit,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-	gboolean	(*fetch_messages_finish)
-						(CamelFolder *folder,
-						 GAsyncResult *result,
-						 GError **error);
-	void		(*get_message)		(CamelFolder *folder,
-						 const gchar *message_uid,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-	CamelMimeMessage *
-			(*get_message_finish)	(CamelFolder *folder,
-						 GAsyncResult *result,
-						 GError **error);
-	void		(*get_quota_info)	(CamelFolder *folder,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-	CamelFolderQuotaInfo *
-			(*get_quota_info_finish)
-						(CamelFolder *folder,
-						 GAsyncResult *result,
-						 GError **error);
-	void		(*purge_message_cache)	(CamelFolder *folder,
-						 gchar *start_uid,
-						 gchar *end_uid,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-	gboolean	(*purge_message_cache_finish)
-						(CamelFolder *folder,
-						 GAsyncResult *result,
-						 GError **error);
-
-	void		(*refresh_info)		(CamelFolder *folder,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-	gboolean	(*refresh_info_finish)	(CamelFolder *folder,
-						 GAsyncResult *result,
-						 GError **error);
-	void		(*synchronize)		(CamelFolder *folder,
-						 gboolean expunge,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-	gboolean	(*synchronize_finish)	(CamelFolder *folder,
-						 GAsyncResult *result,
-						 GError **error);
-	void		(*synchronize_message)	(CamelFolder *folder,
-						 const gchar *message_uid,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-	gboolean	(*synchronize_message_finish)
-						(CamelFolder *folder,
-						 GAsyncResult *result,
-						 GError **error);
-	void		(*transfer_messages_to)	(CamelFolder *source,
-						 GPtrArray *message_uids,
-						 CamelFolder *destination,
-						 gboolean delete_originals,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-	gboolean	(*transfer_messages_to_finish)
-						(CamelFolder *source,
-						 GAsyncResult *result,
-						 GPtrArray **transferred_uids,
-						 GError **error);
+	/* Reserved slots for methods. */
+	gpointer reserved_for_methods[20];
 
 	/* Signals */
 	void		(*changed)		(CamelFolder *folder,
@@ -528,23 +421,6 @@ void		camel_folder_expunge		(CamelFolder *folder,
 gboolean	camel_folder_expunge_finish	(CamelFolder *folder,
 						 GAsyncResult *result,
 						 GError **error);
-gboolean	camel_folder_fetch_messages_sync
-						(CamelFolder *folder,
-						 CamelFetchType type,
-						 gint limit,
-						 GCancellable *cancellable,
-						 GError **error);
-void		camel_folder_fetch_messages	(CamelFolder *folder,
-						 CamelFetchType type,
-						 gint limit,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-gboolean	camel_folder_fetch_messages_finish
-						(CamelFolder *folder,
-						 GAsyncResult *result,
-						 GError **error);
 CamelMimeMessage *
 		camel_folder_get_message_sync	(CamelFolder *folder,
 						 const gchar *message_uid,
