@@ -66,7 +66,6 @@ struct _CamelMimePartPrivate {
 };
 
 struct _AsyncContext {
-	/* arguments */
 	CamelMimeParser *parser;
 };
 
@@ -785,72 +784,6 @@ mime_part_construct_from_parser_sync (CamelMimePart *mime_part,
 	return success;
 }
 
-static void
-mime_part_construct_from_parser_thread (GSimpleAsyncResult *simple,
-                                        GObject *object,
-                                        GCancellable *cancellable)
-{
-	AsyncContext *async_context;
-	GError *error = NULL;
-
-	async_context = g_simple_async_result_get_op_res_gpointer (simple);
-
-	camel_mime_part_construct_from_parser_sync (
-		CAMEL_MIME_PART (object), async_context->parser,
-		cancellable, &error);
-
-	if (error != NULL)
-		g_simple_async_result_take_error (simple, error);
-}
-
-static void
-mime_part_construct_from_parser (CamelMimePart *mime_part,
-                                 CamelMimeParser *parser,
-                                 gint io_priority,
-                                 GCancellable *cancellable,
-                                 GAsyncReadyCallback callback,
-                                 gpointer user_data)
-{
-	GSimpleAsyncResult *simple;
-	AsyncContext *async_context;
-
-	async_context = g_slice_new0 (AsyncContext);
-	async_context->parser = g_object_ref (parser);
-
-	simple = g_simple_async_result_new (
-		G_OBJECT (mime_part), callback, user_data,
-		mime_part_construct_from_parser);
-
-	g_simple_async_result_set_check_cancellable (simple, cancellable);
-
-	g_simple_async_result_set_op_res_gpointer (
-		simple, async_context, (GDestroyNotify) async_context_free);
-
-	g_simple_async_result_run_in_thread (
-		simple, mime_part_construct_from_parser_thread,
-		io_priority, cancellable);
-
-	g_object_unref (simple);
-}
-
-static gboolean
-mime_part_construct_from_parser_finish (CamelMimePart *mime_part,
-                                        GAsyncResult *result,
-                                        GError **error)
-{
-	GSimpleAsyncResult *simple;
-
-	g_return_val_if_fail (
-		g_simple_async_result_is_valid (
-		result, G_OBJECT (mime_part),
-		mime_part_construct_from_parser), FALSE);
-
-	simple = G_SIMPLE_ASYNC_RESULT (result);
-
-	/* Assume success unless a GError is set. */
-	return !g_simple_async_result_propagate_error (simple, error);
-}
-
 static void
 camel_mime_part_class_init (CamelMimePartClass *class)
 {
@@ -879,8 +812,6 @@ camel_mime_part_class_init (CamelMimePartClass *class)
 	data_wrapper_class->construct_from_stream_sync = mime_part_construct_from_stream_sync;
 
 	class->construct_from_parser_sync = mime_part_construct_from_parser_sync;
-	class->construct_from_parser = mime_part_construct_from_parser;
-	class->construct_from_parser_finish = mime_part_construct_from_parser_finish;
 
 	g_object_class_install_property (
 		object_class,
@@ -1440,6 +1371,31 @@ camel_mime_part_construct_from_parser_sync (CamelMimePart *mime_part,
 	return success;
 }
 
+/* Helper for camel_mime_part_construct_from_parser() */
+static void
+mime_part_construct_from_parser_thread (GTask *task,
+                                        gpointer source_object,
+                                        gpointer task_data,
+                                        GCancellable *cancellable)
+{
+	gboolean success;
+	AsyncContext *async_context;
+	GError *local_error = NULL;
+
+	async_context = (AsyncContext *) task_data;
+
+	success = camel_mime_part_construct_from_parser_sync (
+		CAMEL_MIME_PART (source_object),
+		async_context->parser,
+		cancellable, &local_error);
+
+	if (local_error != NULL) {
+		g_task_return_error (task, local_error);
+	} else {
+		g_task_return_boolean (task, success);
+	}
+}
+
 /**
  * camel_mime_part_construct_from_parser:
  * @mime_part: a #CamelMimePart
@@ -1465,17 +1421,26 @@ camel_mime_part_construct_from_parser (CamelMimePart *mime_part,
                                        GAsyncReadyCallback callback,
                                        gpointer user_data)
 {
-	CamelMimePartClass *class;
+	GTask *task;
+	AsyncContext *async_context;
 
 	g_return_if_fail (CAMEL_IS_MIME_PART (mime_part));
 	g_return_if_fail (CAMEL_IS_MIME_PARSER (parser));
 
-	class = CAMEL_MIME_PART_GET_CLASS (mime_part);
-	g_return_if_fail (class->construct_from_parser != NULL);
+	async_context = g_slice_new0 (AsyncContext);
+	async_context->parser = g_object_ref (parser);
 
-	class->construct_from_parser (
-		mime_part, parser, io_priority,
-		cancellable, callback, user_data);
+	task = g_task_new (mime_part, cancellable, callback, user_data);
+	g_task_set_source_tag (task, camel_mime_part_construct_from_parser);
+	g_task_set_priority (task, io_priority);
+
+	g_task_set_task_data (
+		task, async_context,
+		(GDestroyNotify) async_context_free);
+
+	g_task_run_in_thread (task, mime_part_construct_from_parser_thread);
+
+	g_object_unref (task);
 }
 
 /**
@@ -1495,13 +1460,12 @@ camel_mime_part_construct_from_parser_finish (CamelMimePart *mime_part,
                                               GAsyncResult *result,
                                               GError **error)
 {
-	CamelMimePartClass *class;
-
 	g_return_val_if_fail (CAMEL_IS_MIME_PART (mime_part), FALSE);
-	g_return_val_if_fail (G_IS_ASYNC_RESULT (result), FALSE);
+	g_return_val_if_fail (g_task_is_valid (result, mime_part), FALSE);
 
-	class = CAMEL_MIME_PART_GET_CLASS (mime_part);
-	g_return_val_if_fail (class->construct_from_parser_finish != NULL, FALSE);
+	g_return_val_if_fail (
+		g_async_result_is_tagged (
+		result, camel_mime_part_construct_from_parser), FALSE);
 
-	return class->construct_from_parser_finish (mime_part, result, error);
+	return g_task_propagate_boolean (G_TASK (result), error);
 }
