@@ -116,43 +116,8 @@ struct _CamelServiceClass {
 						 GCancellable *cancellable,
 						 GError **error);
 
-	/* Asynchronous I/O Methods (all have defaults) */
-	void		(*connect)		(CamelService *service,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-	gboolean	(*connect_finish)	(CamelService *service,
-						 GAsyncResult *result,
-						 GError **error);
-	void		(*disconnect)		(CamelService *service,
-						 gboolean clean,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-	gboolean	(*disconnect_finish)	(CamelService *service,
-						 GAsyncResult *result,
-						 GError **error);
-	void		(*authenticate)		(CamelService *service,
-						 const gchar *mechanism,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-	CamelAuthenticationResult
-			(*authenticate_finish)	(CamelService *service,
-						 GAsyncResult *result,
-						 GError **error);
-	void		(*query_auth_types)	(CamelService *service,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-	GList *		(*query_auth_types_finish)
-						(CamelService *service,
-						 GAsyncResult *result,
-						 GError **error);
+	/* Reserved slots. */
+	gpointer reserved[8];
 };
 
 /* query_auth_types returns a GList of these */
@@ -197,7 +162,9 @@ CamelSettings *	camel_service_ref_settings	(CamelService *service);
 void		camel_service_set_settings	(CamelService *service,
 						 CamelSettings *settings);
 const gchar *	camel_service_get_uid		(CamelService *service);
-
+void		camel_service_queue_task	(CamelService *service,
+						 GTask *task,
+						 GTaskThreadFunc task_func);
 gboolean	camel_service_connect_sync	(CamelService *service,
 						 GCancellable *cancellable,
 						 GError **error);
@@ -252,11 +219,6 @@ GList *		camel_service_query_auth_types_finish
 						 GAsyncResult *result,
 						 GError **error);
 
-#ifndef CAMEL_DISABLE_DEPRECATED
-struct _CamelSession *
-		camel_service_get_session	(CamelService *service);
-#endif /* CAMEL_DISABLE_DEPRECATED */
-
 G_END_DECLS
 
 #endif /* CAMEL_SERVICE_H */
