@@ -38,6 +38,7 @@
 #include <glib/gstdio.h>
 
 #include "camel-debug.h"
+#include "camel-enumtypes.h"
 #include "camel-file-utils.h"
 #include "camel-folder.h"
 #include "camel-mime-message.h"
@@ -58,6 +59,7 @@
 #define d(x)
 
 typedef struct _AsyncContext AsyncContext;
+typedef struct _SignalClosure SignalClosure;
 typedef struct _JobData JobData;
 
 struct _CamelSessionPrivate {
@@ -72,8 +74,7 @@ struct _CamelSessionPrivate {
 
 	GMainContext *main_context;
 
-	guint check_junk        : 1;
-	guint online            : 1;
+	guint online : 1;
 };
 
 struct _AsyncContext {
@@ -84,6 +85,13 @@ struct _AsyncContext {
 	gchar *auth_mechanism;
 };
 
+struct _SignalClosure {
+	GWeakRef session;
+	CamelService *service;
+	CamelSessionAlertType alert_type;
+	gchar *alert_message;
+};
+
 struct _JobData {
 	CamelSession *session;
 	GCancellable *cancellable;
@@ -94,7 +102,6 @@ struct _JobData {
 
 enum {
 	PROP_0,
-	PROP_CHECK_JUNK,
 	PROP_JUNK_FILTER,
 	PROP_MAIN_CONTEXT,
 	PROP_ONLINE,
@@ -105,6 +112,7 @@ enum {
 enum {
 	JOB_STARTED,
 	JOB_FINISHED,
+	USER_ALERT,
 	LAST_SIGNAL
 };
 
@@ -130,6 +138,19 @@ async_context_free (AsyncContext *async_context)
 	g_slice_free (AsyncContext, async_context);
 }
 
+static void
+signal_closure_free (SignalClosure *signal_closure)
+{
+	g_weak_ref_set (&signal_closure->session, NULL);
+
+	if (signal_closure->service != NULL)
+		g_object_unref (signal_closure->service);
+
+	g_free (signal_closure->alert_message);
+
+	g_slice_free (SignalClosure, signal_closure);
+}
+
 static void
 job_data_free (JobData *job_data)
 {
@@ -143,75 +164,98 @@ job_data_free (JobData *job_data)
 }
 
 static void
-session_finish_job_cb (CamelSession *session,
-                       GSimpleAsyncResult *simple)
+session_finish_job_cb (GObject *source_object,
+                       GAsyncResult *result,
+                       gpointer unused)
 {
-	JobData *job_data;
-	GError *error = NULL;
+	GCancellable *cancellable;
+	GError *local_error = NULL;
 
-	if (!g_simple_async_result_propagate_error (simple, &error))
-		error = NULL;
+	cancellable = g_task_get_cancellable (G_TASK (result));
 
-	job_data = g_simple_async_result_get_op_res_gpointer (simple);
+	/* XXX Ignore the return value, this is just
+	 *     to extract the GError if there is one. */
+	g_task_propagate_boolean (G_TASK (result), &local_error);
 
 	g_signal_emit (
-		job_data->session,
+		CAMEL_SESSION (source_object),
 		signals[JOB_FINISHED], 0,
-		job_data->cancellable, error);
+		cancellable, local_error);
 
-	g_clear_error (&error);
+	g_clear_error (&local_error);
 }
 
 static void
-session_do_job_cb (GSimpleAsyncResult *simple,
-                   CamelSession *session,
+session_do_job_cb (GTask *task,
+                   gpointer source_object,
+                   gpointer task_data,
                    GCancellable *cancellable)
 {
 	JobData *job_data;
-	GError *error = NULL;
+	GError *local_error = NULL;
 
-	job_data = g_simple_async_result_get_op_res_gpointer (simple);
+	job_data = (JobData *) task_data;
 
 	job_data->callback (
-		session, cancellable,
-		job_data->user_data, &error);
+		CAMEL_SESSION (source_object),
+		cancellable,
+		job_data->user_data,
+		&local_error);
 
-	if (error != NULL)
-		g_simple_async_result_take_error (simple, error);
+	if (local_error != NULL) {
+		g_task_return_error (task, local_error);
+	} else {
+		g_task_return_boolean (task, TRUE);
+	}
 }
 
 static gboolean
 session_start_job_cb (gpointer user_data)
 {
 	JobData *job_data = user_data;
-	GSimpleAsyncResult *simple;
+	GTask *task;
 
 	g_signal_emit (
 		job_data->session,
 		signals[JOB_STARTED], 0,
 		job_data->cancellable);
 
-	simple = g_simple_async_result_new (
-		G_OBJECT (job_data->session),
-		(GAsyncReadyCallback) session_finish_job_cb,
-		NULL, camel_session_submit_job);
+	task = g_task_new (
+		job_data->session,
+		job_data->cancellable,
+		session_finish_job_cb, NULL);
 
-	g_simple_async_result_set_check_cancellable (
-		simple, job_data->cancellable);
+	g_task_set_task_data (
+		task, job_data, (GDestroyNotify) job_data_free);
 
-	g_simple_async_result_set_op_res_gpointer (
-		simple, job_data, (GDestroyNotify) job_data_free);
+	g_task_run_in_thread (task, session_do_job_cb);
 
-	g_simple_async_result_run_in_thread (
-		simple, (GSimpleAsyncThreadFunc)
-		session_do_job_cb, JOB_PRIORITY,
-		job_data->cancellable);
-
-	g_object_unref (simple);
+	g_object_unref (task);
 
 	return FALSE;
 }
 
+static gboolean
+session_emit_user_alert_cb (gpointer user_data)
+{
+	SignalClosure *signal_closure = user_data;
+	CamelSession *session;
+
+	session = g_weak_ref_get (&signal_closure->session);
+
+	if (session != NULL) {
+		g_signal_emit (
+			session,
+			signals[USER_ALERT], 0,
+			signal_closure->service,
+			signal_closure->alert_type,
+			signal_closure->alert_message);
+		g_object_unref (session);
+	}
+
+	return G_SOURCE_REMOVE;
+}
+
 static void
 session_set_user_data_dir (CamelSession *session,
                            const gchar *user_data_dir)
@@ -239,12 +283,6 @@ session_set_property (GObject *object,
                       GParamSpec *pspec)
 {
 	switch (property_id) {
-		case PROP_CHECK_JUNK:
-			camel_session_set_check_junk (
-				CAMEL_SESSION (object),
-				g_value_get_boolean (value));
-			return;
-
 		case PROP_JUNK_FILTER:
 			camel_session_set_junk_filter (
 				CAMEL_SESSION (object),
@@ -280,12 +318,6 @@ session_get_property (GObject *object,
                       GParamSpec *pspec)
 {
 	switch (property_id) {
-		case PROP_CHECK_JUNK:
-			g_value_set_boolean (
-				value, camel_session_get_check_junk (
-				CAMEL_SESSION (object)));
-			return;
-
 		case PROP_JUNK_FILTER:
 			g_value_set_object (
 				value, camel_session_get_junk_filter (
@@ -543,77 +575,6 @@ retry:
 	return (result == CAMEL_AUTHENTICATION_ACCEPTED);
 }
 
-static void
-session_authenticate_thread (GSimpleAsyncResult *simple,
-                             GObject *object,
-                             GCancellable *cancellable)
-{
-	AsyncContext *async_context;
-	GError *error = NULL;
-
-	async_context = g_simple_async_result_get_op_res_gpointer (simple);
-
-	if (!camel_session_authenticate_sync (
-		CAMEL_SESSION (object),
-		async_context->service,
-		async_context->auth_mechanism,
-		cancellable, &error)) {
-
-		if (!error)
-			error = g_error_new_literal (CAMEL_ERROR, CAMEL_ERROR_GENERIC, _("Unknown error"));
-	}
-
-	if (error != NULL)
-		g_simple_async_result_take_error (simple, error);
-}
-
-static void
-session_authenticate (CamelSession *session,
-                      CamelService *service,
-                      const gchar *mechanism,
-                      gint io_priority,
-                      GCancellable *cancellable,
-                      GAsyncReadyCallback callback,
-                      gpointer user_data)
-{
-	GSimpleAsyncResult *simple;
-	AsyncContext *async_context;
-
-	async_context = g_slice_new0 (AsyncContext);
-	async_context->service = g_object_ref (service);
-	async_context->auth_mechanism = g_strdup (mechanism);
-
-	simple = g_simple_async_result_new (
-		G_OBJECT (session), callback, user_data, session_authenticate);
-
-	g_simple_async_result_set_check_cancellable (simple, cancellable);
-
-	g_simple_async_result_set_op_res_gpointer (
-		simple, async_context, (GDestroyNotify) async_context_free);
-
-	g_simple_async_result_run_in_thread (
-		simple, session_authenticate_thread, io_priority, cancellable);
-
-	g_object_unref (simple);
-}
-
-static gboolean
-session_authenticate_finish (CamelSession *session,
-                             GAsyncResult *result,
-                             GError **error)
-{
-	GSimpleAsyncResult *simple;
-
-	g_return_val_if_fail (
-		g_simple_async_result_is_valid (
-		result, G_OBJECT (session), session_authenticate), FALSE);
-
-	simple = G_SIMPLE_ASYNC_RESULT (result);
-
-	/* Assume success unless a GError is set. */
-	return !g_simple_async_result_propagate_error (simple, error);
-}
-
 static gboolean
 session_forward_to_sync (CamelSession *session,
                          CamelFolder *folder,
@@ -629,76 +590,6 @@ session_forward_to_sync (CamelSession *session,
 	return FALSE;
 }
 
-static void
-session_forward_to_thread (GSimpleAsyncResult *simple,
-                           GObject *object,
-                           GCancellable *cancellable)
-{
-	AsyncContext *async_context;
-	GError *error = NULL;
-
-	async_context = g_simple_async_result_get_op_res_gpointer (simple);
-
-	camel_session_forward_to_sync (
-		CAMEL_SESSION (object),
-		async_context->folder,
-		async_context->message,
-		async_context->address,
-		cancellable, &error);
-
-	if (error != NULL)
-		g_simple_async_result_take_error (simple, error);
-}
-
-static void
-session_forward_to (CamelSession *session,
-                    CamelFolder *folder,
-                    CamelMimeMessage *message,
-                    const gchar *address,
-                    gint io_priority,
-                    GCancellable *cancellable,
-                    GAsyncReadyCallback callback,
-                    gpointer user_data)
-{
-	GSimpleAsyncResult *simple;
-	AsyncContext *async_context;
-
-	async_context = g_slice_new0 (AsyncContext);
-	async_context->folder = g_object_ref (folder);
-	async_context->message = g_object_ref (message);
-	async_context->address = g_strdup (address);
-
-	simple = g_simple_async_result_new (
-		G_OBJECT (session), callback, user_data, session_forward_to);
-
-	g_simple_async_result_set_check_cancellable (simple, cancellable);
-
-	g_simple_async_result_set_op_res_gpointer (
-		simple, async_context, (GDestroyNotify) async_context_free);
-
-	g_simple_async_result_run_in_thread (
-		simple, session_forward_to_thread, io_priority, cancellable);
-
-	g_object_unref (simple);
-}
-
-static gboolean
-session_forward_to_finish (CamelSession *session,
-                           GAsyncResult *result,
-                           GError **error)
-{
-	GSimpleAsyncResult *simple;
-
-	g_return_val_if_fail (
-		g_simple_async_result_is_valid (
-		result, G_OBJECT (session), session_forward_to), FALSE);
-
-	simple = G_SIMPLE_ASYNC_RESULT (result);
-
-	/* Assume success unless a GError is set. */
-	return !g_simple_async_result_propagate_error (simple, error);
-}
-
 static void
 camel_session_class_init (CamelSessionClass *class)
 {
@@ -718,23 +609,6 @@ camel_session_class_init (CamelSessionClass *class)
 	class->authenticate_sync = session_authenticate_sync;
 	class->forward_to_sync = session_forward_to_sync;
 
-	class->authenticate = session_authenticate;
-	class->authenticate_finish = session_authenticate_finish;
-	class->forward_to = session_forward_to;
-	class->forward_to_finish = session_forward_to_finish;
-
-	g_object_class_install_property (
-		object_class,
-		PROP_CHECK_JUNK,
-		g_param_spec_boolean (
-			"check-junk",
-			"Check Junk",
-			"Check incoming messages for junk",
-			FALSE,
-			G_PARAM_READWRITE |
-			G_PARAM_CONSTRUCT |
-			G_PARAM_STATIC_STRINGS));
-
 	g_object_class_install_property (
 		object_class,
 		PROP_JUNK_FILTER,
@@ -812,6 +686,28 @@ camel_session_class_init (CamelSessionClass *class)
 		G_TYPE_NONE, 2,
 		G_TYPE_CANCELLABLE,
 		G_TYPE_POINTER);
+
+	/**
+	 * CamelSession::user-alert:
+	 * @session: the #CamelSession that received the signal
+	 * @service: the #CamelService issuing the alert
+	 * @type: the #CamelSessionAlertType
+	 * @message: the alert message
+	 *
+	 * This purpose of this signal is to propagate a server-issued alert
+	 * message from @service to a user interface.  The @type hints at the
+	 * severity of the alert message.
+	 **/
+	signals[USER_ALERT] = g_signal_new (
+		"user-alert",
+		G_OBJECT_CLASS_TYPE (class),
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (CamelSessionClass, user_alert),
+		NULL, NULL, NULL,
+		G_TYPE_NONE, 3,
+		CAMEL_TYPE_SERVICE,
+		CAMEL_TYPE_SESSION_ALERT_TYPE,
+		G_TYPE_STRING);
 }
 
 static void
@@ -834,27 +730,6 @@ camel_session_init (CamelSession *session)
 	session->priv->main_context = g_main_context_ref_thread_default ();
 }
 
-/**
- * camel_session_get_main_context:
- * @session: a #CamelSession
- *
- * Returns the #GMainContext on which event sources for @session are to
- * be attached.
- *
- * Returns: a #GMainContext
- *
- * Since: 3.6
- *
- * Deprecated: 3.8: Use camel_session_ref_main_context() instead.
- **/
-GMainContext *
-camel_session_get_main_context (CamelSession *session)
-{
-	g_return_val_if_fail (CAMEL_IS_SESSION (session), NULL);
-
-	return session->priv->main_context;
-}
-
 /**
  * camel_session_ref_main_context:
  * @session: a #CamelSession
@@ -1257,39 +1132,6 @@ camel_session_forget_password (CamelSession *session,
 	return success;
 }
 
-/**
- * camel_session_alert_user:
- * @session: a #CamelSession
- * @type: the type of alert (info, warning, or error)
- * @prompt: the message for the user
- * @button_captions: List of button captions to use. If NULL, only "Dismiss" button is shown.
- * @cancellable: (allow-non): optional #GCancellable object, or %NULL
- *
- * Presents the given @prompt to the user, in the style indicated by
- * @type. If @cancel is %TRUE, the user will be able to accept or
- * cancel. Otherwise, the message is purely informational.
- *
- * Returns: Index of pressed button from @button_captions, -1 if NULL.
- */
-gint
-camel_session_alert_user (CamelSession *session,
-                          CamelSessionAlertType type,
-                          const gchar *prompt,
-                          GList *button_captions,
-                          GCancellable *cancellable)
-{
-	CamelSessionClass *class;
-
-	g_return_val_if_fail (CAMEL_IS_SESSION (session), -1);
-	g_return_val_if_fail (prompt != NULL, -1);
-
-	class = CAMEL_SESSION_GET_CLASS (session);
-	g_return_val_if_fail (class->alert_user != NULL, -1);
-
-	return class->alert_user (
-		session, type, prompt, button_captions, cancellable);
-}
-
 /**
  * camel_session_trust_prompt:
  * @session: a #CamelSession
@@ -1331,6 +1173,47 @@ camel_session_trust_prompt (CamelSession *session,
 	return class->trust_prompt (session, service, certificate, errors);
 }
 
+/**
+ * camel_session_user_alert:
+ * @session: a #CamelSession
+ * @service: a #CamelService
+ * @type: a #CamelSessionAlertType
+ * @message: the message for the user
+ *
+ * Emits a #CamelSession:user_alert signal from an idle source on the main
+ * loop.  The idle source's priority is #G_PRIORITY_LOW.
+ *
+ * The purpose of the signal is to propagate a server-issued alert message
+ * from @service to a user interface.  The @type hints at the nature of the
+ * alert message.
+ *
+ * Since: 3.12
+ */
+void
+camel_session_user_alert (CamelSession *session,
+                          CamelService *service,
+                          CamelSessionAlertType type,
+                          const gchar *message)
+{
+	SignalClosure *signal_closure;
+
+	g_return_if_fail (CAMEL_IS_SESSION (session));
+	g_return_if_fail (CAMEL_IS_SERVICE (service));
+	g_return_if_fail (message != NULL);
+
+	signal_closure = g_slice_new0 (SignalClosure);
+	g_weak_ref_set (&signal_closure->session, session);
+	signal_closure->service = g_object_ref (service);
+	signal_closure->alert_type = type;
+	signal_closure->alert_message = g_strdup (message);
+
+	camel_session_idle_add (
+		session, G_PRIORITY_LOW,
+		session_emit_user_alert_cb,
+		signal_closure,
+		(GDestroyNotify) signal_closure_free);
+}
+
 /**
  * camel_session_lookup_addressbook:
  *
@@ -1576,40 +1459,6 @@ camel_session_submit_job (CamelSession *session,
 		job_data, (GDestroyNotify) NULL);
 }
 
-/**
- * camel_session_get_check_junk:
- * @session: a #CamelSession
- *
- * Do we have to check incoming messages to be junk?
- *
- * Returns: whether or not we are checking incoming messages for junk
- **/
-gboolean
-camel_session_get_check_junk (CamelSession *session)
-{
-	g_return_val_if_fail (CAMEL_IS_SESSION (session), FALSE);
-
-	return session->priv->check_junk;
-}
-
-/**
- * camel_session_set_check_junk:
- * @session: a #CamelSession
- * @check_junk: whether to check incoming messages for junk
- *
- * Set check_junk flag, if set, incoming mail will be checked for being junk.
- **/
-void
-camel_session_set_check_junk (CamelSession *session,
-                              gboolean check_junk)
-{
-	g_return_if_fail (CAMEL_IS_SESSION (session));
-
-	session->priv->check_junk = check_junk;
-
-	g_object_notify (G_OBJECT (session), "check-junk");
-}
-
 /**
  * camel_session_set_junk_headers:
  *
@@ -1694,6 +1543,116 @@ camel_session_authenticate_sync (CamelSession *session,
 	return success;
 }
 
+/* Helper for camel_session_authenticate() */
+static void
+session_authenticate_thread (GTask *task,
+                             gpointer source_object,
+                             gpointer task_data,
+                             GCancellable *cancellable)
+{
+	gboolean success;
+	AsyncContext *async_context;
+	GError *local_error = NULL;
+
+	async_context = (AsyncContext *) task_data;
+
+	success = camel_session_authenticate_sync (
+		CAMEL_SESSION (source_object),
+		async_context->service,
+		async_context->auth_mechanism,
+		cancellable, &local_error);
+
+	if (local_error != NULL) {
+		g_task_return_error (task, local_error);
+	} else {
+		g_task_return_boolean (task, success);
+	}
+}
+
+/**
+ * camel_session_authenticate:
+ * @session: a #CamelSession
+ * @service: a #CamelService
+ * @mechanism: a SASL mechanism name, or %NULL
+ * @io_priority: the I/O priority for the request
+ * @cancellable: optional #GCancellable object, or %NULL
+ * @callback: a #GAsyncReadyCallback to call when the request is satisfied
+ * @user_data: data to pass to the callback function
+ *
+ * Asynchronously authenticates @service, which may involve repeated calls
+ * to camel_service_authenticate() or camel_service_authenticate_sync().
+ * A #CamelSession subclass is largely responsible for implementing this,
+ * and should handle things like user prompts and secure password storage.
+ * These issues are out-of-scope for Camel.
+ *
+ * When the operation is finished, @callback will be called.  You can
+ * then call camel_session_authenticate_finish() to get the result of
+ * the operation.
+ *
+ * Since: 3.4
+ **/
+void
+camel_session_authenticate (CamelSession *session,
+                            CamelService *service,
+                            const gchar *mechanism,
+                            gint io_priority,
+                            GCancellable *cancellable,
+                            GAsyncReadyCallback callback,
+                            gpointer user_data)
+{
+	GTask *task;
+	AsyncContext *async_context;
+
+	g_return_if_fail (CAMEL_IS_SESSION (session));
+	g_return_if_fail (CAMEL_IS_SERVICE (service));
+
+	async_context = g_slice_new0 (AsyncContext);
+	async_context->service = g_object_ref (service);
+	async_context->auth_mechanism = g_strdup (mechanism);
+
+	task = g_task_new (session, cancellable, callback, user_data);
+	g_task_set_source_tag (task, camel_session_authenticate);
+	g_task_set_priority (task, io_priority);
+
+	g_task_set_task_data (
+		task, async_context,
+		(GDestroyNotify) async_context_free);
+
+	g_task_run_in_thread (task, session_authenticate_thread);
+
+	g_object_unref (task);
+}
+
+/**
+ * camel_session_authenticate_finish:
+ * @session: a #CamelSession
+ * @result: a #GAsyncResult
+ * @error: return location for a #GError, or %NULL
+ *
+ * Finishes the operation started with camel_session_authenticate().
+ *
+ * If an error occurred, or if authentication was aborted, the function
+ * sets @error and returns %FALSE.
+ *
+ * Returns: %TRUE on success, %FALSE on failure
+ *
+ * Since: 3.4
+ **/
+gboolean
+camel_session_authenticate_finish (CamelSession *session,
+                                   GAsyncResult *result,
+                                   GError **error)
+{
+	g_return_val_if_fail (CAMEL_IS_SESSION (session), FALSE);
+	g_return_val_if_fail (g_task_is_valid (result, session), FALSE);
+
+	g_return_val_if_fail (
+		g_async_result_is_tagged (
+		result, camel_session_authenticate), FALSE);
+
+	return g_task_propagate_boolean (G_TASK (result), error);
+}
+
 /**
  * camel_session_forward_to_sync:
  * @session: a #CamelSession
@@ -1737,79 +1696,31 @@ camel_session_forward_to_sync (CamelSession *session,
 	return success;
 }
 
-/**
- * camel_session_authenticate:
- * @session: a #CamelSession
- * @service: a #CamelService
- * @mechanism: a SASL mechanism name, or %NULL
- * @io_priority: the I/O priority for the request
- * @cancellable: optional #GCancellable object, or %NULL
- * @callback: a #GAsyncReadyCallback to call when the request is satisfied
- * @user_data: data to pass to the callback function
- *
- * Asynchronously authenticates @service, which may involve repeated calls
- * to camel_service_authenticate() or camel_service_authenticate_sync().
- * A #CamelSession subclass is largely responsible for implementing this,
- * and should handle things like user prompts and secure password storage.
- * These issues are out-of-scope for Camel.
- *
- * When the operation is finished, @callback will be called.  You can
- * then call camel_session_authenticate_finish() to get the result of
- * the operation.
- *
- * Since: 3.4
- **/
-void
-camel_session_authenticate (CamelSession *session,
-                            CamelService *service,
-                            const gchar *mechanism,
-                            gint io_priority,
-                            GCancellable *cancellable,
-                            GAsyncReadyCallback callback,
-                            gpointer user_data)
+/* Helper for camel_session_forward_to() */
+static void
+session_forward_to_thread (GTask *task,
+                           gpointer source_object,
+                           gpointer task_data,
+                           GCancellable *cancellable)
 {
-	CamelSessionClass *class;
+	gboolean success;
+	AsyncContext *async_context;
+	GError *local_error = NULL;
 
-	g_return_if_fail (CAMEL_IS_SESSION (session));
-	g_return_if_fail (CAMEL_IS_SERVICE (service));
+	async_context = (AsyncContext *) task_data;
 
-	class = CAMEL_SESSION_GET_CLASS (session);
-	g_return_if_fail (class->authenticate != NULL);
+	success = camel_session_forward_to_sync (
+		CAMEL_SESSION (source_object),
+		async_context->folder,
+		async_context->message,
+		async_context->address,
+		cancellable, &local_error);
 
-	class->authenticate (
-		session, service, mechanism, io_priority,
-		cancellable, callback, user_data);
-}
-
-/**
- * camel_session_authenticate_finish:
- * @session: a #CamelSession
- * @result: a #GAsyncResult
- * @error: return location for a #GError, or %NULL
- *
- * Finishes the operation started with camel_session_authenticate().
- *
- * If an error occurred, or if authentication was aborted, the function
- * sets @error and returns %FALSE.
- *
- * Returns: %TRUE on success, %FALSE on failure
- *
- * Since: 3.4
- **/
-gboolean
-camel_session_authenticate_finish (CamelSession *session,
-                                   GAsyncResult *result,
-                                   GError **error)
-{
-	CamelSessionClass *class;
-
-	g_return_val_if_fail (CAMEL_IS_SESSION (session), FALSE);
-	g_return_val_if_fail (G_IS_ASYNC_RESULT (result), FALSE);
-
-	class = CAMEL_SESSION_GET_CLASS (session);
-	g_return_val_if_fail (class->authenticate_finish != NULL, FALSE);
-
-	return class->authenticate_finish (session, result, error);
+	if (local_error != NULL) {
+		g_task_return_error (task, local_error);
+	} else {
+		g_task_return_boolean (task, success);
+	}
 }
 
 /**
@@ -1842,19 +1753,30 @@ camel_session_forward_to (CamelSession *session,
                           GAsyncReadyCallback callback,
                           gpointer user_data)
 {
-	CamelSessionClass *class;
+	GTask *task;
+	AsyncContext *async_context;
 
 	g_return_if_fail (CAMEL_IS_SESSION (session));
 	g_return_if_fail (CAMEL_IS_FOLDER (folder));
 	g_return_if_fail (CAMEL_IS_MIME_MESSAGE (message));
 	g_return_if_fail (address != NULL);
 
-	class = CAMEL_SESSION_GET_CLASS (session);
-	g_return_if_fail (class->forward_to != NULL);
+	async_context = g_slice_new0 (AsyncContext);
+	async_context->folder = g_object_ref (folder);
+	async_context->message = g_object_ref (message);
+	async_context->address = g_strdup (address);
 
-	class->forward_to (
-		session, folder, message, address,
-		io_priority, cancellable, callback, user_data);
+	task = g_task_new (session, cancellable, callback, user_data);
+	g_task_set_source_tag (task, camel_session_forward_to);
+	g_task_set_priority (task, io_priority);
+
+	g_task_set_task_data (
+		task, async_context,
+		(GDestroyNotify) async_context_free);
+
+	g_task_run_in_thread (task, session_forward_to_thread);
+
+	g_object_unref (task);
 }
 
 /**
@@ -1876,14 +1798,13 @@ camel_session_forward_to_finish (CamelSession *session,
                                  GAsyncResult *result,
                                  GError **error)
 {
-	CamelSessionClass *class;
-
 	g_return_val_if_fail (CAMEL_IS_SESSION (session), FALSE);
-	g_return_val_if_fail (G_IS_ASYNC_RESULT (result), FALSE);
+	g_return_val_if_fail (g_task_is_valid (result, session), FALSE);
 
-	class = CAMEL_SESSION_GET_CLASS (session);
-	g_return_val_if_fail (class->forward_to_finish != NULL, FALSE);
+	g_return_val_if_fail (
+		g_async_result_is_tagged (
+		result, camel_session_forward_to), FALSE);
 
-	return class->forward_to_finish (session, result, error);
+	return g_task_propagate_boolean (G_TASK (result), error);
 }
 
