@@ -113,11 +113,6 @@ struct _CamelSessionClass {
 						 CamelService *service,
 						 const gchar *item,
 						 GError **error);
-	gint		(*alert_user)		(CamelSession *session,
-						 CamelSessionAlertType type,
-						 const gchar *prompt,
-						 GList *button_captions,
-						 GCancellable *cancellable);
 	CamelCertTrust	(*trust_prompt)		(CamelSession *session,
 						 CamelService *service,
 						 GTlsCertificate *certificate,
@@ -142,28 +137,8 @@ struct _CamelSessionClass {
 						 GCancellable *cancellable,
 						 GError **error);
 
-	/* Asynchronous I/O Methods (all have defaults) */
-	void		(*authenticate)		(CamelSession *session,
-						 CamelService *service,
-						 const gchar *mechanism,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-	gboolean	(*authenticate_finish)	(CamelSession *session,
-						 GAsyncResult *result,
-						 GError **error);
-	void		(*forward_to)		(CamelSession *session,
-						 CamelFolder *folder,
-						 CamelMimeMessage *message,
-						 const gchar *address,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-	gboolean	(*forward_to_finish)	(CamelSession *session,
-						 GAsyncResult *result,
-						 GError **error);
+	/* Reserved slots for methods. */
+	gpointer reserved_for_methods[4];
 
 	/* Signals */
 	void		(*job_started)		(CamelSession *session,
@@ -171,6 +146,10 @@ struct _CamelSessionClass {
 	void		(*job_finished)		(CamelSession *session,
 						 GCancellable *cancellable,
 						 const GError *error);
+	void		(*user_alert)		(CamelSession *session,
+						 CamelService *service,
+						 CamelSessionAlertType type,
+						 const gchar *message);
 };
 
 GType		camel_session_get_type		(void);
@@ -203,15 +182,14 @@ gboolean	camel_session_forget_password	(CamelSession *session,
 						 CamelService *service,
 						 const gchar *item,
 						 GError **error);
-gint		camel_session_alert_user	(CamelSession *session,
-						 CamelSessionAlertType type,
-						 const gchar *prompt,
-						 GList *button_captions,
-						 GCancellable *cancellable);
 CamelCertTrust	camel_session_trust_prompt	(CamelSession *session,
 						 CamelService *service,
 						 GTlsCertificate *certificate,
 						 GTlsCertificateFlags errors);
+void		camel_session_user_alert	(CamelSession *session,
+						 CamelService *service,
+						 CamelSessionAlertType type,
+						 const gchar *message);
 gboolean	camel_session_get_online	(CamelSession *session);
 void		camel_session_set_online	(CamelSession *session,
 						 gboolean online);
@@ -223,9 +201,6 @@ CamelJunkFilter *
 		camel_session_get_junk_filter	(CamelSession *session);
 void		camel_session_set_junk_filter	(CamelSession *session,
 						 CamelJunkFilter *junk_filter);
-gboolean	camel_session_get_check_junk	(CamelSession *session);
-void		camel_session_set_check_junk	(CamelSession *session,
-						 gboolean check_junk);
 guint		camel_session_idle_add		(CamelSession *session,
 						 gint priority,
 						 GSourceFunc function,
@@ -278,10 +253,6 @@ gboolean	camel_session_forward_to_finish	(CamelSession *session,
 						 GAsyncResult *result,
 						 GError **error);
 
-#ifndef CAMEL_DISABLE_DEPRECATED
-GMainContext *	camel_session_get_main_context	(CamelSession *session);
-#endif /* CAMEL_DISABLE_DEPRECATED */
-
 G_END_DECLS
 
 #endif /* CAMEL_SESSION_H */
