@@ -98,8 +98,8 @@ typedef struct _CamelStorePrivate CamelStorePrivate;
 
 /* open mode for folder */
 typedef enum {
-	CAMEL_STORE_FOLDER_CREATE     = 1 << 0,
-	CAMEL_STORE_FOLDER_EXCL       = 1 << 1,
+	CAMEL_STORE_FOLDER_CREATE = 1 << 0,
+	CAMEL_STORE_FOLDER_EXCL = 1 << 1,
 	CAMEL_STORE_FOLDER_BODY_INDEX = 1 << 2,
 	CAMEL_STORE_FOLDER_PRIVATE    = 1 << 3  /* a private folder that
                                                    should not show up in
@@ -187,103 +187,8 @@ struct _CamelStoreClass {
 						 GCancellable *cancellable,
 						 GError **error);
 
-	/* Asyncrhonous I/O Methods (all have defaults) */
-	void		(*get_folder)		(CamelStore *store,
-						 const gchar *folder_name,
-						 CamelStoreGetFolderFlags flags,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-	CamelFolder *	(*get_folder_finish)	(CamelStore *store,
-						 GAsyncResult *result,
-						 GError **error);
-	void		(*get_folder_info)	(CamelStore *store,
-						 const gchar *top,
-						 CamelStoreGetFolderInfoFlags flags,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-	CamelFolderInfo *
-			(*get_folder_info_finish)
-						(CamelStore *store,
-						 GAsyncResult *result,
-						 GError **error);
-	void		(*get_inbox_folder)	(CamelStore *store,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-	CamelFolder *	(*get_inbox_folder_finish)
-						(CamelStore *store,
-						 GAsyncResult *result,
-						 GError **error);
-	void		(*get_junk_folder)	(CamelStore *store,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-	CamelFolder *	(*get_junk_folder_finish)
-						(CamelStore *store,
-						 GAsyncResult *result,
-						 GError **error);
-	void		(*get_trash_folder)	(CamelStore *store,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-	CamelFolder *	(*get_trash_folder_finish)
-						(CamelStore *store,
-						 GAsyncResult *result,
-						 GError **error);
-	void		(*create_folder)	(CamelStore *store,
-						 const gchar *parent_name,
-						 const gchar *folder_name,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-	CamelFolderInfo *
-			(*create_folder_finish)	(CamelStore *store,
-						 GAsyncResult *result,
-						 GError **error);
-	void		(*delete_folder)	(CamelStore *store,
-						 const gchar *folder_name,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-	gboolean	(*delete_folder_finish)	(CamelStore *store,
-						 GAsyncResult *result,
-						 GError **error);
-	void		(*rename_folder)	(CamelStore *store,
-						 const gchar *old_name,
-						 const gchar *new_name,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-	gboolean	(*rename_folder_finish)	(CamelStore *store,
-						 GAsyncResult *result,
-						 GError **error);
-	void		(*synchronize)		(CamelStore *store,
-						 gboolean expunge,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-	gboolean	(*synchronize_finish)	(CamelStore *store,
-						 GAsyncResult *result,
-						 GError **error);
-	void		(*noop)			(CamelStore *store,
-						 gint io_priority,
-						 GCancellable *cancellable,
-						 GAsyncReadyCallback callback,
-						 gpointer user_data);
-	gboolean	(*noop_finish)		(CamelStore *store,
-						 GAsyncResult *result,
-						 GError **error);
+	/* Reserved slots for methods. */
+	gpointer reserved_for_methods[20];
 
 	/* Signals */
 	void		(*folder_created)	(CamelStore *store,
@@ -325,8 +230,6 @@ CamelFolderInfo *
 gboolean	camel_store_can_refresh_folder	(CamelStore *store,
 						 CamelFolderInfo *info,
 						 GError **error);
-void		camel_store_lock		(CamelStore *store);
-void		camel_store_unlock		(CamelStore *store);
 
 CamelFolder *	camel_store_get_folder_sync	(CamelStore *store,
 						 const gchar *folder_name,
