@@ -21,6 +21,7 @@
 #include <config.h>
 #include <glib/gi18n-lib.h>
 
+#include "camel-async-closure.h"
 #include "camel-debug.h"
 #include "camel-session.h"
 #include "camel-vtrash-folder.h"
@@ -144,148 +145,9 @@ subscribable_delete_cached_folder (CamelStore *store,
 	g_object_unref (folder);
 }
 
-static void
-subscribable_subscribe_folder_thread (GSimpleAsyncResult *simple,
-                                      GObject *object,
-                                      GCancellable *cancellable)
-{
-	AsyncContext *async_context;
-	GError *error = NULL;
-
-	async_context = g_simple_async_result_get_op_res_gpointer (simple);
-
-	camel_subscribable_subscribe_folder_sync (
-		CAMEL_SUBSCRIBABLE (object),
-		async_context->folder_name,
-		cancellable, &error);
-
-	if (error != NULL)
-		g_simple_async_result_take_error (simple, error);
-}
-
-static void
-subscribable_subscribe_folder (CamelSubscribable *subscribable,
-                               const gchar *folder_name,
-                               gint io_priority,
-                               GCancellable *cancellable,
-                               GAsyncReadyCallback callback,
-                               gpointer user_data)
-{
-	GSimpleAsyncResult *simple;
-	AsyncContext *async_context;
-
-	async_context = g_slice_new0 (AsyncContext);
-	async_context->folder_name = g_strdup (folder_name);
-
-	simple = g_simple_async_result_new (
-		G_OBJECT (subscribable), callback,
-		user_data, subscribable_subscribe_folder);
-
-	g_simple_async_result_set_check_cancellable (simple, cancellable);
-
-	g_simple_async_result_set_op_res_gpointer (
-		simple, async_context, (GDestroyNotify) async_context_free);
-
-	g_simple_async_result_run_in_thread (
-		simple, subscribable_subscribe_folder_thread,
-		io_priority, cancellable);
-
-	g_object_unref (simple);
-}
-
-static gboolean
-subscribable_subscribe_folder_finish (CamelSubscribable *subscribable,
-                                      GAsyncResult *result,
-                                      GError **error)
-{
-	GSimpleAsyncResult *simple;
-
-	g_return_val_if_fail (
-		g_simple_async_result_is_valid (
-		result, G_OBJECT (subscribable),
-		subscribable_subscribe_folder), FALSE);
-
-	simple = G_SIMPLE_ASYNC_RESULT (result);
-
-	/* Assume success unless a GError is set. */
-	return !g_simple_async_result_propagate_error (simple, error);
-}
-
-static void
-subscribable_unsubscribe_folder_thread (GSimpleAsyncResult *simple,
-                                        GObject *object,
-                                        GCancellable *cancellable)
-{
-	AsyncContext *async_context;
-	GError *error = NULL;
-
-	async_context = g_simple_async_result_get_op_res_gpointer (simple);
-
-	camel_subscribable_unsubscribe_folder_sync (
-		CAMEL_SUBSCRIBABLE (object),
-		async_context->folder_name,
-		cancellable, &error);
-
-	if (error != NULL)
-		g_simple_async_result_take_error (simple, error);
-}
-
-static void
-subscribable_unsubscribe_folder (CamelSubscribable *subscribable,
-                                 const gchar *folder_name,
-                                 gint io_priority,
-                                 GCancellable *cancellable,
-                                 GAsyncReadyCallback callback,
-                                 gpointer user_data)
-{
-	GSimpleAsyncResult *simple;
-	AsyncContext *async_context;
-
-	async_context = g_slice_new0 (AsyncContext);
-	async_context->folder_name = g_strdup (folder_name);
-
-	simple = g_simple_async_result_new (
-		G_OBJECT (subscribable), callback,
-		user_data, subscribable_unsubscribe_folder);
-
-	g_simple_async_result_set_check_cancellable (simple, cancellable);
-
-	g_simple_async_result_set_op_res_gpointer (
-		simple, async_context, (GDestroyNotify) async_context_free);
-
-	g_simple_async_result_run_in_thread (
-		simple, subscribable_unsubscribe_folder_thread,
-		io_priority, cancellable);
-
-	g_object_unref (simple);
-}
-
-static gboolean
-subscribable_unsubscribe_folder_finish (CamelSubscribable *subscribable,
-                                        GAsyncResult *result,
-                                        GError **error)
-{
-	GSimpleAsyncResult *simple;
-
-	g_return_val_if_fail (
-		g_simple_async_result_is_valid (
-		result, G_OBJECT (subscribable),
-		subscribable_unsubscribe_folder), FALSE);
-
-	simple = G_SIMPLE_ASYNC_RESULT (result);
-
-	/* Assume success unless a GError is set. */
-	return !g_simple_async_result_propagate_error (simple, error);
-}
-
 static void
 camel_subscribable_default_init (CamelSubscribableInterface *interface)
 {
-	interface->subscribe_folder = subscribable_subscribe_folder;
-	interface->subscribe_folder_finish = subscribable_subscribe_folder_finish;
-	interface->unsubscribe_folder = subscribable_unsubscribe_folder;
-	interface->unsubscribe_folder_finish = subscribable_unsubscribe_folder_finish;
-
 	signals[FOLDER_SUBSCRIBED] = g_signal_new (
 		"folder-subscribed",
 		G_OBJECT_CLASS_TYPE (interface),
@@ -325,7 +187,6 @@ camel_subscribable_folder_is_subscribed (CamelSubscribable *subscribable,
                                          const gchar *folder_name)
 {
 	CamelSubscribableInterface *interface;
-	gboolean is_subscribed;
 
 	g_return_val_if_fail (CAMEL_IS_SUBSCRIBABLE (subscribable), FALSE);
 	g_return_val_if_fail (folder_name != NULL, FALSE);
@@ -333,14 +194,7 @@ camel_subscribable_folder_is_subscribed (CamelSubscribable *subscribable,
 	interface = CAMEL_SUBSCRIBABLE_GET_INTERFACE (subscribable);
 	g_return_val_if_fail (interface->folder_is_subscribed != NULL, FALSE);
 
-	camel_store_lock (CAMEL_STORE (subscribable));
-
-	is_subscribed = interface->folder_is_subscribed (
-		subscribable, folder_name);
-
-	camel_store_unlock (CAMEL_STORE (subscribable));
-
-	return is_subscribed;
+	return interface->folder_is_subscribed (subscribable, folder_name);
 }
 
 /**
@@ -362,44 +216,76 @@ camel_subscribable_subscribe_folder_sync (CamelSubscribable *subscribable,
                                           GCancellable *cancellable,
                                           GError **error)
 {
-	CamelSubscribableInterface *interface;
-	const gchar *message;
+	CamelAsyncClosure *closure;
+	GAsyncResult *result;
 	gboolean success;
 
 	g_return_val_if_fail (CAMEL_IS_SUBSCRIBABLE (subscribable), FALSE);
 	g_return_val_if_fail (folder_name != NULL, FALSE);
 
+	closure = camel_async_closure_new ();
+
+	camel_subscribable_subscribe_folder (
+		subscribable, folder_name,
+		G_PRIORITY_DEFAULT, cancellable,
+		camel_async_closure_callback, closure);
+
+	result = camel_async_closure_wait (closure);
+
+	success = camel_subscribable_subscribe_folder_finish (
+		subscribable, result, error);
+
+	camel_async_closure_free (closure);
+
+	return success;
+}
+
+/* Helper for camel_subscribable_subscribe_folder() */
+static void
+subscribable_subscribe_folder_thread (GTask *task,
+                                      gpointer source_object,
+                                      gpointer task_data,
+                                      GCancellable *cancellable)
+{
+	CamelSubscribable *subscribable;
+	CamelSubscribableInterface *interface;
+	const gchar *folder_name;
+	const gchar *message;
+	gboolean success;
+	AsyncContext *async_context;
+	GError *local_error = NULL;
+
+	subscribable = CAMEL_SUBSCRIBABLE (source_object);
+	async_context = (AsyncContext *) task_data;
+
+	folder_name = async_context->folder_name;
+
 	interface = CAMEL_SUBSCRIBABLE_GET_INTERFACE (subscribable);
-	g_return_val_if_fail (interface->subscribe_folder_sync != NULL, FALSE);
-
-	camel_store_lock (CAMEL_STORE (subscribable));
-
-	/* Check for cancellation after locking. */
-	if (g_cancellable_set_error_if_cancelled (cancellable, error)) {
-		success = FALSE;
-		goto exit;
-	}
+	g_return_if_fail (interface->subscribe_folder_sync != NULL);
 
 	/* Need to establish a connection before subscribing. */
-	success = camel_service_connect_sync (
-		CAMEL_SERVICE (subscribable), cancellable, error);
-	if (!success)
-		goto exit;
+	camel_service_connect_sync (
+		CAMEL_SERVICE (subscribable), cancellable, &local_error);
+	if (local_error != NULL) {
+		g_task_return_error (task, local_error);
+		return;
+	}
 
 	message = _("Subscribing to folder '%s'");
 	camel_operation_push_message (cancellable, message, folder_name);
 
 	success = interface->subscribe_folder_sync (
-		subscribable, folder_name, cancellable, error);
-	CAMEL_CHECK_GERROR (
-		subscribable, subscribe_folder_sync, success, error);
+		subscribable, folder_name, cancellable, &local_error);
+	CAMEL_CHECK_LOCAL_GERROR (
+		subscribable, subscribe_folder_sync, success, local_error);
 
 	camel_operation_pop_message (cancellable);
 
-exit:
-	camel_store_unlock (CAMEL_STORE (subscribable));
-
-	return success;
+	if (local_error != NULL) {
+		g_task_return_error (task, local_error);
+	} else {
+		g_task_return_boolean (task, success);
+	}
 }
 
 /**
@@ -427,17 +313,30 @@ camel_subscribable_subscribe_folder (CamelSubscribable *subscribable,
                                      GAsyncReadyCallback callback,
                                      gpointer user_data)
 {
-	CamelSubscribableInterface *interface;
+	GTask *task;
+	CamelService *service;
+	AsyncContext *async_context;
 
 	g_return_if_fail (CAMEL_IS_SUBSCRIBABLE (subscribable));
 	g_return_if_fail (folder_name != NULL);
 
-	interface = CAMEL_SUBSCRIBABLE_GET_INTERFACE (subscribable);
-	g_return_if_fail (interface->subscribe_folder != NULL);
+	service = CAMEL_SERVICE (subscribable);
 
-	interface->subscribe_folder (
-		subscribable, folder_name, io_priority,
-		cancellable, callback, user_data);
+	async_context = g_slice_new0 (AsyncContext);
+	async_context->folder_name = g_strdup (folder_name);
+
+	task = g_task_new (subscribable, cancellable, callback, user_data);
+	g_task_set_source_tag (task, camel_subscribable_subscribe_folder);
+	g_task_set_priority (task, io_priority);
+
+	g_task_set_task_data (
+		task, async_context,
+		(GDestroyNotify) async_context_free);
+
+	camel_service_queue_task (
+		service, task, subscribable_subscribe_folder_thread);
+
+	g_object_unref (task);
 }
 
 /**
@@ -457,17 +356,14 @@ camel_subscribable_subscribe_folder_finish (CamelSubscribable *subscribable,
                                             GAsyncResult *result,
                                             GError **error)
 {
-	CamelSubscribableInterface *interface;
-
 	g_return_val_if_fail (CAMEL_IS_SUBSCRIBABLE (subscribable), FALSE);
-	g_return_val_if_fail (G_IS_ASYNC_RESULT (result), FALSE);
+	g_return_val_if_fail (g_task_is_valid (result, subscribable), FALSE);
 
-	interface = CAMEL_SUBSCRIBABLE_GET_INTERFACE (subscribable);
 	g_return_val_if_fail (
-		interface->subscribe_folder_finish != NULL, FALSE);
+		g_async_result_is_tagged (
+		result, camel_subscribable_subscribe_folder), FALSE);
 
-	return interface->subscribe_folder_finish (
-		subscribable, result, error);
+	return g_task_propagate_boolean (G_TASK (result), error);
 }
 
 /**
@@ -489,38 +385,68 @@ camel_subscribable_unsubscribe_folder_sync (CamelSubscribable *subscribable,
                                             GCancellable *cancellable,
                                             GError **error)
 {
-	CamelSubscribableInterface *interface;
-	const gchar *message;
+	CamelAsyncClosure *closure;
+	GAsyncResult *result;
 	gboolean success;
 
 	g_return_val_if_fail (CAMEL_IS_SUBSCRIBABLE (subscribable), FALSE);
 	g_return_val_if_fail (folder_name != NULL, FALSE);
 
+	closure = camel_async_closure_new ();
+
+	camel_subscribable_unsubscribe_folder (
+		subscribable, folder_name,
+		G_PRIORITY_DEFAULT, cancellable,
+		camel_async_closure_callback, closure);
+
+	result = camel_async_closure_wait (closure);
+
+	success = camel_subscribable_unsubscribe_folder_finish (
+		subscribable, result, error);
+
+	camel_async_closure_free (closure);
+
+	return success;
+}
+
+/* Helper for camel_subscribable_unsubscribe_folder() */
+static void
+subscribable_unsubscribe_folder_thread (GTask *task,
+                                        gpointer source_object,
+                                        gpointer task_data,
+                                        GCancellable *cancellable)
+{
+	CamelSubscribable *subscribable;
+	CamelSubscribableInterface *interface;
+	const gchar *folder_name;
+	const gchar *message;
+	gboolean success;
+	AsyncContext *async_context;
+	GError *local_error = NULL;
+
+	subscribable = CAMEL_SUBSCRIBABLE (source_object);
+	async_context = (AsyncContext *) task_data;
+
+	folder_name = async_context->folder_name;
+
 	interface = CAMEL_SUBSCRIBABLE_GET_INTERFACE (subscribable);
-	g_return_val_if_fail (
-		interface->unsubscribe_folder_sync != NULL, FALSE);
-
-	camel_store_lock (CAMEL_STORE (subscribable));
-
-	/* Check for cancellation after locking. */
-	if (g_cancellable_set_error_if_cancelled (cancellable, error)) {
-		success = FALSE;
-		goto exit;
-	}
+	g_return_if_fail (interface->unsubscribe_folder_sync != NULL);
 
 	/* Need to establish a connection before unsubscribing. */
-	success = camel_service_connect_sync (
-		CAMEL_SERVICE (subscribable), cancellable, error);
-	if (!success)
-		goto exit;
+	camel_service_connect_sync (
+		CAMEL_SERVICE (subscribable), cancellable, &local_error);
+	if (local_error != NULL) {
+		g_task_return_error (task, local_error);
+		return;
+	}
 
 	message = _("Unsubscribing from folder '%s'");
 	camel_operation_push_message (cancellable, message, folder_name);
 
 	success = interface->unsubscribe_folder_sync (
-		subscribable, folder_name, cancellable, error);
-	CAMEL_CHECK_GERROR (
-		subscribable, unsubscribe_folder_sync, success, error);
+		subscribable, folder_name, cancellable, &local_error);
+	CAMEL_CHECK_LOCAL_GERROR (
+		subscribable, unsubscribe_folder_sync, success, local_error);
 
 	if (success)
 		subscribable_delete_cached_folder (
@@ -528,10 +454,11 @@ camel_subscribable_unsubscribe_folder_sync (CamelSubscribable *subscribable,
 
 	camel_operation_pop_message (cancellable);
 
-exit:
-	camel_store_unlock (CAMEL_STORE (subscribable));
-
-	return success;
+	if (local_error != NULL) {
+		g_task_return_error (task, local_error);
+	} else {
+		g_task_return_boolean (task, success);
+	}
 }
 
 /**
@@ -559,17 +486,30 @@ camel_subscribable_unsubscribe_folder (CamelSubscribable *subscribable,
                                        GAsyncReadyCallback callback,
                                        gpointer user_data)
 {
-	CamelSubscribableInterface *interface;
+	GTask *task;
+	CamelService *service;
+	AsyncContext *async_context;
 
 	g_return_if_fail (CAMEL_IS_SUBSCRIBABLE (subscribable));
 	g_return_if_fail (folder_name != NULL);
 
-	interface = CAMEL_SUBSCRIBABLE_GET_INTERFACE (subscribable);
-	g_return_if_fail (interface->unsubscribe_folder != NULL);
+	service = CAMEL_SERVICE (subscribable);
 
-	interface->unsubscribe_folder (
-		subscribable, folder_name, io_priority,
-		cancellable, callback, user_data);
+	async_context = g_slice_new0 (AsyncContext);
+	async_context->folder_name = g_strdup (folder_name);
+
+	task = g_task_new (subscribable, cancellable, callback, user_data);
+	g_task_set_source_tag (task, camel_subscribable_unsubscribe_folder);
+	g_task_set_priority (task, io_priority);
+
+	g_task_set_task_data (
+		task, async_context,
+		(GDestroyNotify) async_context_free);
+
+	camel_service_queue_task (
+		service, task, subscribable_unsubscribe_folder_thread);
+
+	g_object_unref (task);
 }
 
 /**
@@ -589,17 +529,14 @@ camel_subscribable_unsubscribe_folder_finish (CamelSubscribable *subscribable,
                                               GAsyncResult *result,
                                               GError **error)
 {
-	CamelSubscribableInterface *interface;
-
 	g_return_val_if_fail (CAMEL_IS_SUBSCRIBABLE (subscribable), FALSE);
-	g_return_val_if_fail (G_IS_ASYNC_RESULT (result), FALSE);
+	g_return_val_if_fail (g_task_is_valid (result, subscribable), FALSE);
 
-	interface = CAMEL_SUBSCRIBABLE_GET_INTERFACE (subscribable);
 	g_return_val_if_fail (
-		interface->unsubscribe_folder_finish != NULL, FALSE);
+		g_async_result_is_tagged (
+		result, camel_subscribable_unsubscribe_folder), FALSE);
 
-	return interface->unsubscribe_folder_finish (
-		subscribable, result, error);
+	return g_task_propagate_boolean (G_TASK (result), error);
 }
 
 /**
