@@ -28,6 +28,7 @@
 #endif
 
 #include "camel-address.h"
+#include "camel-async-closure.h"
 #include "camel-debug.h"
 #include "camel-mime-message.h"
 #include "camel-transport.h"
@@ -39,11 +40,10 @@
 typedef struct _AsyncContext AsyncContext;
 
 struct _CamelTransportPrivate {
-	GMutex send_lock;   /* for locking send operations */
+	gint placeholder;
 };
 
 struct _AsyncContext {
-	/* arguments */
 	CamelAddress *from;
 	CamelAddress *recipients;
 	CamelMimeMessage *message;
@@ -66,107 +66,16 @@ async_context_free (AsyncContext *async_context)
 	g_slice_free (AsyncContext, async_context);
 }
 
-static void
-transport_finalize (GObject *object)
-{
-	CamelTransportPrivate *priv;
-
-	priv = CAMEL_TRANSPORT_GET_PRIVATE (object);
-
-	g_mutex_clear (&priv->send_lock);
-
-	/* Chain up to parent's finalize() method. */
-	G_OBJECT_CLASS (camel_transport_parent_class)->finalize (object);
-}
-
-static void
-transport_send_to_thread (GSimpleAsyncResult *simple,
-                          GObject *object,
-                          GCancellable *cancellable)
-{
-	AsyncContext *async_context;
-	GError *error = NULL;
-
-	async_context = g_simple_async_result_get_op_res_gpointer (simple);
-
-	camel_transport_send_to_sync (
-		CAMEL_TRANSPORT (object), async_context->message,
-		async_context->from, async_context->recipients,
-		cancellable, &error);
-
-	if (error != NULL)
-		g_simple_async_result_take_error (simple, error);
-}
-
-static void
-transport_send_to (CamelTransport *transport,
-                   CamelMimeMessage *message,
-                   CamelAddress *from,
-                   CamelAddress *recipients,
-                   gint io_priority,
-                   GCancellable *cancellable,
-                   GAsyncReadyCallback callback,
-                   gpointer user_data)
-{
-	GSimpleAsyncResult *simple;
-	AsyncContext *async_context;
-
-	async_context = g_slice_new0 (AsyncContext);
-	async_context->from = g_object_ref (from);
-	async_context->recipients = g_object_ref (recipients);
-	async_context->message = g_object_ref (message);
-
-	simple = g_simple_async_result_new (
-		G_OBJECT (transport), callback, user_data, transport_send_to);
-
-	g_simple_async_result_set_check_cancellable (simple, cancellable);
-
-	g_simple_async_result_set_op_res_gpointer (
-		simple, async_context, (GDestroyNotify) async_context_free);
-
-	g_simple_async_result_run_in_thread (
-		simple, transport_send_to_thread, io_priority, cancellable);
-
-	g_object_unref (simple);
-}
-
-static gboolean
-transport_send_to_finish (CamelTransport *transport,
-                          GAsyncResult *result,
-                          GError **error)
-{
-	GSimpleAsyncResult *simple;
-
-	g_return_val_if_fail (
-		g_simple_async_result_is_valid (
-		result, G_OBJECT (transport), transport_send_to), FALSE);
-
-	simple = G_SIMPLE_ASYNC_RESULT (result);
-
-	/* Assume success unless a GError is set. */
-	return !g_simple_async_result_propagate_error (simple, error);
-}
-
 static void
 camel_transport_class_init (CamelTransportClass *class)
 {
-	GObjectClass *object_class;
-
 	g_type_class_add_private (class, sizeof (CamelTransportPrivate));
-
-	object_class = G_OBJECT_CLASS (class);
-	object_class->finalize = transport_finalize;
-
-	class->send_to = transport_send_to;
-	class->send_to_finish = transport_send_to_finish;
 }
 
 static void
 camel_transport_init (CamelTransport *transport)
 {
 	transport->priv = CAMEL_TRANSPORT_GET_PRIVATE (transport);
-
-	g_mutex_init (&transport->priv->send_lock);
 }
 
 /**
@@ -194,7 +103,8 @@ camel_transport_send_to_sync (CamelTransport *transport,
                               GCancellable *cancellable,
                               GError **error)
 {
-	CamelTransportClass *class;
+	CamelAsyncClosure *closure;
+	GAsyncResult *result;
 	gboolean success;
 
 	g_return_val_if_fail (CAMEL_IS_TRANSPORT (transport), FALSE);
@@ -202,26 +112,57 @@ camel_transport_send_to_sync (CamelTransport *transport,
 	g_return_val_if_fail (CAMEL_IS_ADDRESS (from), FALSE);
 	g_return_val_if_fail (CAMEL_IS_ADDRESS (recipients), FALSE);
 
-	class = CAMEL_TRANSPORT_GET_CLASS (transport);
-	g_return_val_if_fail (class->send_to_sync != NULL, FALSE);
+	closure = camel_async_closure_new ();
 
-	g_mutex_lock (&transport->priv->send_lock);
+	camel_transport_send_to (
+		transport, message, from, recipients,
+		G_PRIORITY_DEFAULT, cancellable,
+		camel_async_closure_callback, closure);
 
-	/* Check for cancellation after locking. */
-	if (g_cancellable_set_error_if_cancelled (cancellable, error)) {
-		g_mutex_unlock (&transport->priv->send_lock);
-		return FALSE;
-	}
+	result = camel_async_closure_wait (closure);
 
-	success = class->send_to_sync (
-		transport, message, from, recipients, cancellable, error);
-	CAMEL_CHECK_GERROR (transport, send_to_sync, success, error);
+	success = camel_transport_send_to_finish (transport, result, error);
 
-	g_mutex_unlock (&transport->priv->send_lock);
+	camel_async_closure_free (closure);
 
 	return success;
 }
 
+/* Helper for camel_transport_send_to() */
+static void
+transport_send_to_thread (GTask *task,
+                          gpointer source_object,
+                          gpointer task_data,
+                          GCancellable *cancellable)
+{
+	CamelTransport *transport;
+	CamelTransportClass *class;
+	AsyncContext *async_context;
+	gboolean success;
+	GError *local_error = NULL;
+
+	transport = CAMEL_TRANSPORT (source_object);
+	async_context = (AsyncContext *) task_data;
+
+	class = CAMEL_TRANSPORT_GET_CLASS (transport);
+	g_return_if_fail (class->send_to_sync != NULL);
+
+	success = class->send_to_sync (
+		CAMEL_TRANSPORT (source_object),
+		async_context->message,
+		async_context->from,
+		async_context->recipients,
+		cancellable, &local_error);
+	CAMEL_CHECK_LOCAL_GERROR (
+		transport, send_to_sync, success, local_error);
+
+	if (local_error != NULL) {
+		g_task_return_error (task, local_error);
+	} else {
+		g_task_return_boolean (task, success);
+	}
+}
+
 /**
  * camel_transport_send_to:
  * @transport: a #CamelTransport
@@ -252,19 +193,34 @@ camel_transport_send_to (CamelTransport *transport,
                          GAsyncReadyCallback callback,
                          gpointer user_data)
 {
-	CamelTransportClass *class;
+	GTask *task;
+	CamelService *service;
+	AsyncContext *async_context;
 
 	g_return_if_fail (CAMEL_IS_TRANSPORT (transport));
 	g_return_if_fail (CAMEL_IS_MIME_MESSAGE (message));
 	g_return_if_fail (CAMEL_IS_ADDRESS (from));
 	g_return_if_fail (CAMEL_IS_ADDRESS (recipients));
 
-	class = CAMEL_TRANSPORT_GET_CLASS (transport);
-	g_return_if_fail (class->send_to != NULL);
+	service = CAMEL_SERVICE (transport);
 
-	class->send_to (
-		transport, message, from, recipients, io_priority,
-		cancellable, callback, user_data);
+	async_context = g_slice_new0 (AsyncContext);
+	async_context->from = g_object_ref (from);
+	async_context->recipients = g_object_ref (recipients);
+	async_context->message = g_object_ref (message);
+
+	task = g_task_new (transport, cancellable, callback, user_data);
+	g_task_set_source_tag (task, camel_transport_send_to);
+	g_task_set_priority (task, io_priority);
+
+	g_task_set_task_data (
+		task, async_context,
+		(GDestroyNotify) async_context_free);
+
+	camel_service_queue_task (
+		service, task, transport_send_to_thread);
+
+	g_object_unref (task);
 }
 
 /**
@@ -284,13 +240,12 @@ camel_transport_send_to_finish (CamelTransport *transport,
                                 GAsyncResult *result,
                                 GError **error)
 {
-	CamelTransportClass *class;
-
 	g_return_val_if_fail (CAMEL_IS_TRANSPORT (transport), FALSE);
-	g_return_val_if_fail (G_IS_ASYNC_RESULT (result), FALSE);
+	g_return_val_if_fail (g_task_is_valid (result, transport), FALSE);
 
-	class = CAMEL_TRANSPORT_GET_CLASS (transport);
-	g_return_val_if_fail (class->send_to_finish != NULL, FALSE);
+	g_return_val_if_fail (
+		g_async_result_is_tagged (
+		result, camel_transport_send_to), FALSE);
 
-	return class->send_to_finish (transport, result, error);
+	return g_task_propagate_boolean (G_TASK (result), error);
 }
