@@ -19,8 +19,7 @@
 #ifndef CAMEL_IMAPX_COMMAND_H
 #define CAMEL_IMAPX_COMMAND_H
 
-#include <camel/camel.h>
-
+#include "camel-imapx-mailbox.h"
 #include "camel-imapx-utils.h"
 
 #define CAMEL_IS_IMAPX_COMMAND(command) \
@@ -35,10 +34,8 @@ struct _CamelIMAPXServer;
 typedef struct _CamelIMAPXCommand CamelIMAPXCommand;
 typedef struct _CamelIMAPXCommandPart CamelIMAPXCommandPart;
 
-typedef gboolean
-		(*CamelIMAPXCommandFunc)	(struct _CamelIMAPXServer *is,
-						 CamelIMAPXCommand *ic,
-						 GError **error);
+typedef void	(*CamelIMAPXCommandFunc)	(struct _CamelIMAPXServer *is,
+						 CamelIMAPXCommand *ic);
 
 typedef enum {
 	CAMEL_IMAPX_COMMAND_SIMPLE = 0,
@@ -74,9 +71,6 @@ struct _CamelIMAPXCommand {
 	/* Command name/type (e.g. FETCH) */
 	const gchar *name;
 
-	/* Folder to select */
-	CamelFolder *select;
-
 	/* Status for command, indicates it is complete if != NULL. */
 	struct _status_info *status;
 
@@ -92,7 +86,7 @@ struct _CamelIMAPXCommand {
 CamelIMAPXCommand *
 		camel_imapx_command_new		(struct _CamelIMAPXServer *is,
 						 const gchar *name,
-						 CamelFolder *select,
+						 CamelIMAPXMailbox *mailbox,
 						 const gchar *format,
 						 ...);
 CamelIMAPXCommand *
@@ -105,6 +99,8 @@ struct _CamelIMAPXJob *
 		camel_imapx_command_get_job	(CamelIMAPXCommand *ic);
 void		camel_imapx_command_set_job	(CamelIMAPXCommand *ic,
 						 struct _CamelIMAPXJob *job);
+CamelIMAPXMailbox *
+		camel_imapx_command_ref_mailbox	(CamelIMAPXCommand *ic);
 void		camel_imapx_command_add		(CamelIMAPXCommand *ic,
 						 const gchar *format,
 						 ...);
@@ -117,6 +113,8 @@ void		camel_imapx_command_add_part	(CamelIMAPXCommand *ic,
 void		camel_imapx_command_close	(CamelIMAPXCommand *ic);
 void		camel_imapx_command_wait	(CamelIMAPXCommand *ic);
 void		camel_imapx_command_done	(CamelIMAPXCommand *ic);
+void		camel_imapx_command_failed	(CamelIMAPXCommand *ic,
+						 const GError *error);
 gboolean	camel_imapx_command_set_error_if_failed
 						(CamelIMAPXCommand *ic,
 						 GError **error);
@@ -154,6 +152,10 @@ gboolean	camel_imapx_command_queue_remove
 void		camel_imapx_command_queue_delete_link
 						(CamelIMAPXCommandQueue *queue,
 						 GList *link);
+CamelIMAPXCommand *
+		camel_imapx_command_queue_ref_by_tag
+						(CamelIMAPXCommandQueue *queue,
+						 guint32 tag);
 
 G_END_DECLS
 
