@@ -26,6 +26,8 @@
 
 #include <camel/camel.h>
 
+#include "camel-imapx-mailbox.h"
+
 /* Standard GObject macros */
 #define CAMEL_TYPE_IMAPX_FOLDER \
 	(camel_imapx_folder_get_type ())
@@ -49,25 +51,17 @@ G_BEGIN_DECLS
 
 typedef struct _CamelIMAPXFolder CamelIMAPXFolder;
 typedef struct _CamelIMAPXFolderClass CamelIMAPXFolderClass;
+typedef struct _CamelIMAPXFolderPrivate CamelIMAPXFolderPrivate;
 
 struct _CamelIMAPXFolder {
 	CamelOfflineFolder parent;
+	CamelIMAPXFolderPrivate *priv;
 
-	gchar *raw_name;
 	CamelDataCache *cache;
 	CamelFolderSearch *search;
 
-	guint32 exists_on_server;
-	guint32 unread_on_server;
-	guint64 modseq_on_server;
-	guint64 uidvalidity_on_server;
-	guint32 uidnext_on_server;
-
-	/* hash table of UIDs to ignore as recent when updating folder */
-	GHashTable *ignore_recent;
-
-	GMutex *search_lock;
-	GMutex *stream_lock;
+	GMutex search_lock;
+	GMutex stream_lock;
 
 	gboolean apply_filters;		/* persistent property */
 };
@@ -81,9 +75,25 @@ CamelFolder *	camel_imapx_folder_new		(CamelStore *parent,
 						 const gchar *path,
 						 const gchar *raw,
 						 GError **error);
-gchar *		imapx_get_filename		(CamelFolder *folder,
-						 const gchar *uid,
+CamelIMAPXMailbox *
+		camel_imapx_folder_ref_mailbox	(CamelIMAPXFolder *folder);
+void		camel_imapx_folder_set_mailbox	(CamelIMAPXFolder *folder,
+						 CamelIMAPXMailbox *mailbox);
+CamelIMAPXMailbox *
+		camel_imapx_folder_list_mailbox	(CamelIMAPXFolder *folder,
+						 GCancellable *cancellable,
 						 GError **error);
+GSequence *	camel_imapx_folder_copy_message_map
+						(CamelIMAPXFolder *folder);
+void		camel_imapx_folder_add_move_to_real_junk
+						(CamelIMAPXFolder *folder,
+						 const gchar *message_uid);
+void		camel_imapx_folder_add_move_to_real_trash
+						(CamelIMAPXFolder *folder,
+						 const gchar *message_uid);
+void		camel_imapx_folder_invalidate_local_cache
+						(CamelIMAPXFolder *folder,
+						 guint64 new_uidvalidity);
 
 G_END_DECLS
 
