@@ -52,33 +52,6 @@ struct _CamelPOP3FolderInfo {
 
 G_DEFINE_TYPE (CamelPOP3Folder, camel_pop3_folder, CAMEL_TYPE_FOLDER)
 
-static void
-free_fi (CamelPOP3Folder *pop3_folder,
-         CamelPOP3FolderInfo *fi)
-{
-
-	CamelPOP3Store *pop3_store;
-	CamelStore *store;
-
-	store = camel_folder_get_parent_store (CAMEL_FOLDER (pop3_folder));
-	pop3_store = CAMEL_POP3_STORE (store);
-
-	g_hash_table_remove (pop3_folder->uids_id, GINT_TO_POINTER (fi->id));
-	if (fi->cmd != NULL) {
-		CamelPOP3Engine *pop3_engine;
-
-		pop3_engine = camel_pop3_store_ref_engine (pop3_store);
-
-		camel_pop3_engine_command_free (pop3_engine, fi->cmd);
-		fi->cmd = NULL;
-
-		g_object_unref (pop3_engine);
-	}
-	g_free (fi->uid);
-	g_free (fi);
-
-}
-
 static void
 cmd_uidl (CamelPOP3Engine *pe,
           CamelPOP3Stream *stream,
@@ -174,26 +147,12 @@ cmd_list (CamelPOP3Engine *pe,
 	guint len, id, size;
 	guchar *line;
 	CamelFolder *folder = data;
-	CamelStore *parent_store;
 	CamelPOP3FolderInfo *fi;
-	gint i = 0, total, last_uid=-1;
 	CamelPOP3Folder *pop3_folder;
-	CamelService *service;
-	CamelSettings *settings;
-	gint batch_fetch_count;
 
 	g_return_if_fail (pe != NULL);
 
-	parent_store = camel_folder_get_parent_store (folder);
 	pop3_folder = (CamelPOP3Folder *) folder;
-	service = (CamelService *) parent_store;
-
-	settings = camel_service_ref_settings (service);
-
-	batch_fetch_count = camel_pop3_settings_get_batch_fetch_count (
-		CAMEL_POP3_SETTINGS (settings));
-
-	g_object_unref (settings);
 
 	do {
 		ret = camel_pop3_stream_line (stream, &line, &len, cancellable, error);
@@ -217,125 +176,6 @@ cmd_list (CamelPOP3Engine *pe,
 			}
 		}
 	} while (ret > 0);
-
-	/* Trim the list for mobile devices*/
-	if (pop3_folder->mobile_mode && pop3_folder->uids->len) {
-		gint y = 0;
-		gboolean save_uid = FALSE;
-
-		/* Preserve the first message's ID */
-		fi = pop3_folder->uids->pdata[0];
-		pop3_folder->first_id = fi->id;
-
-		total = pop3_folder->uids->len;
-		if (pop3_folder->key_file) {
-			last_uid = g_key_file_get_integer (pop3_folder->key_file, "UIDConfig", "last-saved-uid", NULL);
-			if (!last_uid) {
-				/* First time downloading the POP folder, lets just download only a batch. */
-				last_uid = -1;
-			}
-			d (printf ("Last stored' first uid: %d\n", last_uid));
-		}
-
-		if (last_uid == -1)
-			save_uid = TRUE;
-
-		for (i = total - 1; i >= 0; i--) {
-			fi = pop3_folder->uids->pdata[i];
-
-			if ((last_uid != -1 && last_uid >= fi->id) || (last_uid == -1 && i == total - batch_fetch_count)) {
-				if (last_uid != -1 && last_uid < fi->id)
-					i++; /* if the last uid was deleted on the server, then we need the last but 1 */
-				break;
-			}
-
-		}
-		if (i> 0 && pop3_folder->fetch_type == CAMEL_FETCH_OLD_MESSAGES && pop3_folder->fetch_more) {
-			gint k = 0;
-			/* Lets pull another window of old messages */
-			save_uid = TRUE;
-			/* Decrement 'i' by another batch count or till we reach the first message */
-			d (printf ("Fetch more (%d): from %d", pop3_folder->fetch_more, i));
-			for (k = 0; k< pop3_folder->fetch_more && i>= 0; k++, i--);
-			d (printf (" to %d\n", i));
-
-			/* Don't load messages newer than the latest we already had. We had to just get old messages and not 
-			 * new messages. */
-			for (y = i; y < total; y++) {
-				fi = pop3_folder->uids->pdata[y];
-				if (fi->id == pop3_folder->latest_id) {
-					/* Delete everything after this. */
-
-					for (y = k + 1; y < total; y++) {
-						fi = pop3_folder->uids->pdata[y];
-						free_fi (pop3_folder, fi);
-					}
-					g_ptr_array_remove_range (pop3_folder->uids, k + 1, total - k - 1);
-					break;
-				}
-			}
-
-		} else if (pop3_folder->fetch_more == CAMEL_FETCH_NEW_MESSAGES && pop3_folder->fetch_more) {
-			/* We need to download new messages. */
-			gint k = 0;
-
-			for (k = i; k < total; k++) {
-				fi = pop3_folder->uids->pdata[k];
-				if (fi->id == pop3_folder->latest_id) {
-					/* We need to just download the specified number of messages. */
-					y= (k + pop3_folder->fetch_more) < total ? (k + pop3_folder->fetch_more) : total - 1;
-					break;
-				}
-			}
-
-		}
-
-		/* Delete the unnecessary old messages */
-		if (i > 0) {
-			gint j = 0;
-			/* i is the start of the last fetch UID, so remove everything else from 0 to i */
-			for (; j < i; j++) {
-				fi = pop3_folder->uids->pdata[j];
-				free_fi (pop3_folder, fi);
-			}
-			g_ptr_array_remove_range (pop3_folder->uids, 0, i);
-			d (printf ("Removing %d uids that are old\n", i));
-
-		}
-
-		/* Delete the unnecessary new message references. */
-		if (y + 1 < total) {
-			gint k;
-
-			for (k = y + 1; k < total; k++) {
-				fi = pop3_folder->uids->pdata[k];
-				free_fi (pop3_folder, fi);
-			}
-			g_ptr_array_remove_range (pop3_folder->uids, y + 1, total - y - 1);
-		}
-
-		if (save_uid) {
-			gchar *contents;
-			gsize len;
-			const gchar *root;
-			gchar *path;
-
-			/* Save the last fetched UID */
-			fi = pop3_folder->uids->pdata[0];
-			g_key_file_set_integer (pop3_folder->key_file, "UIDConfig", "last-saved-uid", fi->id);
-			contents = g_key_file_to_data (pop3_folder->key_file, &len, NULL);
-			root = camel_service_get_user_cache_dir (service);
-			path = g_build_filename (root, "uidconfig", NULL);
-			g_file_set_contents (path, contents, len, NULL);
-			g_key_file_load_from_file (pop3_folder->key_file, path, G_KEY_FILE_NONE, NULL);
-			g_free (contents);
-			g_free (path);
-			d (printf ("Saving last uid %d\n", fi->id));
-
-		}
-
-	}
-
 }
 
 static void
@@ -712,7 +552,7 @@ pop3_folder_get_message_sync (CamelFolder *folder,
 	}
 
 	message = camel_mime_message_new ();
-	if (!camel_data_wrapper_construct_from_stream_sync (
+	if (stream != NULL && !camel_data_wrapper_construct_from_stream_sync (
 		CAMEL_DATA_WRAPPER (message), stream, cancellable, error)) {
 		g_prefix_error (error, _("Cannot get message %s: "), uid);
 		g_object_unref (message);
@@ -722,7 +562,7 @@ pop3_folder_get_message_sync (CamelFolder *folder,
 		camel_medium_add_header (CAMEL_MEDIUM (message), "X-Evolution-POP3-UID", uid);
 	}
 done:
-	g_object_unref (stream);
+	g_clear_object (&stream);
 fail:
 	g_clear_object (&pop3_engine);
 
@@ -884,53 +724,6 @@ pop3_folder_refresh_info_sync (CamelFolder *folder,
 	return success;
 }
 
-static gboolean
-pop3_fetch_messages_sync (CamelFolder *folder,
-                          CamelFetchType type,
-                          gint limit,
-                          GCancellable *cancellable,
-                          GError **error)
-{
-	CamelPOP3FolderInfo *fi;
-	CamelPOP3Folder *pop3_folder = (CamelPOP3Folder *) folder;
-	gint old_len;
-	CamelStore *parent_store;
-	CamelService *service;
-	CamelSettings *settings;
-	gint batch_fetch_count;
-
-	parent_store = camel_folder_get_parent_store (folder);
-	service = (CamelService *) parent_store;
-
-	settings = camel_service_ref_settings (service);
-
-	batch_fetch_count = camel_pop3_settings_get_batch_fetch_count (
-		CAMEL_POP3_SETTINGS (settings));
-
-	g_object_unref (settings);
-
-	old_len = pop3_folder->uids->len;
-
-	/* If we have the first message already, then return FALSE */
-	fi = pop3_folder->uids->pdata[0];
-	if (type == CAMEL_FETCH_OLD_MESSAGES && fi->id == pop3_folder->first_id)
-		return FALSE;
-
-	pop3_folder->fetch_type = type;
-	pop3_folder->fetch_more = (limit > 0) ? limit : batch_fetch_count;
-	pop3_folder_refresh_info_sync (folder, cancellable, error);
-	pop3_folder->fetch_more = 0;
-
-	/* Even if we downloaded the first/oldest message, just now, return TRUE so that we wont waste another cycle */
-	fi = pop3_folder->uids->pdata[0];
-	if (type == CAMEL_FETCH_OLD_MESSAGES && fi->id == pop3_folder->first_id)
-		return FALSE;
-	else if (type == CAMEL_FETCH_NEW_MESSAGES && old_len == pop3_folder->uids->len)
-		return FALSE; /* We didnt fetch any new messages as there were none probably. */
-
-	return TRUE;
-}
-
 static gboolean
 pop3_folder_synchronize_sync (CamelFolder *folder,
                               gboolean expunge,
@@ -1077,7 +870,6 @@ camel_pop3_folder_class_init (CamelPOP3FolderClass *class)
 	object_class->dispose = pop3_folder_dispose;
 
 	folder_class = CAMEL_FOLDER_CLASS (class);
-	folder_class->fetch_messages_sync = pop3_fetch_messages_sync;
 	folder_class->get_message_count = pop3_folder_get_message_count;
 	folder_class->get_uids = pop3_folder_get_uids;
 	folder_class->free_uids = camel_folder_free_shallow;
@@ -1102,12 +894,8 @@ camel_pop3_folder_new (CamelStore *parent,
                        GError **error)
 {
 	CamelFolder *folder;
-	CamelService *service;
-	CamelSettings *settings;
 	CamelPOP3Folder *pop3_folder;
 
-	service = CAMEL_SERVICE (parent);
-
 	d (printf ("opening pop3 INBOX folder\n"));
 
 	folder = g_object_new (
@@ -1115,28 +903,9 @@ camel_pop3_folder_new (CamelStore *parent,
 		"full-name", "inbox", "display-name", "inbox",
 		"parent-store", parent, NULL);
 
-	settings = camel_service_ref_settings (service);
-
 	pop3_folder = (CamelPOP3Folder *) folder;
-	pop3_folder->mobile_mode = camel_pop3_settings_get_mobile_mode (
-		CAMEL_POP3_SETTINGS (settings));
-
-	g_object_unref (settings);
 
 	pop3_folder->fetch_more = 0;
-	if (pop3_folder->mobile_mode) {
-		/* Setup Keyfile */
-		gchar *path;
-		const gchar *root;
-
-		pop3_folder->key_file = g_key_file_new ();
-		root = camel_service_get_user_cache_dir (service);
-		path = g_build_filename (root, "uidconfig", NULL);
-		g_key_file_load_from_file (pop3_folder->key_file, path, G_KEY_FILE_NONE, NULL);
-
-		g_free (path);
-	}
-
 	if (camel_service_get_connection_status (CAMEL_SERVICE (parent)) != CAMEL_SERVICE_CONNECTED)
 		return folder;
 
