@@ -111,9 +111,11 @@ connect_to_server (CamelService *service,
 	CamelNetworkSettings *network_settings;
 	CamelNetworkSecurityMethod method;
 	CamelSettings *settings;
-	CamelStream *tcp_stream;
+	CamelStream *stream;
 	CamelPOP3Engine *pop3_engine = NULL;
 	CamelPOP3Command *pc;
+	GIOStream *base_stream;
+	GIOStream *tls_stream;
 	gboolean disable_extensions;
 	gboolean success = TRUE;
 	gchar *host;
@@ -132,10 +134,13 @@ connect_to_server (CamelService *service,
 
 	g_object_unref (settings);
 
-	tcp_stream = camel_network_service_connect_sync (
+	base_stream = camel_network_service_connect_sync (
 		CAMEL_NETWORK_SERVICE (service), cancellable, error);
 
-	if (tcp_stream == NULL) {
+	if (base_stream != NULL) {
+		stream = camel_stream_new (base_stream);
+		g_object_unref (base_stream);
+	} else {
 		success = FALSE;
 		goto exit;
 	}
@@ -143,7 +148,7 @@ connect_to_server (CamelService *service,
 	/* parent class connect initialization */
 	if (CAMEL_SERVICE_CLASS (camel_pop3_store_parent_class)->
 		connect_sync (service, cancellable, error) == FALSE) {
-		g_object_unref (tcp_stream);
+		g_object_unref (stream);
 		success = FALSE;
 		goto exit;
 	}
@@ -151,7 +156,7 @@ connect_to_server (CamelService *service,
 	if (disable_extensions)
 		flags |= CAMEL_POP3_ENGINE_DISABLE_EXTENSIONS;
 
-	if (!(pop3_engine = camel_pop3_engine_new (tcp_stream, flags, cancellable, &local_error)) ||
+	if (!(pop3_engine = camel_pop3_engine_new (stream, flags, cancellable, &local_error)) ||
 	    local_error != NULL) {
 		if (local_error)
 			g_propagate_error (error, local_error);
@@ -160,13 +165,13 @@ connect_to_server (CamelService *service,
 				error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
 				_("Failed to read a valid greeting from POP server %s"),
 				host);
-		g_object_unref (tcp_stream);
+		g_object_unref (stream);
 		success = FALSE;
 		goto exit;
 	}
 
 	if (method != CAMEL_NETWORK_SECURITY_METHOD_STARTTLS_ON_STANDARD_PORT) {
-		g_object_unref (tcp_stream);
+		g_object_unref (stream);
 		goto exit;
 	}
 
@@ -202,9 +207,15 @@ connect_to_server (CamelService *service,
 	}
 
 	/* Okay, now toggle SSL/TLS mode */
-	success = camel_network_service_starttls (
-		CAMEL_NETWORK_SERVICE (service), tcp_stream, error);
-	if (!success) {
+	base_stream = camel_stream_ref_base_stream (stream);
+	tls_stream = camel_network_service_starttls (
+		CAMEL_NETWORK_SERVICE (service), base_stream, error);
+	g_object_unref (base_stream);
+
+	if (tls_stream != NULL) {
+		camel_stream_set_base_stream (stream, tls_stream);
+		g_object_unref (tls_stream);
+	} else {
 		g_prefix_error (
 			error,
 			_("Failed to connect to POP server %s in secure mode: "),
@@ -212,7 +223,7 @@ connect_to_server (CamelService *service,
 		goto stls_exception;
 	}
 
-	g_object_unref (tcp_stream);
+	g_object_unref (stream);
 
 	/* rfc2595, section 4 states that after a successful STLS
 	 * command, the client MUST discard prior CAPA responses */
@@ -235,7 +246,7 @@ stls_exception:
 	}*/
 
 exception:
-	g_object_unref (tcp_stream);
+	g_object_unref (stream);
 
 	g_clear_object (&pop3_engine);
 
@@ -336,6 +347,7 @@ try_sasl (CamelPOP3Store *store,
 		    || (resp = (guchar *) camel_sasl_challenge_base64_sync (sasl, (const gchar *) line + 2, cancellable, &local_error)) == NULL) {
 			camel_stream_write_string (
 				CAMEL_STREAM (pop3_stream), "*\r\n", cancellable, NULL);
+			/* coverity[unchecked_value] */
 			camel_pop3_stream_line (pop3_stream, &line, &len, cancellable, NULL);
 
 			if (local_error) {
@@ -680,6 +692,7 @@ pop3_store_authenticate_sync (CamelService *service,
 
 	} else if (strcmp (mechanism, "+APOP") == 0 && pop3_engine->apop) {
 		gchar *secret, *md5asc, *d;
+		gsize secret_len;
 
 		if (password == NULL) {
 			g_set_error_literal (
@@ -710,10 +723,13 @@ pop3_store_authenticate_sync (CamelService *service,
 			d++;
 		}
 
-		secret = g_alloca (
+		secret_len =
 			strlen (pop3_engine->apop) +
-			strlen (password) + 1);
-		sprintf (secret, "%s%s", pop3_engine->apop, password);
+			strlen (password) + 1;
+		secret = g_alloca (secret_len);
+		g_snprintf (
+			secret, secret_len, "%s%s",
+			pop3_engine->apop, password);
 		md5asc = g_compute_checksum_for_string (
 			G_CHECKSUM_MD5, secret, -1);
 		pcp = camel_pop3_engine_command_new (
@@ -1073,3 +1089,124 @@ camel_pop3_store_expunge (CamelPOP3Store *store,
 
 	return TRUE;
 }
+
+/**
+ * camel_pop3_store_cache_add:
+ * @store: a #CamelPOP3Store
+ * @uid: a message UID
+ * @error: return location for a #GError, or %NULL
+ *
+ * Creates a cache file for @uid in @store and returns a #CamelStream for it.
+ * If an error occurs in opening the cache file, the function sets @error and
+ * returns %NULL.
+ *
+ * The returned #CamelStream is referenced for thread-safety and must be
+ * unreferenced when finished with it.
+ *
+ * Returns: a #CamelStream, or %NULL
+ **/
+CamelStream *
+camel_pop3_store_cache_add (CamelPOP3Store *store,
+                            const gchar *uid,
+                            GError **error)
+{
+	CamelDataCache *cache;
+	GIOStream *base_stream;
+	CamelStream *stream = NULL;
+
+	g_return_val_if_fail (CAMEL_IS_POP3_STORE (store), NULL);
+	g_return_val_if_fail (uid != NULL, NULL);
+
+	cache = camel_pop3_store_ref_cache (store);
+	g_return_val_if_fail (cache != NULL, NULL);
+
+	base_stream = camel_data_cache_add (cache, "cache", uid, error);
+	if (base_stream != NULL) {
+		stream = camel_stream_new (base_stream);
+		g_object_unref (base_stream);
+	}
+
+	g_object_unref (cache);
+
+	return stream;
+}
+
+/**
+ * camel_pop3_store_cache_get:
+ * @store: a #CamelPOP3Store
+ * @uid: a message UID
+ * @error: return location for a #GError, or %NULL
+ *
+ * Opens the cache file for @uid in @store and returns a #CamelStream for it.
+ * If no matching cache file exists, the function returns %NULL.  If an error
+ * occurs in opening the cache file, the function sets @error and returns
+ * %NULL.
+ *
+ * The returned #CamelStream is referenced for thread-safety and must be
+ * unreferenced when finished with it.
+ *
+ * Returns: a #CamelStream, or %NULL
+ **/
+CamelStream *
+camel_pop3_store_cache_get (CamelPOP3Store *store,
+                            const gchar *uid,
+                            GError **error)
+{
+	CamelDataCache *cache;
+	GIOStream *base_stream;
+	CamelStream *stream = NULL;
+
+	g_return_val_if_fail (CAMEL_IS_POP3_STORE (store), NULL);
+	g_return_val_if_fail (uid != NULL, NULL);
+
+	cache = camel_pop3_store_ref_cache (store);
+	g_return_val_if_fail (cache != NULL, NULL);
+
+	base_stream = camel_data_cache_get (cache, "cache", uid, error);
+	if (base_stream != NULL) {
+		GInputStream *input_stream;
+		gchar buffer[1];
+		gssize n_bytes;
+
+		input_stream = g_io_stream_get_input_stream (base_stream);
+
+		n_bytes = g_input_stream_read (
+			input_stream, buffer, 1, NULL, error);
+
+		if (n_bytes == 1 && buffer[0] == '#')
+			stream = camel_stream_new (base_stream);
+
+		g_object_unref (base_stream);
+	}
+
+	g_object_unref (cache);
+
+	return stream;
+}
+
+/**
+ * camel_pop3_store_cache_has:
+ * @store: a #CamelPOP3Store
+ * @uid: a message UID
+ *
+ * Returns whether @store has a cached message for @uid.
+ *
+ * Returns: whether @uid is cached
+ **/
+gboolean
+camel_pop3_store_cache_has (CamelPOP3Store *store,
+                            const gchar *uid)
+{
+	CamelStream *stream;
+	gboolean uid_is_cached;
+
+	g_return_val_if_fail (CAMEL_IS_POP3_STORE (store), FALSE);
+	g_return_val_if_fail (uid != NULL, FALSE);
+
+	stream = camel_pop3_store_cache_get (store, uid, NULL);
+	uid_is_cached = (stream != NULL);
+	g_clear_object (&stream);
+
+	return uid_is_cached;
+}
+
