@@ -55,28 +55,37 @@ struct _CamelSendmailSettingsClass {
 	CamelSettingsClass parent_class;
 };
 
-GType		camel_sendmail_settings_get_type	(void);
-gboolean	camel_sendmail_settings_get_use_custom_binary	(CamelSendmailSettings *settings);
-void		camel_sendmail_settings_set_use_custom_binary	(CamelSendmailSettings *settings,
-								 gboolean use_custom_binary);
-
-const gchar *	camel_sendmail_settings_get_custom_binary	(CamelSendmailSettings *settings);
-gchar *		camel_sendmail_settings_dup_custom_binary	(CamelSendmailSettings *settings);
-void		camel_sendmail_settings_set_custom_binary	(CamelSendmailSettings *settings,
-								 const gchar *custom_binary);
-
-gboolean	camel_sendmail_settings_get_use_custom_args	(CamelSendmailSettings *settings);
-void		camel_sendmail_settings_set_use_custom_args	(CamelSendmailSettings *settings,
-								 gboolean use_custom_args);
-
-const gchar *	camel_sendmail_settings_get_custom_args		(CamelSendmailSettings *settings);
-gchar *		camel_sendmail_settings_dup_custom_args		(CamelSendmailSettings *settings);
-void		camel_sendmail_settings_set_custom_args		(CamelSendmailSettings *settings,
-								 const gchar *custom_args);
-
-gboolean	camel_sendmail_settings_get_send_in_offline	(CamelSendmailSettings *settings);
-void		camel_sendmail_settings_set_send_in_offline	(CamelSendmailSettings *settings,
-								 gboolean send_in_offline);
+GType		camel_sendmail_settings_get_type
+					(void) G_GNUC_CONST;
+gboolean	camel_sendmail_settings_get_use_custom_binary
+					(CamelSendmailSettings *settings);
+void		camel_sendmail_settings_set_use_custom_binary
+					(CamelSendmailSettings *settings,
+					 gboolean use_custom_binary);
+const gchar *	camel_sendmail_settings_get_custom_binary
+					(CamelSendmailSettings *settings);
+gchar *		camel_sendmail_settings_dup_custom_binary
+					(CamelSendmailSettings *settings);
+void		camel_sendmail_settings_set_custom_binary
+					(CamelSendmailSettings *settings,
+					 const gchar *custom_binary);
+gboolean	camel_sendmail_settings_get_use_custom_args
+					(CamelSendmailSettings *settings);
+void		camel_sendmail_settings_set_use_custom_args
+					(CamelSendmailSettings *settings,
+					 gboolean use_custom_args);
+const gchar *	camel_sendmail_settings_get_custom_args
+					(CamelSendmailSettings *settings);
+gchar *		camel_sendmail_settings_dup_custom_args
+					(CamelSendmailSettings *settings);
+void		camel_sendmail_settings_set_custom_args
+					(CamelSendmailSettings *settings,
+					 const gchar *custom_args);
+gboolean	camel_sendmail_settings_get_send_in_offline
+					(CamelSendmailSettings *settings);
+void		camel_sendmail_settings_set_send_in_offline
+					(CamelSendmailSettings *settings,
+					 gboolean send_in_offline);
 
 G_END_DECLS
 
