@@ -57,7 +57,7 @@ G_DEFINE_QUARK (e-collator-error-quark, e_collator_error)
 
 G_DEFINE_BOXED_TYPE (ECollator,
 		     e_collator,
-		     e_collator_ref, 
+		     e_collator_ref,
 		     e_collator_unref)
 
 struct _ECollator
@@ -89,21 +89,21 @@ print_available_locales (void)
 
 	u_init (&status);
 
-	g_printerr ("List of available locales (default locale is: %s)\n", uloc_getDefault());
+	g_printerr ("List of available locales (default locale is: %s)\n", uloc_getDefault ());
 
-	count = uloc_countAvailable();
+	count = uloc_countAvailable ();
 	for (i = 0; i < count; i++) {
 		UEnumeration *keywords;
 		const gchar *keyword;
 
-		uloc_getDisplayName(uloc_getAvailable(i), NULL, result, 100, &status);
+		uloc_getDisplayName (uloc_getAvailable (i), NULL, result, 100, &status);
 
 		u_austrncpy (printable, result, sizeof (printable));
 
 		/* print result */
-		g_printerr ("\t%s - %s", uloc_getAvailable(i), printable);
+		g_printerr ("\t%s - %s", uloc_getAvailable (i), printable);
 
-		keywords = uloc_openKeywords (uloc_getAvailable(i), &status);
+		keywords = uloc_openKeywords (uloc_getAvailable (i), &status);
 		if (keywords) {
 			UErrorCode kstatus = U_ZERO_ERROR;
 
@@ -122,13 +122,15 @@ print_available_locales (void)
 #endif
 
 static gchar *
-canonicalize_locale (const gchar  *posix_locale,
-		     gchar       **language_code,
-		     GError      **error)
+canonicalize_locale (const gchar *posix_locale,
+                     gchar **language_code,
+                     gchar **country_code,
+                     GError **error)
 {
 	UErrorCode status = U_ZERO_ERROR;
 	gchar  locale_buffer[LOCALE_BUFFER_LEN];
 	gchar  language_buffer[8];
+	gchar  country_buffer[8];
 	gchar *icu_locale;
 	gchar *final_locale;
 	gint   len;
@@ -137,11 +139,12 @@ canonicalize_locale (const gchar  *posix_locale,
 	len = uloc_canonicalize (posix_locale, locale_buffer, LOCALE_BUFFER_LEN, &status);
 
 	if (U_FAILURE (status)) {
-		g_set_error (error, E_COLLATOR_ERROR,
-			     E_COLLATOR_ERROR_INVALID_LOCALE,
-			     "Failed to interpret locale '%s' (%s)",
-			     posix_locale,
-			     u_errorName (status));
+		g_set_error (
+			error, E_COLLATOR_ERROR,
+			E_COLLATOR_ERROR_INVALID_LOCALE,
+			"Failed to interpret locale '%s' (%s)",
+			posix_locale,
+			u_errorName (status));
 		return NULL;
 	}
 
@@ -155,13 +158,26 @@ canonicalize_locale (const gchar  *posix_locale,
 
 	status = U_ZERO_ERROR;
 	len = uloc_getLanguage (icu_locale, language_buffer, 8, &status);
-
 	if (U_FAILURE (status)) {
-		g_set_error (error, E_COLLATOR_ERROR,
-			     E_COLLATOR_ERROR_INVALID_LOCALE,
-			     "Failed to interpret language for locale '%s': %s",
-			     icu_locale,
-			     u_errorName (status));
+		g_set_error (
+			error, E_COLLATOR_ERROR,
+			E_COLLATOR_ERROR_INVALID_LOCALE,
+			"Failed to interpret language for locale '%s': %s",
+			icu_locale,
+			u_errorName (status));
+		g_free (icu_locale);
+		return NULL;
+	}
+
+	status = U_ZERO_ERROR;
+	len = uloc_getCountry (icu_locale, country_buffer, 8, &status);
+	if (U_FAILURE (status)) {
+		g_set_error (
+			error, E_COLLATOR_ERROR,
+			E_COLLATOR_ERROR_INVALID_LOCALE,
+			"Failed to interpret country for locale '%s': %s",
+			icu_locale,
+			u_errorName (status));
 		g_free (icu_locale);
 		return NULL;
 	}
@@ -186,6 +202,9 @@ canonicalize_locale (const gchar  *posix_locale,
 	if (language_code)
 		*language_code = g_strdup (language_buffer);
 
+	if (country_code)
+		*country_code = g_strdup (country_buffer);
+
 	return final_locale;
 }
 
@@ -194,12 +213,12 @@ canonicalize_locale (const gchar  *posix_locale,
  * is valid UTF-8
  */
 static const UChar *
-convert_to_ustring (const gchar  *string,
-		    UChar        *buffer,
-		    gint          buffer_len,
-		    gint         *result_len,
-		    UChar       **free_me,
-		    GError      **error)
+convert_to_ustring (const gchar *string,
+                    UChar *buffer,
+                    gint buffer_len,
+                    gint *result_len,
+                    UChar **free_me,
+                    GError **error)
 {
 	UErrorCode status = U_ZERO_ERROR;
 	const gchar *source_utf8;
@@ -216,12 +235,13 @@ convert_to_ustring (const gchar  *string,
 	}
 
 	/* First pass, try converting to UChar in the given buffer */
-	converted_buffer = u_strFromUTF8Lenient (buffer,
-						 buffer_len,
-						 &converted_len,
-						 source_utf8,
-						 -1,
-						 &status);
+	converted_buffer = u_strFromUTF8Lenient (
+		buffer,
+		buffer_len,
+		&converted_len,
+		source_utf8,
+		-1,
+		&status);
 
 	/* Set the result length right away... */
 	*result_len = converted_len;
@@ -235,12 +255,13 @@ convert_to_ustring (const gchar  *string,
 	if (converted_len > buffer_len) {
 		*free_me = g_new (UChar, converted_len);
 
-		converted_buffer = u_strFromUTF8Lenient (*free_me,
-							 converted_len,
-							 NULL,
-							 source_utf8,
-							 -1,
-							 &status);
+		converted_buffer = u_strFromUTF8Lenient (
+			*free_me,
+			converted_len,
+			NULL,
+			source_utf8,
+			-1,
+			&status);
 
 		if (U_FAILURE (status)) {
 			g_free (*free_me);
@@ -254,10 +275,11 @@ convert_to_ustring (const gchar  *string,
 	g_free (alloc_utf8);
 
 	if (U_FAILURE (status))
-		g_set_error (error, E_COLLATOR_ERROR,
-			     E_COLLATOR_ERROR_CONVERSION,
-			     "Error occured while converting character encoding (%s)",
-			     u_errorName (status));
+		g_set_error (
+			error, E_COLLATOR_ERROR,
+			E_COLLATOR_ERROR_CONVERSION,
+			"Error occured while converting character encoding (%s)",
+			u_errorName (status));
 
 	return converted_buffer;
 }
@@ -269,7 +291,7 @@ convert_to_ustring (const gchar  *string,
 /**
  * e_collator_new:
  * @locale: The locale under which to sort
- * @error: (allow none): A location to store a #GError from the #E_COLLATOR_ERROR domain
+ * @error: (allow-none): A location to store a #GError from the #E_COLLATOR_ERROR domain
  *
  * Creates a new #ECollator for the given @locale,
  * the returned collator should be freed with e_collator_unref().
@@ -279,14 +301,39 @@ convert_to_ustring (const gchar  *string,
  * Since: 3.12
  */
 ECollator *
-e_collator_new (const gchar     *locale,
-		GError         **error)
+e_collator_new (const gchar *locale,
+                GError **error)
+{
+	return e_collator_new_interpret_country (locale, NULL, error);
+}
+
+/**
+ * e_collator_new_interpret_country:
+ * @locale: The locale under which to sort
+ * @country_code: (allow-none) (out) (transfer full): A location to store the interpreted country code from @locale 
+ * @error: (allow-none): A location to store a #GError from the #E_COLLATOR_ERROR domain
+ *
+ * Creates a new #ECollator for the given @locale,
+ * the returned collator should be freed with e_collator_unref().
+ *
+ * In addition, this also reliably interprets the country
+ * code from the @locale string and stores it to @country_code.
+ *
+ * Returns: (transfer full): A newly created #ECollator.
+ *
+ * Since: 3.12
+ */
+ECollator *
+e_collator_new_interpret_country (const gchar *locale,
+                                  gchar **country_code,
+                                  GError **error)
 {
 	ECollator *collator;
 	UCollator *coll;
 	UErrorCode status = U_ZERO_ERROR;
 	gchar     *icu_locale;
 	gchar     *language_code = NULL;
+	gchar     *local_country_code = NULL;
 
 	g_return_val_if_fail (locale && locale[0], NULL);
 
@@ -294,20 +341,26 @@ e_collator_new (const gchar     *locale,
 	print_available_locales ();
 #endif
 
-	icu_locale = canonicalize_locale (locale, &language_code, error);
+	icu_locale = canonicalize_locale (
+		locale,
+		&language_code,
+		&local_country_code,
+		error);
 	if (!icu_locale)
 		return NULL;
 
 	coll = ucol_open (icu_locale, &status);
 
 	if (U_FAILURE (status)) {
-		g_set_error (error, E_COLLATOR_ERROR,
-			     E_COLLATOR_ERROR_OPEN,
-			     "Unable to open collator for locale '%s' (%s)",
-			     icu_locale,
-			     u_errorName (status));
+		g_set_error (
+			error, E_COLLATOR_ERROR,
+			E_COLLATOR_ERROR_OPEN,
+			"Unable to open collator for locale '%s' (%s)",
+			icu_locale,
+			u_errorName (status));
 
 		g_free (language_code);
+		g_free (local_country_code);
 		g_free (icu_locale);
 		ucol_close (coll);
 		return NULL;
@@ -328,14 +381,20 @@ e_collator_new (const gchar     *locale,
 		collator->transliterator = _e_transliterator_cxx_new ("Han-Latin");
 
 	collator->alpha_index = _e_alphabet_index_cxx_new_for_language (language_code);
-	collator->labels = _e_alphabet_index_cxx_get_labels (collator->alpha_index,
-							     &collator->n_labels,
-							     &collator->underflow,
-							     &collator->inflow,
-							     &collator->overflow);
+	collator->labels = _e_alphabet_index_cxx_get_labels (
+		collator->alpha_index,
+		&collator->n_labels,
+		&collator->underflow,
+		&collator->inflow,
+		&collator->overflow);
 
 	g_free (language_code);
 
+	if (country_code)
+		*country_code = local_country_code;
+	else
+		g_free (local_country_code);
+
 	return collator;
 }
 
@@ -395,7 +454,7 @@ e_collator_unref (ECollator *collator)
  * e_collator_generate_key:
  * @collator: An #ECollator
  * @str: The string to generate a collation key for
- * @error: (allow none): A location to store a #GError from the #E_COLLATOR_ERROR domain
+ * @error: (allow-none): A location to store a #GError from the #E_COLLATOR_ERROR domain
  *
  * Generates a collation key for @str, the result of comparing
  * two collation keys with strcmp() will be the same result
@@ -408,9 +467,9 @@ e_collator_unref (ECollator *collator)
  * Since: 3.12
  */
 gchar *
-e_collator_generate_key (ECollator    *collator,
-			 const gchar  *str,
-			 GError      **error)
+e_collator_generate_key (ECollator *collator,
+                         const gchar *str,
+                         GError **error)
 {
 	UChar  source_buffer[CONVERT_BUFFER_LEN];
 	UChar *free_me = NULL;
@@ -433,12 +492,13 @@ e_collator_generate_key (ECollator    *collator,
 		input_str = str;
 	}
 
-	source = convert_to_ustring (input_str,
-				     source_buffer,
-				     CONVERT_BUFFER_LEN,
-				     &source_len,
-				     &free_me,
-				     error);
+	source = convert_to_ustring (
+		input_str,
+		source_buffer,
+		CONVERT_BUFFER_LEN,
+		&source_len,
+		&free_me,
+		error);
 
 	if (!source) {
 		g_free (translit_str);
@@ -449,8 +509,9 @@ e_collator_generate_key (ECollator    *collator,
 	alphabet_index = _e_alphabet_index_cxx_get_index (collator->alpha_index, input_str);
 
 	/* First try to generate a key in a predefined buffer size */
-	key_len = ucol_getSortKey (collator->coll, source, source_len,
-				   (guchar *)stack_buffer, COLLATION_KEY_BUFFER_LEN);
+	key_len = ucol_getSortKey (
+		collator->coll, source, source_len,
+		(guchar *) stack_buffer, COLLATION_KEY_BUFFER_LEN);
 
 	if (key_len > COLLATION_KEY_BUFFER_LEN) {
 
@@ -467,8 +528,9 @@ e_collator_generate_key (ECollator    *collator,
 		snprintf (collation_key, 4, "%03d-", alphabet_index);
 
 		/* Get the sort key and put it in &collation_key[4] */
-		ucol_getSortKey (collator->coll, source, source_len,
-				 (guchar *)(collation_key + 4), key_len);
+		ucol_getSortKey (
+			collator->coll, source, source_len,
+			(guchar *)(collation_key + 4), key_len);
 
 		/* Just being paranoid, make sure we're null terminated since the API
 		 * doesn't specify if the result length is null character inclusive
@@ -489,7 +551,7 @@ e_collator_generate_key (ECollator    *collator,
 	g_free (free_me);
 	g_free (translit_str);
 
-	return (gchar *)collation_key;
+	return (gchar *) collation_key;
 }
 
 /**
@@ -515,8 +577,8 @@ e_collator_generate_key (ECollator    *collator,
  * Since: 3.12
  */
 gchar *
-e_collator_generate_key_for_index (ECollator       *collator,
-				   gint             index)
+e_collator_generate_key_for_index (ECollator *collator,
+                                   gint index)
 {
 	g_return_val_if_fail (collator != NULL, NULL);
 	g_return_val_if_fail (index >= 0 && index < collator->n_labels, NULL);
@@ -527,16 +589,18 @@ e_collator_generate_key_for_index (ECollator       *collator,
 /**
  * e_collator_collate:
  * @collator: An #ECollator
- * @str_a: A string to compare
- * @str_b: The string to compare with @str_a
+ * @str_a: (allow-none): A string to compare
+ * @str_b: (allow-none): The string to compare with @str_a
  * @result: (out): A location to store the comparison result
- * @error: (allow none): A location to store a #GError from the #E_COLLATOR_ERROR domain
+ * @error: (allow-none): A location to store a #GError from the #E_COLLATOR_ERROR domain
  *
  * Compares @str_a with @str_b, the order of strings is determined by the parameters of @collator.
  *
  * The @result will be set to integer less than, equal to, or greater than zero if @str_a is found,
  * respectively, to be less than, to match, or be greater than @str_b.
  *
+ * Either @str_a or @str_b can be %NULL, %NULL strings are considered to sort below other strings.
+ *
  * This function will first ensure that both strings are valid UTF-8.
  *
  * Returns: %TRUE on success, otherwise if %FALSE is returned then @error will be set.
@@ -544,19 +608,22 @@ e_collator_generate_key_for_index (ECollator       *collator,
  * Since: 3.12
  */
 gboolean
-e_collator_collate (ECollator    *collator,
-		    const gchar  *str_a,
-		    const gchar  *str_b,
-		    gint         *result,
-		    GError      **error)
+e_collator_collate (ECollator *collator,
+                    const gchar *str_a,
+                    const gchar *str_b,
+                    gint *result,
+                    GError **error)
 {
 	gchar *sort_key_a, *sort_key_b;
 
 	g_return_val_if_fail (collator != NULL, -1);
-	g_return_val_if_fail (str_a != NULL, -1);
-	g_return_val_if_fail (str_b != NULL, -1);
 	g_return_val_if_fail (result != NULL, -1);
 
+	if (!str_a || !str_b) {
+		*result = g_strcmp0 (str_a, str_b);
+		return TRUE;
+	}
+
 	sort_key_a = e_collator_generate_key (collator, str_a, error);
 	if (!sort_key_a)
 		return FALSE;
@@ -591,11 +658,11 @@ e_collator_collate (ECollator    *collator,
  * Since: 3.12
  */
 const gchar *const  *
-e_collator_get_index_labels (ECollator       *collator,
-			     gint            *n_labels,
-			     gint            *underflow,
-			     gint            *inflow,
-			     gint            *overflow)
+e_collator_get_index_labels (ECollator *collator,
+                             gint *n_labels,
+                             gint *underflow,
+                             gint *inflow,
+                             gint *overflow)
 {
 	g_return_val_if_fail (collator != NULL, NULL);
 
@@ -608,7 +675,7 @@ e_collator_get_index_labels (ECollator       *collator,
 	if (overflow)
 		*overflow = collator->overflow;
 
-	return (const gchar *const  *)collator->labels;
+	return (const gchar *const  *) collator->labels;
 }
 
 /**
@@ -624,8 +691,8 @@ e_collator_get_index_labels (ECollator       *collator,
  * Since: 3.12
  */
 gint
-e_collator_get_index (ECollator       *collator,
-		      const gchar     *str)
+e_collator_get_index (ECollator *collator,
+                      const gchar *str)
 {
 	gint index;
 	gchar *translit_str = NULL;
