@@ -40,6 +40,8 @@ const gchar *	e_get_user_cache_dir		(void);
 const gchar *	e_get_user_config_dir		(void);
 const gchar *	e_get_user_data_dir		(void);
 
+gboolean	e_util_strv_equal		(gconstpointer v1,
+						 gconstpointer v2);
 gchar *		e_util_strdup_strip		(const gchar *string);
 gchar *		e_util_strstrcase		(const gchar *haystack,
 						 const gchar *needle);
@@ -161,6 +163,43 @@ gboolean	e_named_parameters_test		(const ENamedParameters *parameters,
 						 const gchar *value,
 						 gboolean case_sensitively);
 
+#define e_named_timeout_add(interval, function, data) \
+	(e_timeout_add_with_name ( \
+		G_PRIORITY_DEFAULT, (interval), \
+		"[" PACKAGE "] " G_STRINGIFY (function), \
+		(function), (data), NULL))
+
+#define e_named_timeout_add_full(priority, interval, function, data, notify) \
+	(e_timeout_add_with_name ( \
+		(priority), (interval), \
+		"[" PACKAGE "] " G_STRINGIFY (function), \
+		(function), (data), (notify)))
+
+#define e_named_timeout_add_seconds(interval, function, data) \
+	(e_timeout_add_seconds_with_name ( \
+		G_PRIORITY_DEFAULT, (interval), \
+		"[" PACKAGE "] " G_STRINGIFY (function), \
+		(function), (data), NULL))
+
+#define e_named_timeout_add_seconds_full(priority, interval, function, data, notify) \
+	(e_timeout_add_seconds_with_name ( \
+		(priority), (interval), \
+		"[" PACKAGE "] " G_STRINGIFY (function), \
+		(function), (data), (notify)))
+
+guint		e_timeout_add_with_name		(gint priority,
+						 guint interval,
+						 const gchar *name,
+						 GSourceFunc function,
+						 gpointer data,
+						 GDestroyNotify notify);
+guint		e_timeout_add_seconds_with_name	(gint priority,
+						 guint interval,
+						 const gchar *name,
+						 GSourceFunc function,
+						 gpointer data,
+						 GDestroyNotify notify);
+
 #ifndef EDS_DISABLE_DEPRECATED
 void		e_util_free_string_slist	(GSList *strings);
 void		e_util_free_object_slist	(GSList *objects);
