@@ -205,7 +205,8 @@ e_sexp_result_free (struct _ESExp *f,
 
 	switch (t->type) {
 	case ESEXP_RES_ARRAY_PTR:
-		g_ptr_array_free (t->value.ptrarray, TRUE);
+		if (t->value.ptrarray)
+			g_ptr_array_free (t->value.ptrarray, TRUE);
 		break;
 	case ESEXP_RES_BOOL:
 	case ESEXP_RES_INT:
@@ -1481,6 +1482,8 @@ e_sexp_finalise (gpointer o)
 		s->tree = NULL;
 	}
 
+	g_free (s->error);
+
 	e_memchunk_destroy (s->term_chunks);
 	e_memchunk_destroy (s->result_chunks);
 
@@ -1742,8 +1745,8 @@ e_sexp_evaluate_occur_times (ESExp *f,
 
 /**
  * e_sexp_encode_bool:
- * @s:
- * @state:
+ * @s: A #GString to append to
+ * @state: The boolean value
  *
  * Encode a bool into an s-expression @s.  Bools are
  * encoded using #t #f syntax.
