@@ -23,11 +23,9 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n-lib.h>
 
-#include <cert.h>
 #include <libebackend/libebackend.h>
 
 #include "trust-prompt.h"
-#include "certificate-viewer.h"
 
 static void
 trust_prompt_add_info_line (GtkGrid *grid,
@@ -76,27 +74,7 @@ trust_prompt_add_info_line (GtkGrid *grid,
 	pango_attr_list_unref (bold);
 }
 
-static void
-trust_prompt_free_certificate (gpointer cert)
-{
-	if (!cert)
-		return;
-
-	CERT_DestroyCertificate (cert);
-}
-
-static void
-trust_prompt_free_issuers (gpointer issuers)
-{
-	if (!issuers)
-		return;
-
-	g_slist_free_full (issuers, trust_prompt_free_certificate);
-}
-
 #define TRUST_PROMP_ID_KEY	"ETrustPrompt::prompt-id-key"
-#define TRUST_PROMP_CERT_KEY	"ETrustPrompt::cert-key"
-#define TRUST_PROMP_ISSUERS_KEY	"ETrustPrompt::issuers-key"
 
 static void
 trust_prompt_response_cb (GtkWidget *dialog,
@@ -105,20 +83,6 @@ trust_prompt_response_cb (GtkWidget *dialog,
 {
 	gint prompt_id;
 
-	if (response == GTK_RESPONSE_HELP) {
-		GtkWidget *viewer;
-
-		viewer = certificate_viewer_new (
-			GTK_WINDOW (dialog),
-			g_object_get_data (G_OBJECT (dialog), TRUST_PROMP_CERT_KEY),
-			g_object_get_data (G_OBJECT (dialog), TRUST_PROMP_ISSUERS_KEY));
-
-		gtk_dialog_run (GTK_DIALOG (viewer));
-		gtk_widget_destroy (viewer);
-
-		return;
-	}
-
 	prompt_id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (dialog), TRUST_PROMP_ID_KEY));
 	gtk_widget_destroy (dialog);
 
@@ -139,51 +103,46 @@ trust_prompt_show (EUserPrompterServerExtension *extension,
                    gint prompt_id,
                    const gchar *host,
                    const gchar *markup,
-                   const CERTCertificate *pcert,
-                   const gchar *cert_fingerprint,
-                   const gchar *reason,
-                   const GSList *pissuers)
+                   GcrParsed *parsed,
+                   const gchar *reason)
 {
+	GcrCertificateWidget *certificate_widget;
+	GcrCertificate *certificate;
+	GckAttributes *attributes;
 	GtkWidget *dialog, *widget;
 	GtkGrid *grid;
-	gchar *tmp, *issuer, *subject, *head;
-	GSList *issuers, *iter;
-	CERTCertificate *cert;
+	const guchar *data;
+	gsize length;
+	gchar *tmp;
 	gint row = 0;
 
-	cert = CERT_DupCertificate ((CERTCertificate *) pcert);
-	issuers = g_slist_copy ((GSList *) pissuers);
-	for (iter = issuers; iter; iter = g_slist_next (iter)) {
-		if (iter->data)
-			iter->data = CERT_DupCertificate (iter->data);
-	}
-
 	dialog = gtk_dialog_new_with_buttons (
 		_("Certificate trust..."), NULL, 0,
-		_("_View Certificate"), GTK_RESPONSE_HELP,
 		_("_Reject"), GTK_RESPONSE_REJECT,
 		_("Accept _Temporarily"), GTK_RESPONSE_YES,
 		_("_Accept Permanently"), GTK_RESPONSE_ACCEPT,
 		NULL);
 
+	widget = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+
 	gtk_window_set_icon_name (GTK_WINDOW (dialog), "evolution");
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_YES);
+	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
 
 	grid = g_object_new (
 		GTK_TYPE_GRID,
 		"orientation", GTK_ORIENTATION_HORIZONTAL,
 		"row-homogeneous", FALSE,
-		"row-spacing", 2,
+		"row-spacing", 6,
 		"column-homogeneous", FALSE,
-		"column-spacing", 6,
+		"column-spacing", 12,
 		"hexpand", TRUE,
 		"halign", GTK_ALIGN_FILL,
 		"vexpand", TRUE,
 		"valign", GTK_ALIGN_FILL,
-		"border-width", 12,
 		NULL);
 
-	widget = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+	gtk_container_set_border_width (GTK_CONTAINER (grid), 5);
 	gtk_container_add (GTK_CONTAINER (widget), GTK_WIDGET (grid));
 
 	widget = gtk_image_new_from_icon_name ("dialog-warning", GTK_ICON_SIZE_DIALOG);
@@ -191,7 +150,6 @@ trust_prompt_show (EUserPrompterServerExtension *extension,
 		G_OBJECT (widget),
 		"vexpand", FALSE,
 		"valign", GTK_ALIGN_START,
-		"xpad", 6,
 		NULL);
 	gtk_grid_attach (grid, widget, 0, row, 1, 3);
 
@@ -206,31 +164,32 @@ trust_prompt_show (EUserPrompterServerExtension *extension,
 		markup = tmp;
 	}
 
-	head = g_strdup_printf ("%s\n\n%s", markup, _("Detailed information about the certificate:"));
-
 	widget = gtk_label_new (NULL);
-	gtk_label_set_markup (GTK_LABEL (widget), head);
+	gtk_label_set_markup (GTK_LABEL (widget), markup);
 	gtk_misc_set_alignment (GTK_MISC (widget), 0.0, 0.0);
-	g_free (head);
+
 	g_free (tmp);
 
 	gtk_grid_attach (grid, widget, 1, row, 2, 1);
 	row++;
 
-	issuer = CERT_NameToAscii (&cert->issuer);
-	subject = CERT_NameToAscii (&cert->subject);
-
-	trust_prompt_add_info_line (grid, _("Issuer:"), issuer, TRUE, &row);
-	trust_prompt_add_info_line (grid, _("Subject:"), subject, TRUE, &row);
-	trust_prompt_add_info_line (grid, _("Fingerprint:"), cert_fingerprint, TRUE, &row);
 	trust_prompt_add_info_line (grid, _("Reason:"), reason, FALSE, &row);
 
-	PORT_Free (issuer);
-	PORT_Free (subject);
+	data = gcr_parsed_get_data (parsed, &length);
+	attributes = gcr_parsed_get_attributes (parsed);
+
+	certificate = gcr_simple_certificate_new (data, length);
+
+	certificate_widget = gcr_certificate_widget_new (certificate);
+	gcr_certificate_widget_set_attributes (certificate_widget, attributes);
+
+	widget = GTK_WIDGET (certificate_widget);
+	gtk_grid_attach (grid, widget, 1, row, 2, 1);
+	gtk_widget_show (widget);
+
+	g_clear_object (&certificate);
 
 	g_object_set_data (G_OBJECT (dialog), TRUST_PROMP_ID_KEY, GINT_TO_POINTER (prompt_id));
-	g_object_set_data_full (G_OBJECT (dialog), TRUST_PROMP_CERT_KEY, cert, trust_prompt_free_certificate);
-	g_object_set_data_full (G_OBJECT (dialog), TRUST_PROMP_ISSUERS_KEY, issuers, trust_prompt_free_issuers);
 
 	g_signal_connect (dialog, "response", G_CALLBACK (trust_prompt_response_cb), extension);
 
