@@ -82,8 +82,8 @@ e_ag_account_google_got_userinfo_cb (RestProxyCall *call,
 	if (rest_proxy_call_get_status_code (call) != 200) {
 		g_simple_async_result_set_error (
 			simple, G_IO_ERROR, G_IO_ERROR_FAILED,
-			_("Expected status 200 when requesting guid, "
-			"instead got status %d (%s)"),
+			_("Expected status 200 when requesting your "
+			"identity, instead got status %d (%s)"),
 			rest_proxy_call_get_status_code (call),
 			rest_proxy_call_get_status_message (call));
 		goto exit;
@@ -117,7 +117,7 @@ e_ag_account_google_got_userinfo_cb (RestProxyCall *call,
 	} else {
 		g_simple_async_result_set_error (
 			simple, G_IO_ERROR, G_IO_ERROR_FAILED,
-			_("Didn't find email member in JSON data"));
+			_("Didn't find 'email' in JSON data"));
 	}
 
 	g_object_unref (json_parser);
