@@ -20,6 +20,21 @@
  * Authors: Tristan Van Berkom <tristanvb@openismus.com>
  */
 
+/**
+ * SECTION: e-test-server-utils
+ * @short_description: A utility for unit testing EDS
+ *
+ * This test fixture provides an encapsulated testing environment for test
+ * cases to test #EBookClient and #ECalClient.
+ *
+ * The #ETestServerFixture should be used as a fixture and must be coupled
+ * with the #ETestServerClosure to configure how the test fixture will operate.
+ *
+ * Both the #ETestServerFixture and #ETestServerClosure can be extended with
+ * more complex test fixtures, which must remember to call e_test_server_utils_setup()
+ * and e_test_server_utils_teardown() in thier fixture's setup and teardown routines.
+ **/
+
 #include "e-test-server-utils.h"
 
 #include <sys/wait.h>
@@ -27,6 +42,8 @@
 #define ADDRESS_BOOK_SOURCE_UID "test-address-book"
 #define CALENDAR_SOURCE_UID     "test-calendar"
 
+#define FINALIZE_SECONDS         10
+
 /* FIXME, currently we are unable to achieve server activation
  * twice in a single test case, so we're using one D-Bus server
  * throughout an entire test suite.
@@ -49,6 +66,133 @@ static GTestDBus *global_test_dbus = NULL;
  */
 static gint global_test_source_id = 0;
 
+/*****************************************************************
+ *                    Reference management                       *
+ *****************************************************************
+ *
+ * We need to test that an EClient actually finalizes properly
+ * at the end of every test, however these EClient's have a 
+ * habit of finalizing asynchronously, not even in the same thread
+ * in which they were created.
+ *
+ * This is why we do this crazy stuff below to ensure asyncrhonous
+ * finalization of the client actually happens.
+ */
+static void
+add_weak_ref (ETestServerFixture *fixture,
+              ETestServiceType service_type)
+{
+	GObject *object;
+
+	switch (service_type) {
+	case E_TEST_SERVER_NONE:
+		g_weak_ref_set (&fixture->registry_ref, fixture->registry);
+		break;
+
+	case E_TEST_SERVER_ADDRESS_BOOK:
+	case E_TEST_SERVER_DIRECT_ADDRESS_BOOK:
+	case E_TEST_SERVER_CALENDAR:
+	case E_TEST_SERVER_DEPRECATED_ADDRESS_BOOK:
+	case E_TEST_SERVER_DEPRECATED_CALENDAR:
+
+		/* They're all the same object pointer */
+		object = E_TEST_SERVER_UTILS_SERVICE (fixture, GObject);
+		g_weak_ref_set (&fixture->client_ref, object);
+		break;
+	}
+}
+
+static gboolean
+object_finalize_timeout (gpointer user_data)
+{
+	const gchar *message = (const gchar *) user_data;
+
+	g_error ("%s", message);
+
+	return FALSE;
+}
+
+static gboolean
+object_unref_idle (gpointer user_data)
+{
+	g_object_unref (user_data);
+
+	return FALSE;
+}
+
+static void
+weak_notify_loop_quit (gpointer data,
+                       GObject *where_the_object_was)
+{
+	ETestServerFixture *fixture = (ETestServerFixture *) data;
+
+	g_main_loop_quit (fixture->loop);
+}
+
+static void
+assert_object_finalized (ETestServerFixture *fixture,
+                         ETestServiceType service_type)
+{
+	const gchar *message = NULL;
+	GObject *object = NULL;
+	GWeakRef *ref = NULL;
+
+	switch (service_type) {
+	case E_TEST_SERVER_NONE:
+		message = "Timed out waiting for source registery to finalize";
+		ref = &fixture->registry_ref;
+		break;
+	case E_TEST_SERVER_ADDRESS_BOOK:
+	case E_TEST_SERVER_DIRECT_ADDRESS_BOOK:
+	case E_TEST_SERVER_DEPRECATED_ADDRESS_BOOK:
+		message = "Timed out waiting for addressbook client to finalize";
+		ref = &fixture->client_ref;
+		break;
+	case E_TEST_SERVER_CALENDAR:
+	case E_TEST_SERVER_DEPRECATED_CALENDAR:
+		message = "Timed out waiting for calendar client to finalize";
+		ref = &fixture->client_ref;
+		break;
+	}
+
+	/* Give her a second chance */
+	object = g_weak_ref_get (ref);
+	if (object) {
+		guint timeout_id;
+
+		/* Add a finalize callback while we have a strong reference */
+		g_object_weak_ref (object, weak_notify_loop_quit, fixture);
+
+		/* Fail the test if we reach the timeout */
+		timeout_id = g_timeout_add_seconds (
+			FINALIZE_SECONDS,
+			object_finalize_timeout,
+			(gpointer) message);
+
+		/* We can't release the strong reference yet, it might try
+		 * to quit the main loop before we've started it.
+		 *
+		 * Instead release it in the loop (no need to track the source id,
+		 * it's guaranteed to be removed in the scope of this test).
+		 */
+		g_idle_add (object_unref_idle, object);
+
+		/* Wait for asynchronous finalization, if this loop
+		 * returns then we finalized properly, if the timeout
+		 * is reached then we've aborted with an error message
+		 */
+		g_main_loop_run (fixture->loop);
+
+		/* If we reached here, better remove the timeout source to
+		 * avoid it timing out in following tests
+		 */
+		g_source_remove (timeout_id);
+	}
+}
+
+/*****************************************************************
+ * Bootstrapping, manage work directory and create test ESource  *
+ *****************************************************************/
 typedef struct {
 	ETestServerFixture *fixture;
 	ETestServerClosure *closure;
@@ -68,6 +212,32 @@ test_installed_services (void)
 	return use_installed_services;
 }
 
+static gchar *
+generate_source_name (void)
+{
+	gchar *source_name = NULL;
+
+	if (test_installed_services ()) {
+		gchar buffer[128] = "eds-source-XXXXXX";
+		gint  fd;
+
+		fd = g_mkstemp (buffer);
+		if (fd < 0)
+			g_error ("Failed to generate source ID with temporary file");
+		close (fd);
+
+		source_name = g_strdup (buffer);
+
+	} else {
+		source_name = g_strdup_printf (
+			"%s-%d",
+			ADDRESS_BOOK_SOURCE_UID,
+			global_test_source_id++);
+	}
+
+	return source_name;
+}
+
 static void
 setup_environment (void)
 {
@@ -118,24 +288,6 @@ e_test_server_utils_bootstrap_timeout (FixturePair *pair)
 	return FALSE;
 }
 
-static void
-registry_weak_notify (gpointer data,
-                      GObject *where_the_object_was)
-{
-	ETestServerFixture *fixture = (ETestServerFixture *) data;
-
-	fixture->registry_finalized = TRUE;
-}
-
-static void
-client_weak_notify (gpointer data,
-                    GObject *where_the_object_was)
-{
-	ETestServerFixture *fixture = (ETestServerFixture *) data;
-
-	fixture->client_finalized = TRUE;
-}
-
 static void
 e_test_server_utils_client_ready (GObject *source_object,
                                   GAsyncResult *res,
@@ -152,9 +304,6 @@ e_test_server_utils_client_ready (GObject *source_object,
 		if (!pair->fixture->service.book_client)
 			g_error ("Unable to create the test book: %s", error->message);
 
-		g_object_weak_ref (
-			G_OBJECT (pair->fixture->service.book_client),
-			client_weak_notify, pair->fixture);
 		break;
 	case E_TEST_SERVER_DIRECT_ADDRESS_BOOK:
 		pair->fixture->service.book_client = (EBookClient *)
@@ -163,9 +312,6 @@ e_test_server_utils_client_ready (GObject *source_object,
 		if (!pair->fixture->service.book_client)
 			g_error ("Unable to create the test book: %s", error->message);
 
-		g_object_weak_ref (
-			G_OBJECT (pair->fixture->service.book_client),
-			client_weak_notify, pair->fixture);
 		break;
 	case E_TEST_SERVER_CALENDAR:
 		pair->fixture->service.calendar_client = (ECalClient *)
@@ -174,9 +320,6 @@ e_test_server_utils_client_ready (GObject *source_object,
 		if (!pair->fixture->service.calendar_client)
 			g_error ("Unable to create the test calendar: %s", error->message);
 
-		g_object_weak_ref (
-			G_OBJECT (pair->fixture->service.calendar_client),
-			client_weak_notify, pair->fixture);
 		break;
 	case E_TEST_SERVER_DEPRECATED_ADDRESS_BOOK:
 	case E_TEST_SERVER_DEPRECATED_CALENDAR:
@@ -184,6 +327,9 @@ e_test_server_utils_client_ready (GObject *source_object,
 		g_assert_not_reached ();
 	}
 
+	/* Track ref counts now that we have a client */
+	add_weak_ref (pair->fixture, pair->closure->type);
+
 	g_main_loop_quit (pair->fixture->loop);
 }
 
@@ -218,14 +364,9 @@ e_test_server_utils_source_added (ESourceRegistry *registry,
 					e_book_client_connect_sync (source, NULL, &error);
 		}
 
-		if (!pair->closure->use_async_connect) {
-			if (!pair->fixture->service.book_client)
-				g_error ("Unable to create the test book: %s", error->message);
-
-			g_object_weak_ref (
-				G_OBJECT (pair->fixture->service.book_client),
-				client_weak_notify, pair->fixture);
-		}
+		if (!pair->closure->use_async_connect &&
+		    !pair->fixture->service.book_client)
+			g_error ("Unable to create the test book: %s", error->message);
 
 		break;
 
@@ -239,10 +380,6 @@ e_test_server_utils_source_added (ESourceRegistry *registry,
 		if (!e_book_open (pair->fixture->service.book, FALSE, &error))
 			g_error ("Unable to open book: %s", error->message);
 
-		g_object_weak_ref (
-			G_OBJECT (pair->fixture->service.book),
-			client_weak_notify, pair->fixture);
-
 		break;
 
 	case E_TEST_SERVER_CALENDAR:
@@ -260,10 +397,6 @@ e_test_server_utils_source_added (ESourceRegistry *registry,
 					pair->closure->calendar_source_type, NULL, &error);
 			if (!pair->fixture->service.calendar_client)
 				g_error ("Unable to create the test calendar: %s", error->message);
-
-			g_object_weak_ref (
-				G_OBJECT (pair->fixture->service.calendar_client),
-				client_weak_notify, pair->fixture);
 		}
 
 		break;
@@ -278,16 +411,17 @@ e_test_server_utils_source_added (ESourceRegistry *registry,
 		if (!e_cal_open (pair->fixture->service.calendar, FALSE, &error))
 			g_error ("Unable to open calendar: %s", error->message);
 
-		g_object_weak_ref (
-			G_OBJECT (pair->fixture->service.calendar),
-			client_weak_notify, pair->fixture);
-
 		break;
 
 	case E_TEST_SERVER_NONE:
 		return;
 	}
 
+	/* Add the weak ref now if we just created it */
+	if (pair->closure->type != E_TEST_SERVER_NONE &&
+	    pair->closure->use_async_connect == FALSE)
+		add_weak_ref (pair->fixture, pair->closure->type);
+
 	if (!pair->closure->use_async_connect)
 		g_main_loop_quit (pair->fixture->loop);
 }
@@ -304,9 +438,8 @@ e_test_server_utils_bootstrap_idle (FixturePair *pair)
 	if (!pair->fixture->registry)
 		g_error ("Unable to create the test registry: %s", error->message);
 
-	g_object_weak_ref (
-		G_OBJECT (pair->fixture->registry),
-		registry_weak_notify, pair->fixture);
+	/* Add weak ref for the registry */
+	add_weak_ref (pair->fixture, E_TEST_SERVER_NONE);
 
 	g_signal_connect (
 		pair->fixture->registry, "source-added",
@@ -319,7 +452,7 @@ e_test_server_utils_bootstrap_idle (FixturePair *pair)
 	case E_TEST_SERVER_DEPRECATED_ADDRESS_BOOK:
 
 		if (!pair->fixture->source_name)
-			pair->fixture->source_name = g_strdup_printf ("%s-%d", ADDRESS_BOOK_SOURCE_UID, global_test_source_id++);
+			pair->fixture->source_name = generate_source_name ();
 
 		scratch = e_source_new_with_uid (pair->fixture->source_name, NULL, &error);
 		if (!scratch)
@@ -334,7 +467,7 @@ e_test_server_utils_bootstrap_idle (FixturePair *pair)
 	case E_TEST_SERVER_DEPRECATED_CALENDAR:
 
 		if (!pair->fixture->source_name)
-			pair->fixture->source_name = g_strdup_printf ("%s-%d", CALENDAR_SOURCE_UID, global_test_source_id++);
+			pair->fixture->source_name = generate_source_name ();
 
 		scratch = e_source_new_with_uid (pair->fixture->source_name, NULL, &error);
 		if (!scratch)
@@ -386,6 +519,9 @@ e_test_server_utils_bootstrap_idle (FixturePair *pair)
 	return FALSE;
 }
 
+/*****************************************************************
+ *                  Fixture setup and teardown                   *
+ *****************************************************************/
 /**
  * e_test_server_utils_setup:
  * @fixture: A #ETestServerFixture
@@ -398,10 +534,15 @@ e_test_server_utils_setup (ETestServerFixture *fixture,
                            gconstpointer user_data)
 {
 	ETestServerClosure *closure = (ETestServerClosure *) user_data;
-	FixturePair         pair    = { fixture, closure };
+	FixturePair         pair = { fixture, closure };
 
 	/* Create work directory */
-	g_assert (g_mkdir_with_parents (EDS_TEST_WORK_DIR, 0755) == 0);
+	if (!test_installed_services ())
+		g_assert (g_mkdir_with_parents (EDS_TEST_WORK_DIR, 0755) == 0);
+
+	/* Init refs */
+	g_weak_ref_init (&fixture->registry_ref, NULL);
+	g_weak_ref_init (&fixture->client_ref, NULL);
 
 	fixture->loop = g_main_loop_new (NULL, FALSE);
 
@@ -449,10 +590,12 @@ e_test_server_utils_teardown (ETestServerFixture *fixture,
 	ETestServerClosure *closure = (ETestServerClosure *) user_data;
 	GError             *error = NULL;
 
+	/* Try to finalize the EClient */
 	switch (closure->type) {
 	case E_TEST_SERVER_ADDRESS_BOOK:
 	case E_TEST_SERVER_DIRECT_ADDRESS_BOOK:
-		if (!closure->keep_work_directory && !e_client_remove_sync (E_CLIENT (fixture->service.book_client), NULL, &error)) {
+		if (!closure->keep_work_directory &&
+		    !e_client_remove_sync (E_CLIENT (fixture->service.book_client), NULL, &error)) {
 			g_message ("Failed to remove test book: %s (ignoring)", error->message);
 			g_clear_error (&error);
 		}
@@ -461,7 +604,8 @@ e_test_server_utils_teardown (ETestServerFixture *fixture,
 		break;
 
 	case E_TEST_SERVER_DEPRECATED_ADDRESS_BOOK:
-		if (!closure->keep_work_directory && !e_book_remove (fixture->service.book, &error)) {
+		if (!closure->keep_work_directory &&
+		    !e_book_remove (fixture->service.book, &error)) {
 			g_message ("Failed to remove test book: %s (ignoring)", error->message);
 			g_clear_error (&error);
 		}
@@ -470,7 +614,8 @@ e_test_server_utils_teardown (ETestServerFixture *fixture,
 		break;
 
 	case E_TEST_SERVER_CALENDAR:
-		if (!closure->keep_work_directory && !e_client_remove_sync (E_CLIENT (fixture->service.calendar_client), NULL, &error)) {
+		if (!closure->keep_work_directory &&
+		    !e_client_remove_sync (E_CLIENT (fixture->service.calendar_client), NULL, &error)) {
 			g_message ("Failed to remove test calendar: %s (ignoring)", error->message);
 			g_clear_error (&error);
 		}
@@ -479,7 +624,8 @@ e_test_server_utils_teardown (ETestServerFixture *fixture,
 		break;
 
 	case E_TEST_SERVER_DEPRECATED_CALENDAR:
-		if (!closure->keep_work_directory && !e_cal_remove (fixture->service.calendar, &error)) {
+		if (!closure->keep_work_directory &&
+		    !e_cal_remove (fixture->service.calendar, &error)) {
 			g_message ("Failed to remove test calendar: %s (ignoring)", error->message);
 			g_clear_error (&error);
 		}
@@ -490,20 +636,26 @@ e_test_server_utils_teardown (ETestServerFixture *fixture,
 		break;
 	}
 
-	if (closure->type != E_TEST_SERVER_NONE &&
-	    fixture->client_finalized == FALSE)
-		g_error ("Failed to destroy client while tearing down test case; reference count imbalance");
+	/* Assert that our EClient has finalized */
+	if (closure->type != E_TEST_SERVER_NONE)
+		assert_object_finalized (fixture, closure->type);
 
-	g_free (fixture->source_name);
+	/* Try to finalize the registry now */
 	g_object_run_dispose (G_OBJECT (fixture->registry));
 	g_object_unref (fixture->registry);
-	fixture->registry = NULL;
 
-	if (fixture->registry_finalized == FALSE)
-		g_error ("Failed to destroy registry while tearing down test case; reference count imbalance");
+	/* Assert that the registry finalizes */
+	assert_object_finalized (fixture, E_TEST_SERVER_NONE);
 
+	g_free (fixture->source_name);
 	g_main_loop_unref (fixture->loop);
+	fixture->registry = NULL;
 	fixture->loop = NULL;
+	fixture->service.generic = NULL;
+
+	/* Clear refs */
+	g_weak_ref_clear (&fixture->registry_ref);
+	g_weak_ref_clear (&fixture->client_ref);
 
 	if (!test_installed_services ()) {
 #if !GLOBAL_DBUS_DAEMON
@@ -551,10 +703,13 @@ e_test_server_utils_run_full (ETestServerFlags flags)
 	gint tests_ret;
 
 	/* Cleanup work directory */
-	if ((flags & E_TEST_SERVER_KEEP_WORK_DIRECTORY) == 0)
-		delete_work_directory ();
+	if (!test_installed_services ()) {
 
-	setup_environment ();
+		if ((flags & E_TEST_SERVER_KEEP_WORK_DIRECTORY) == 0)
+			delete_work_directory ();
+
+		setup_environment ();
+	}
 
 #if GLOBAL_DBUS_DAEMON
 	if (!test_installed_services ()) {
